@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-from __future__ import print_function
+
 
 """GNUmed vaccination related definitions (only).
 
@@ -33,50 +33,50 @@ _VACCINE_SUBSTANCES = {
 	# bacterial
 
 	'anthrax': {
-		'name': u'bacillus anthracis antigen',
-		'atc4target': u'J07AC01',
-		'target': _(u'J07AC01-target::anthrax'),
-		'v21_indications': [u'bacillus anthracis (Anthrax)']
+		'name': 'bacillus anthracis antigen',
+		'atc4target': 'J07AC01',
+		'target': _('J07AC01-target::anthrax'),
+		'v21_indications': ['bacillus anthracis (Anthrax)']
 	},
 
 	'brucellosis': {
-		'name': u'brucella antigen',
-		'atc4target': u'J07AD01',
-		'target': _(u'J07AD01-target::brucellosis')
+		'name': 'brucella antigen',
+		'atc4target': 'J07AD01',
+		'target': _('J07AD01-target::brucellosis')
 	},
 
 	'cholera': {
-		'name': u'cholera, inactivated',
-		'atc4target': u'J07AE0',
-		'target': _(u'J07AE0-target::cholera'),
-		'v21_indications': [u'cholera']
+		'name': 'cholera, inactivated',
+		'atc4target': 'J07AE0',
+		'target': _('J07AE0-target::cholera'),
+		'v21_indications': ['cholera']
 	},
 	'cholera-live': {
-		'name': u'cholera, live, attenuated',
-		'atc4target': u'J07AE0',
-		'target': _(u'J07AE0-target::cholera'),
-		'v21_indications_live': [u'cholera']
+		'name': 'cholera, live, attenuated',
+		'atc4target': 'J07AE0',
+		'target': _('J07AE0-target::cholera'),
+		'v21_indications_live': ['cholera']
 	},
 
 	'diphtheria': {
-		'name': u'diphtheria toxoid',
-		'atc4target': u'J07AF01',
-		'target': _(u'J07AF01-target::diphtheria'),
-		'v21_indications': [u'diphtheria']
+		'name': 'diphtheria toxoid',
+		'atc4target': 'J07AF01',
+		'target': _('J07AF01-target::diphtheria'),
+		'v21_indications': ['diphtheria']
 	},
 
 	'HiB': {
-		'name': u'hemophilus influenzae B antigen',
-		'atc4target': u'J07AG01',
-		'target': _(u'J07AG01-target::HiB'),
-		'v21_indications': [u'haemophilus influenzae b']
+		'name': 'hemophilus influenzae B antigen',
+		'atc4target': 'J07AG01',
+		'target': _('J07AG01-target::HiB'),
+		'v21_indications': ['haemophilus influenzae b']
 	},
 
 	'menA': {
-		'name': u'meningococcus A antigen',
-		'atc4target': u'J07AH01',
-		'target': _(u'J07AH01-target::meningococcus A'),
-		'v21_indications': [u'meningococcus A']
+		'name': 'meningococcus A antigen',
+		'atc4target': 'J07AH01',
+		'target': _('J07AH01-target::meningococcus A'),
+		'v21_indications': ['meningococcus A']
 	},
 #	'menA-conj': {
 #		'name': u'meningococcus A antigen, conjugated',
@@ -84,9 +84,9 @@ _VACCINE_SUBSTANCES = {
 #		'target': _(u'J07AH10-target::meningococcus A')
 #	},
 	'menBmem': {
-		'name': u'meningococcus B membrane',
-		'atc4target': u'J07AH06',
-		'target': _(u'J07AH06-target::meningococcus B')
+		'name': 'meningococcus B membrane',
+		'atc4target': 'J07AH06',
+		'target': _('J07AH06-target::meningococcus B')
 	},
 #	'menBmulti': {
 #		'name': u'meningococcus B multicomponent',
@@ -94,29 +94,29 @@ _VACCINE_SUBSTANCES = {
 #		'target': _(u'J07AH09-target::meningococcus B'),
 #	},
 	'menC': {
-		'name': u'meningococcus C antigen',
-		'atc4target': u'J07AH07',
-		'target': _(u'J07AH07-target::meningococcus C'),
-		'v21_indications': [u'meningococcus C']
+		'name': 'meningococcus C antigen',
+		'atc4target': 'J07AH07',
+		'target': _('J07AH07-target::meningococcus C'),
+		'v21_indications': ['meningococcus C']
 	},
 	'menY': {		# fake
-		'name': u'meningococcus Y antigen',
-		'atc4target': u'J07AH0Y',
-		'target': _(u'J07AH0Y-target::meningococcus Y'),
-		'v21_indications': [u'meningococcus Y']
+		'name': 'meningococcus Y antigen',
+		'atc4target': 'J07AH0Y',
+		'target': _('J07AH0Y-target::meningococcus Y'),
+		'v21_indications': ['meningococcus Y']
 	},
 	'menW': {		# fake
-		'name': u'meningococcus W-135 antigen',
-		'atc4target': u'J07AH0W',
-		'target': _(u'J07AH0W-target::meningococcus W'),
-		'v21_indications': [u'meningococcus W']
+		'name': 'meningococcus W-135 antigen',
+		'atc4target': 'J07AH0W',
+		'target': _('J07AH0W-target::meningococcus W'),
+		'v21_indications': ['meningococcus W']
 	},
 
 	'pertussis': {		# fake
-		'name': u'pertussis',
-		'atc4target': u'J07AJ0',
-		'target': _(u'J07AJ0-target::pertussis'),
-		'v21_indications': [u'pertussis']
+		'name': 'pertussis',
+		'atc4target': 'J07AJ0',
+		'target': _('J07AJ0-target::pertussis'),
+		'v21_indications': ['pertussis']
 	},
 #	'pertussis-inactivated': {
 #		'name': u'pertussis, inactivated',
@@ -130,17 +130,17 @@ _VACCINE_SUBSTANCES = {
 #	},
 
 	'plague': {
-		'name': u'yersinia pestis, inactivated',
-		'atc4target': u'J07AK01',
-		'target': _(u'J07AK01-target::plague'),
-		'v21_indications': [u'yersinia pestis']
+		'name': 'yersinia pestis, inactivated',
+		'atc4target': 'J07AK01',
+		'target': _('J07AK01-target::plague'),
+		'v21_indications': ['yersinia pestis']
 	},
 
 	'pneumococcus': {
-		'name': u'pneumococcus antigen',
-		'atc4target': u'J07AL0',
-		'target': _(u'J07AL0-target::pneumococcus'),
-		'v21_indications': [u'pneumococcus']
+		'name': 'pneumococcus antigen',
+		'atc4target': 'J07AL0',
+		'target': _('J07AL0-target::pneumococcus'),
+		'v21_indications': ['pneumococcus']
 	},
 #	'pneumococcus-conjugated': {
 #		'name': u'pneumococcus antigen, conjugated',
@@ -149,29 +149,29 @@ _VACCINE_SUBSTANCES = {
 #	},
 
 	'tetanus': {
-		'name': u'tetanus toxoid',
-		'atc4target': u'J07AM01',
-		'target': _(u'J07AM01-target::tetanus'),
-		'v21_indications': [u'tetanus']
+		'name': 'tetanus toxoid',
+		'atc4target': 'J07AM01',
+		'target': _('J07AM01-target::tetanus'),
+		'v21_indications': ['tetanus']
 	},
 
 	'tbc-live': {
-		'name': u'tuberculosis, live, attenuated',
-		'atc4target': u'J07AN01',
-		'target': _(u'J07AN01-target::tbc'),
-		'v21_indications_live': [u'tuberculosis']
+		'name': 'tuberculosis, live, attenuated',
+		'atc4target': 'J07AN01',
+		'target': _('J07AN01-target::tbc'),
+		'v21_indications_live': ['tuberculosis']
 	},
 
 	'salmo-live': {
-		'name': u'salmonella typhi, live, attenuated',
-		'atc4target': u'J07AP0',
-		'target': _(u'J07AP0-target::typhoid'),
-		'v21_indications_live': [u'salmonella typhi (typhoid)']
+		'name': 'salmonella typhi, live, attenuated',
+		'atc4target': 'J07AP0',
+		'target': _('J07AP0-target::typhoid'),
+		'v21_indications_live': ['salmonella typhi (typhoid)']
 	},
 	'salmo-inact': {
-		'name': u'salmonella typhi, inactivated',
-		'atc4target': u'J07AP0',
-		'target': _(u'J07AP0-target::typhoid')
+		'name': 'salmonella typhi, inactivated',
+		'atc4target': 'J07AP0',
+		'target': _('J07AP0-target::typhoid')
 	},
 #	'salmo-antigen': {
 #		'name': u'salmonella typhi antigen',
@@ -179,52 +179,52 @@ _VACCINE_SUBSTANCES = {
 #		'target': _(u'J07AP03-target::typhoid')
 #	},
 	'salmo-typh+ent': {
-		'name': u'salmonella typhi, enterica',
-		'atc4target': u'J07AP1',
-		'target': _(u'J07AP1-target::typhoid, paratyphus'),
-		'v21_indications': [u'salmonella typhi (typhoid)']
+		'name': 'salmonella typhi, enterica',
+		'atc4target': 'J07AP1',
+		'target': _('J07AP1-target::typhoid, paratyphus'),
+		'v21_indications': ['salmonella typhi (typhoid)']
 	},
 
 	'typh-exanth': {
-		'name': u'rickettsia prowazekii, inactivated',
-		'atc4target': u'J07AR01',
-		'target': _(u'J07AR01-target::typhus exanthematicus')
+		'name': 'rickettsia prowazekii, inactivated',
+		'atc4target': 'J07AR01',
+		'target': _('J07AR01-target::typhus exanthematicus')
 	},
 
 	'qfever': {		# fake
-		'name': u'coxiella burnetii',
-		'atc4target': u'J07AXQF',
-		'target': _(u'J07AXQF-target::Q fever'),
-		'v21_indications': [u'coxiella burnetii (Q fever)']
+		'name': 'coxiella burnetii',
+		'atc4target': 'J07AXQF',
+		'target': _('J07AXQF-target::Q fever'),
+		'v21_indications': ['coxiella burnetii (Q fever)']
 	},
 
 	# viral
 
 	'fsme': {
-		'name': u'flavivirus, tick-borne',
-		'atc4target': u'J07BA01',
-		'target': _(u'J07BA01-target::tick-borne encephalitis'),
-		'v21_indications': [u'tick-borne meningoencephalitis']
+		'name': 'flavivirus, tick-borne',
+		'atc4target': 'J07BA01',
+		'target': _('J07BA01-target::tick-borne encephalitis'),
+		'v21_indications': ['tick-borne meningoencephalitis']
 	},
 
 	'japEnc': {
-		'name': u'flavivirus, japanese',
-		'atc4target': u'J07BA0J',		# fake
-		'target': _(u'J07BA0J-target::japanese encephalitis'),
-		'v21_indications': [u'japanese B encephalitis']
+		'name': 'flavivirus, japanese',
+		'atc4target': 'J07BA0J',		# fake
+		'target': _('J07BA0J-target::japanese encephalitis'),
+		'v21_indications': ['japanese B encephalitis']
 	},
 	'japEnc-live': {
-		'name': u'flavivirus, japanese, live, attenuated',
-		'atc4target': u'J07BA0J',		# fake
-		'target': _(u'J07BA0J-target::japanese encephalitis'),
-		'v21_indications_live': [u'japanese B encephalitis']
+		'name': 'flavivirus, japanese, live, attenuated',
+		'atc4target': 'J07BA0J',		# fake
+		'target': _('J07BA0J-target::japanese encephalitis'),
+		'v21_indications_live': ['japanese B encephalitis']
 	},
 
 	'influ-inact': {
-		'name': u'influenza, inactivated',
-		'atc4target': u'J07BB0',
-		'target': _(u'J07BB0-target::influenza'),
-		'v21_indications': [u'influenza (seasonal)', u'influenza (H3N2)', u'influenza (H1N1)']
+		'name': 'influenza, inactivated',
+		'atc4target': 'J07BB0',
+		'target': _('J07BB0-target::influenza'),
+		'v21_indications': ['influenza (seasonal)', 'influenza (H3N2)', 'influenza (H1N1)']
 	},
 #	'influ-inact-surf': {
 #		'name': u'influenza, inactivated, surface',
@@ -232,24 +232,24 @@ _VACCINE_SUBSTANCES = {
 #		'target': _(u'J07BB0-target::influenza')
 #	},
 	'influ-live': {
-		'name': u'influenza, live, attenuated',
-		'atc4target': u'J07BB0',
-		'target': _(u'J07BB0-target::influenza'),
-		'v21_indications_live': [u'influenza (seasonal)', u'influenza (H3N2)', u'influenza (H1N1)']
+		'name': 'influenza, live, attenuated',
+		'atc4target': 'J07BB0',
+		'target': _('J07BB0-target::influenza'),
+		'v21_indications_live': ['influenza (seasonal)', 'influenza (H3N2)', 'influenza (H1N1)']
 	},
 
 	'hepB': {
-		'name': u'hepatitis B antigen',
-		'atc4target': u'J07BC01',
-		'target': _(u'J07BC01-target::hepatitis B'),
-		'v21_indications': [u'hepatitis B']
+		'name': 'hepatitis B antigen',
+		'atc4target': 'J07BC01',
+		'target': _('J07BC01-target::hepatitis B'),
+		'v21_indications': ['hepatitis B']
 	},
 
 	'hepA-inact': {
-		'name': u'hepatitis A, inactivated',
-		'atc4target': u'J07BC0',	# 02
-		'target': _(u'J07BC0-target::hepatitis A'),
-		'v21_indications': [u'hepatitis A']
+		'name': 'hepatitis A, inactivated',
+		'atc4target': 'J07BC0',	# 02
+		'target': _('J07BC0-target::hepatitis A'),
+		'v21_indications': ['hepatitis A']
 	},
 #	'hepA-antig': {
 #		'name': u'hepatitis A antigen',
@@ -258,43 +258,43 @@ _VACCINE_SUBSTANCES = {
 #	},
 
 	'measles-live': {
-		'name': u'measles, live, attenuated',
-		'atc4target': u'J07BD01',
-		'target': _(u'J07BD01-target::measles'),
-		'v21_indications_live': [u'measles']
+		'name': 'measles, live, attenuated',
+		'atc4target': 'J07BD01',
+		'target': _('J07BD01-target::measles'),
+		'v21_indications_live': ['measles']
 	},
 
 	'mumps-live': {
-		'name': u'mumps, live, attenuated',
-		'atc4target': u'J07BE01',
-		'target': _(u'J07BE01-target::mumps'),
-		'v21_indications_live': [u'mumps']
+		'name': 'mumps, live, attenuated',
+		'atc4target': 'J07BE01',
+		'target': _('J07BE01-target::mumps'),
+		'v21_indications_live': ['mumps']
 	},
 
 	'polio-live': {
-		'name': u'poliomyelitis, live, attenuated',
-		'atc4target': u'J07BF0',	# 01
-		'target': _(u'J07BF0-target::poliomyelitis'),
-		'v21_indications_live': [u'poliomyelitis']
+		'name': 'poliomyelitis, live, attenuated',
+		'atc4target': 'J07BF0',	# 01
+		'target': _('J07BF0-target::poliomyelitis'),
+		'v21_indications_live': ['poliomyelitis']
 	},
 	'polio-inact': {
-		'name': u'poliomyelitis, inactivated',
-		'atc4target': u'J07BF0',
-		'target': _(u'J07BF0-target::poliomyelitis'),
-		'v21_indications': [u'poliomyelitis']
+		'name': 'poliomyelitis, inactivated',
+		'atc4target': 'J07BF0',
+		'target': _('J07BF0-target::poliomyelitis'),
+		'v21_indications': ['poliomyelitis']
 	},
 
 	'rabies': {
-		'name': u'rabies, inactivated',
-		'atc4target': u'J07BG01',
-		'target': _(u'J07BG01-target::rabies'),
-		'v21_indications': [u'rabies']
+		'name': 'rabies, inactivated',
+		'atc4target': 'J07BG01',
+		'target': _('J07BG01-target::rabies'),
+		'v21_indications': ['rabies']
 	},
 
 	'rota-live-atten': {
-		'name': u'rotavirus, live, attenuated',
-		'atc4target': u'J07BH0',		# 01
-		'target': _(u'J07BH0-target::rotavirus diarrhea')
+		'name': 'rotavirus, live, attenuated',
+		'atc4target': 'J07BH0',		# 01
+		'target': _('J07BH0-target::rotavirus diarrhea')
 	},
 #	'rota-live': {
 #		'name': u'rotavirus, live',
@@ -304,58 +304,58 @@ _VACCINE_SUBSTANCES = {
 #	},
 
 	'rubella-live': {
-		'name': u'rubella, live',
-		'atc4target': u'J07BJ01',
-		'target': _(u'J07BJ01-target::rubella'),
-		'v21_indications_live': [u'rubella']
+		'name': 'rubella, live',
+		'atc4target': 'J07BJ01',
+		'target': _('J07BJ01-target::rubella'),
+		'v21_indications_live': ['rubella']
 	},
 
 	'chickenpox-live': {
-		'name': u'herpes virus (chickenpox), live',
-		'atc4target': u'J07BK0',	# 01
-		'target': _(u'J07BK0-target::varicella (chickenpox)'),
-		'v21_indications_live': [u'varicella (chickenpox, shingles)']
+		'name': 'herpes virus (chickenpox), live',
+		'atc4target': 'J07BK0',	# 01
+		'target': _('J07BK0-target::varicella (chickenpox)'),
+		'v21_indications_live': ['varicella (chickenpox, shingles)']
 	},
 	'shingles-live': {
-		'name': u'herpes virus (shingles), live',
-		'atc4target': u'J07BK0',	# 02
-		'target': _(u'J07BK0-target::zoster (shingles)')
+		'name': 'herpes virus (shingles), live',
+		'atc4target': 'J07BK0',	# 02
+		'target': _('J07BK0-target::zoster (shingles)')
 	},
 
 	'yellow_fever-live': {
-		'name': u'yellow fever virus, live',
-		'atc4target': u'J07BL01',
-		'target': _(u'J07BL01-target::yellow fever'),
-		'v21_indications_live': [u'yellow fever']
+		'name': 'yellow fever virus, live',
+		'atc4target': 'J07BL01',
+		'target': _('J07BL01-target::yellow fever'),
+		'v21_indications_live': ['yellow fever']
 	},
 
 	'pap-generic': {		# fake
-		'name': u'papillomavirus',
-		'atc4target': u'J07BM0',
-		'target': _(u'J07BM0-target::HPV'),
-		'v21_indications': [u'human papillomavirus']
+		'name': 'papillomavirus',
+		'atc4target': 'J07BM0',
+		'target': _('J07BM0-target::HPV'),
+		'v21_indications': ['human papillomavirus']
 	},
 	'pap6-11-16-18': {
-		'name': u'papillomavirus 6,11,16,18',
-		'atc4target': u'J07BM01',
-		'target': _(u'J07BM01-target::HPV')
+		'name': 'papillomavirus 6,11,16,18',
+		'atc4target': 'J07BM01',
+		'target': _('J07BM01-target::HPV')
 	},
 	'pap16-18': {
-		'name': u'papillomavirus 16,18',
-		'atc4target': u'J07BM02',
-		'target': _(u'J07BM02-target::HPV')
+		'name': 'papillomavirus 16,18',
+		'atc4target': 'J07BM02',
+		'target': _('J07BM02-target::HPV')
 	},
 	'pap6-11-16-18-31-33-45-52-58': {
-		'name': u'papillomavirus 6,11,16,18,31,33,45,52,58',
-		'atc4target': u'J07BM03',
-		'target': _(u'J07BM03-target::HPV')
+		'name': 'papillomavirus 6,11,16,18,31,33,45,52,58',
+		'atc4target': 'J07BM03',
+		'target': _('J07BM03-target::HPV')
 	},
 
 	'smallpox': {
-		'name': u'variola virus, live',
-		'atc4target': u'J07BX01',
-		'target': _(u'J07BX01-target::variola (smallpox)'),
-		'v21_indications_live': [u'variola virus (smallpox)']
+		'name': 'variola virus, live',
+		'atc4target': 'J07BX01',
+		'target': _('J07BX01-target::variola (smallpox)'),
+		'v21_indications_live': ['variola virus (smallpox)']
 	}
 
 }
@@ -367,132 +367,132 @@ _GENERIC_VACCINES = {
 	# J07A - antibacterial vaccines
 
 	'anthrax': {
-		'name': _(u'generic anthrax vaccine'),
-		'atc': u'J07AC',
+		'name': _('generic anthrax vaccine'),
+		'atc': 'J07AC',
 		'live': False,
 		'ingredients': ['anthrax']
 	},
 
 	'brucellosis': {
-		'name': _(u'generic brucellosis vaccine'),
-		'atc': u'J07AD',
+		'name': _('generic brucellosis vaccine'),
+		'atc': 'J07AD',
 		'live': False,
 		'ingredients': ['brucellosis']
 	},
 
 	'cholera': {
-		'name': _(u'generic cholera vaccine'),
-		'atc': u'J07AE01',
+		'name': _('generic cholera vaccine'),
+		'atc': 'J07AE01',
 		'live': False,
 		'ingredients': ['cholera']
 	},
 	'cholera, live': {
-		'name': _(u'generic cholera vaccine, live'),
-		'atc': u'J07AE02',
+		'name': _('generic cholera vaccine, live'),
+		'atc': 'J07AE02',
 		'live': True,
 		'ingredients': ['cholera-live']
 	},
 
 	'diphtheria': {
-		'name': _(u'generic diphtheria vaccine'),
-		'atc': u'J07AF',
+		'name': _('generic diphtheria vaccine'),
+		'atc': 'J07AF',
 		'live': False,
 		'ingredients': ['diphtheria']
 	},
 
 	'HiB': {
-		'name': _(u'generic HiB vaccine'),
-		'atc': u'J07AG',
+		'name': _('generic HiB vaccine'),
+		'atc': 'J07AG',
 		'live': False,
 		'ingredients': ['HiB']
 	},
 
 	'meningococcus A': {
-		'name': _(u'generic meningococcus A vaccine'),
-		'atc': u'J07AH01',
+		'name': _('generic meningococcus A vaccine'),
+		'atc': 'J07AH01',
 		'live': False,
 		# for generic vaccine do not differentiate between conjugated or not
 		'ingredients': ['menA']
 	},
 	'meningococcus B': {
-		'name': _(u'generic meningococcus B vaccine'),
-		'atc': u'J07AH06',
+		'name': _('generic meningococcus B vaccine'),
+		'atc': 'J07AH06',
 		'live': False,
 		# do not differentiate membrane vs multicomponent in generic vaccine
 		'ingredients': ['menBmem']
 	},
 	'meningococcus C': {
-		'name': _(u'generic meningococcus C vaccine'),
-		'atc': u'J07AH',
+		'name': _('generic meningococcus C vaccine'),
+		'atc': 'J07AH',
 		'live': False,
 		'ingredients': ['menC']
 	},
 
 	'pertussis': {
-		'name': _(u'generic pertussis vaccine'),
-		'atc': u'J07AJ01',
+		'name': _('generic pertussis vaccine'),
+		'atc': 'J07AJ01',
 		'live': False,
 		'ingredients': ['pertussis']
 	},
 
 	'plague': {
-		'name': _(u'generic plague vaccine'),
-		'atc': u'J07AK',
+		'name': _('generic plague vaccine'),
+		'atc': 'J07AK',
 		'live': False,
 		'ingredients': ['plague']
 	},
 
 	'pneumococcus': {
-		'name': _(u'generic pneumococcus vaccine'),
-		'atc': u'J07AL0',
+		'name': _('generic pneumococcus vaccine'),
+		'atc': 'J07AL0',
 		'live': False,
 		# for generic vaccine do not differentiate between conjugated or not
 		'ingredients': ['pneumococcus']
 	},
 
 	'tetanus': {
-		'name': _(u'generic tetanus vaccine'),
-		'atc': u'J07AM',
+		'name': _('generic tetanus vaccine'),
+		'atc': 'J07AM',
 		'live': False,
 		'ingredients': ['tetanus']
 	},
 
 	'Tbc': {
-		'name': _(u'generic Tbc vaccine'),
-		'atc': u'J07AN',
+		'name': _('generic Tbc vaccine'),
+		'atc': 'J07AN',
 		'live': True,
 		'ingredients': ['tbc-live']
 	},
 
 	'typhoid': {
-		'name': _(u'generic typhoid vaccine'),
-		'atc': u'J07AP02',
+		'name': _('generic typhoid vaccine'),
+		'atc': 'J07AP02',
 		'live': False,
 		'ingredients': ['salmo-inact']
 	},
 	'typhoid, live': {
-		'name': _(u'generic typhoid vaccine, live'),
-		'atc': u'J07AP01',
+		'name': _('generic typhoid vaccine, live'),
+		'atc': 'J07AP01',
 		'live': True,
 		'ingredients': ['salmo-live']
 	},
 	'typhoid, paratyphus': {
-		'name': _(u'generic typhoid/paratyphus vaccine'),
-		'atc': u'J07AP10',
+		'name': _('generic typhoid/paratyphus vaccine'),
+		'atc': 'J07AP10',
 		'live': False,
 		'ingredients': ['salmo-typh+ent']
 	},
 
 	'typhus exanthematicus': {
-		'name': _(u'generic typhus exanthematicus vaccine'),
-		'atc': u'J07AR',
+		'name': _('generic typhus exanthematicus vaccine'),
+		'atc': 'J07AR',
 		'live': False,
 		'ingredients': ['typh-exanth']
 	},
 
 	'q fever': {
-		'name': _(u'generic Q fever vaccine'),
-		'atc': u'J07AXQF',
+		'name': _('generic Q fever vaccine'),
+		'atc': 'J07AXQF',
 		'live': False,
 		'ingredients': ['qfever']
 	},
@@ -500,131 +500,131 @@ _GENERIC_VACCINES = {
 	# J07B - antiviral vaccines
 
 	'tick-borne encephalitis': {
-		'name': _(u'generic tick-borne encephalitis vaccine'),
-		'atc': u'J07BA',
+		'name': _('generic tick-borne encephalitis vaccine'),
+		'atc': 'J07BA',
 		'live': False,
 		'ingredients': ['fsme']
 	},
 
 	'japanese encephalitis': {
-		'name': _(u'generic japanese encephalitis vaccine'),
-		'atc': u'J07BA02',
+		'name': _('generic japanese encephalitis vaccine'),
+		'atc': 'J07BA02',
 		'live': False,
 		'ingredients': ['japEnc']
 	},
 	'japanese encephalitis, live': {
-		'name': _(u'generic japanese encephalitis vaccine, live'),
-		'atc': u'J07BA03',
+		'name': _('generic japanese encephalitis vaccine, live'),
+		'atc': 'J07BA03',
 		'live': True,
 		'ingredients': ['japEnc-live']
 	},
 
 	'influenza': {
-		'name': _(u'generic influenza vaccine'),
-		'atc': u'J07BB01',
+		'name': _('generic influenza vaccine'),
+		'atc': 'J07BB01',
 		'live': False,
 		'ingredients': ['influ-inact']
 	},
 	'influenza, live': {
-		'name': _(u'generic influenza vaccine, live'),
-		'atc': u'J07BB03',
+		'name': _('generic influenza vaccine, live'),
+		'atc': 'J07BB03',
 		'live': True,
 		'ingredients': ['influ-live']
 	},
 
 	'hepatitis A': {
-		'name': _(u'generic hepatitis A vaccine'),
-		'atc': u'J07BC02',
+		'name': _('generic hepatitis A vaccine'),
+		'atc': 'J07BC02',
 		'live': False,
 		'ingredients': ['hepA-inact']
 	},
 
 	'hepatitis B': {
-		'name': _(u'generic hepatitis B vaccine'),
-		'atc': u'J07BC01',
+		'name': _('generic hepatitis B vaccine'),
+		'atc': 'J07BC01',
 		'live': False,
 		'ingredients': ['hepB']
 	},
 
 	'measles-live': {
-		'name': _(u'generic measles vaccine, live'),
-		'atc': u'J07BD',
+		'name': _('generic measles vaccine, live'),
+		'atc': 'J07BD',
 		'live': True,
 		'ingredients': ['measles-live']
 	},
 
 	'mumps-live': {
-		'name': _(u'generic mumps vaccine, live'),
-		'atc': u'J07BE',
+		'name': _('generic mumps vaccine, live'),
+		'atc': 'J07BE',
 		'live': True,
 		'ingredients': ['mumps-live']
 	},
 
 	'poliomyelitis': {
-		'name': _(u'generic poliomyelitis vaccine'),
-		'atc': u'J07BF03',
+		'name': _('generic poliomyelitis vaccine'),
+		'atc': 'J07BF03',
 		'live': False,
 		'ingredients': ['polio-inact']
 	},
 
 	'poliomyelitis, live': {
-		'name': _(u'generic poliomyelitis vaccine, live'),
-		'atc': u'J07BF01',
+		'name': _('generic poliomyelitis vaccine, live'),
+		'atc': 'J07BF01',
 		'live': True,
 		'ingredients': ['polio-live']
 	},
 
 	'rabies': {
-		'name': _(u'generic rabies vaccine'),
-		'atc': u'J07BG',
+		'name': _('generic rabies vaccine'),
+		'atc': 'J07BG',
 		'live': False,
 		'ingredients': ['rabies']
 	},
 
 	'rotavirus, live': {
-		'name': _(u'generic rotavirus vaccine, live'),
-		'atc': u'J07BH01',
+		'name': _('generic rotavirus vaccine, live'),
+		'atc': 'J07BH01',
 		'live': True,
 		'ingredients': ['rota-live-atten']
 	},
 
 	'rubella': {
-		'name': _(u'generic rubella vaccine, live'),
-		'atc': u'J07BJ',
+		'name': _('generic rubella vaccine, live'),
+		'atc': 'J07BJ',
 		'live': True,
 		'ingredients': ['rubella-live']
 	},
 
 	'varicella': {
-		'name': _(u'generic varicella vaccine, live'),
-		'atc': u'J07BK01',
+		'name': _('generic varicella vaccine, live'),
+		'atc': 'J07BK01',
 		'live': True,
 		'ingredients': ['chickenpox-live']
 	},
 	'zoster': {
-		'name': _(u'generic zoster vaccine, live'),
-		'atc': u'J07BK02',
+		'name': _('generic zoster vaccine, live'),
+		'atc': 'J07BK02',
 		'live': True,
 		'ingredients': ['shingles-live']
 	},
 
 	'yellow fever': {
-		'name': _(u'generic yellow fever vaccine'),
-		'atc': u'J07BL',
+		'name': _('generic yellow fever vaccine'),
+		'atc': 'J07BL',
 		'live': True,
 		'ingredients': ['yellow_fever-live']
 	},
 
 	'HPV': {
-		'name': _(u'generic HPV vaccine'),
-		'atc': u'J07BM',
+		'name': _('generic HPV vaccine'),
+		'atc': 'J07BM',
 		'live': False,
 		'ingredients': ['pap-generic']
 	},
 
 	'variola': {
-		'name': _(u'generic variola vaccine, live'),
-		'atc': u'J07BX01',
+		'name': _('generic variola vaccine, live'),
+		'atc': 'J07BX01',
 		'live': True,
 		'ingredients': ['smallpox']
 	},
@@ -632,70 +632,70 @@ _GENERIC_VACCINES = {
 	# combinations
 
 	'meningococcus AC': {
-		'name': _(u'generic meningococcus AC vaccine'),
-		'atc': u'J07AH03',
+		'name': _('generic meningococcus AC vaccine'),
+		'atc': 'J07AH03',
 		'live': False,
 		'ingredients': ['menA', 'menC']
 	},
 	'meningococcus ACYW135': {
-		'name': _(u'generic meningococcus ACYW135 vaccine'),
-		'atc': u'J07AH04', # 4/8
+		'name': _('generic meningococcus ACYW135 vaccine'),
+		'atc': 'J07AH04', # 4/8
 		'live': False,
 		'ingredients': ['menA', 'menC', 'menY', 'menW']
 	},
 
 	'measles, mumps': {
-		'name': _(u'generic measles-mumps vaccine, live'),
-		'atc': u'J07BD51',
+		'name': _('generic measles-mumps vaccine, live'),
+		'atc': 'J07BD51',
 		'live': True,
 		'ingredients': ['measles-live', 'mumps-live']
 	},
 
 	'measles, mumps, rubella': {
-		'name': _(u'generic MMR vaccine, live'),
-		'atc': u'J07BD52',
+		'name': _('generic MMR vaccine, live'),
+		'atc': 'J07BD52',
 		'live': True,
 		'ingredients': ['measles-live', 'mumps-live', 'rubella-live']
 	},
 
 	'measles, mumps, rubella, varicella': {
-		'name': _(u'generic MMRV vaccine, live'),
-		'atc': u'J07BD54',
+		'name': _('generic MMRV vaccine, live'),
+		'atc': 'J07BD54',
 		'live': True,
 		'ingredients': ['measles-live', 'mumps-live', 'rubella-live', 'chickenpox-live']
 	},
 
 	'measles, rubella': {
-		'name': _(u'generic measles-rubella vaccine, live'),
-		'atc': u'J07BD53',
+		'name': _('generic measles-rubella vaccine, live'),
+		'atc': 'J07BD53',
 		'live': True,
 		'ingredients': ['measles-live', 'rubella-live']
 	},
 
 	'mumps, rubella': {
-		'name': _(u'generic mumps-rubella vaccine, live'),
-		'atc': u'J07BJ51',
+		'name': _('generic mumps-rubella vaccine, live'),
+		'atc': 'J07BJ51',
 		'live': True,
 		'ingredients': ['mumps-live', 'rubella-live']
 	},
 
 	'cholera, typhoid': {
-		'name': _(u'generic cholera-typhoid vaccine'),
-		'atc': u'J07AE51',
+		'name': _('generic cholera-typhoid vaccine'),
+		'atc': 'J07AE51',
 		'live': False,
 		'ingredients': ['cholera', 'salmo-inact']
 	},
 
 	'typhoid, hepatitis A': {
-		'name': _(u'generic typhoid-hepA vaccine'),
-		'atc': u'J07CA10',
+		'name': _('generic typhoid-hepA vaccine'),
+		'atc': 'J07CA10',
 		'live': False,
 		'ingredients': ['salmo-inact', 'hepA-inact']
 	},
 
 	'tetanus, diphtheria': {
-		'name': _(u'generic Td vaccine'),
-		'atc': u'J07AM51',
+		'name': _('generic Td vaccine'),
+		'atc': 'J07AM51',
 		'live': False,
 		'ingredients': ['tetanus', 'diphtheria']
 	},
@@ -708,84 +708,84 @@ _GENERIC_VACCINES = {
 	#J07AM52,,"Tetanus-Toxoid, Kombinationen mit Tetanus-Immunglobulin",,,,,,
 
 	'tetanus, diphtheria, poliomyelitis': {
-		'name': _(u'generic TdPol vaccine'),
-		'atc': u'J07CA01',
+		'name': _('generic TdPol vaccine'),
+		'atc': 'J07CA01',
 		'live': False,
 		'ingredients': ['tetanus', 'diphtheria', 'polio-inact']
 	},
 	'tetanus, diphtheria, poliomyelitis': {
-		'name': _(u'generic DTPol vaccine'),
-		'atc': u'J07CA01',
+		'name': _('generic DTPol vaccine'),
+		'atc': 'J07CA01',
 		'live': False,
 		'ingredients': ['tetanus', 'diphtheria', 'polio-inact']
 	},
 
 	'tetanus, diphtheria, HepB': {
-		'name': _(u'generic Td-HepB vaccine'),
-		'atc': u'J07CA07',
+		'name': _('generic Td-HepB vaccine'),
+		'atc': 'J07CA07',
 		'live': False,
 		'ingredients': ['tetanus', 'diphtheria', 'hepB']
 	},
 
 	'tetanus, diphtheria, rubella': {
-		'name': _(u'generic Td-rubella vaccine'),
-		'atc': u'J07CA03',
+		'name': _('generic Td-rubella vaccine'),
+		'atc': 'J07CA03',
 		'live': True,
 		'ingredients': ['tetanus', 'diphtheria', 'rubella-live']
 	},
 
 	'tetanus, diphtheria, pertussis, poliomyelitis': {
-		'name': _(u'generic TdaPPol vaccine'),
-		'atc': u'J07CA02',
+		'name': _('generic TdaPPol vaccine'),
+		'atc': 'J07CA02',
 		'live': False,
 		'ingredients': ['tetanus', 'diphtheria', 'pertussis', 'polio-inact']
 	},
 
 	'tetanus, diphtheria, pertussis, HepB': {
-		'name': _(u'generic TdaP-HepB vaccine'),
-		'atc': u'J07CA05',
+		'name': _('generic TdaP-HepB vaccine'),
+		'atc': 'J07CA05',
 		'live': False,
 		'ingredients': ['tetanus', 'diphtheria', 'pertussis', 'hepB']
 	},
 
 	'tetanus, diphtheria, pertussis': {
-		'name': _(u'generic TdaP vaccine'),
-		'atc': u'J07CA',
+		'name': _('generic TdaP vaccine'),
+		'atc': 'J07CA',
 		'live': False,
 		'ingredients': ['tetanus', 'diphtheria', 'pertussis']
 	},
 
 	'tetanus, diphtheria, pertussis, poliomyelitis, HiB': {
-		'name': _(u'generic DTaPPol-Hib vaccine'),
-		'atc': u'J07CA06',
+		'name': _('generic DTaPPol-Hib vaccine'),
+		'atc': 'J07CA06',
 		'live': False,
 		'ingredients': ['tetanus', 'diphtheria', 'pertussis', 'polio-inact', 'HiB']
 	},
 
 	'tetanus, diphtheria, pertussis, HiB, HepB': {
-		'name': _(u'generic TdaP-Hib-HepB vaccine'),
-		'atc': u'J07CA11',
+		'name': _('generic TdaP-Hib-HepB vaccine'),
+		'atc': 'J07CA11',
 		'live': False,
 		'ingredients': ['tetanus', 'diphtheria', 'pertussis', 'HiB', 'hepB']
 	},
 
 	'tetanus, diphtheria, pertussis, poliomyelitis, HepB': {
-		'name': _(u'generic TdaPPol-HepB vaccine'),
-		'atc': u'J07CA12',
+		'name': _('generic TdaPPol-HepB vaccine'),
+		'atc': 'J07CA12',
 		'live': False,
 		'ingredients': ['tetanus', 'diphtheria', 'pertussis', 'polio-inact', 'hepB']
 	},
 
 	'tetanus, diphtheria, pertussis, poliomyelitis, HiB, HepB': {
-		'name': _(u'generic TdaPPol-HiB-HepB vaccine'),
-		'atc': u'J07CA09',
+		'name': _('generic TdaPPol-HiB-HepB vaccine'),
+		'atc': 'J07CA09',
 		'live': False,
 		'ingredients': ['tetanus', 'diphtheria', 'pertussis', 'polio-inact', 'HiB', 'hepB']
 	},
 
 	'tetanus, diphtheria, pertussis, poliomyelitis, HiB, HepB, MenAC': {
-		'name': _(u'generic TdaPPol-HiB-HepB-MenAC vaccine'),
-		'atc': u'J07CA13',
+		'name': _('generic TdaPPol-HiB-HepB-MenAC vaccine'),
+		'atc': 'J07CA13',
 		'live': False,
 		'ingredients': ['tetanus', 'diphtheria', 'pertussis', 'polio-inact', 'HiB', 'hepB', 'menA', 'menC']
 	}
@@ -793,7 +793,7 @@ _GENERIC_VACCINES = {
 }
 
 #------------------------------------------------------------
-u"""
+"""
 # single
 
 J07AX  ,,Andere bakterielle Impfstoffe,,,,,,
@@ -841,7 +841,7 @@ if __name__ == '__main__':
 	if len(sys.argv) < 2:
 		sys.exit()
 
-	if sys.argv[1] != u'test':
+	if sys.argv[1] != 'test':
 		sys.exit()
 
 
@@ -856,12 +856,12 @@ if __name__ == '__main__':
 	def print_vaccs():
 		for key in _GENERIC_VACCINES:
 			vacc = _GENERIC_VACCINES[key]
-			print(u'vaccine "%s" (ATC %s)' % (vacc['name'], vacc['atc']))
+			print('vaccine "%s" (ATC %s)' % (vacc['name'], vacc['atc']))
 			for key in vacc['ingredients']:
 				subst = _VACCINE_SUBSTANCES[key]
 				atc = subst['atc4target']
 				print(' contains: %s (ATC %s)' % (subst['name'], atc))
-				print('  protects against: "%s" [%s]' % (_(subst['target']).lstrip(u'%s-target::' % atc), subst['target']))
+				print('  protects against: "%s" [%s]' % (_(subst['target']).lstrip('%s-target::' % atc), subst['target']))
 
 
 	print_vaccs()
