@@ -1,4 +1,4 @@
-"""GNUmed database backend listener.
+__doc__ = """GNUmed database backend listener.
 
 This module implements threaded listening for asynchronuous
 notifications from the database backend.
@@ -24,9 +24,9 @@ _log = logging.getLogger('gm.db')
 
 
 signals2listen4 = [
-	u'db_maintenance_warning',		# warns of impending maintenance and asks for disconnect
-	u'db_maintenance_disconnect',	# announces a forced disconnect and disconnects
-	u'gm_table_mod'					# sent for any (registered) table modification, payload contains details
+	'db_maintenance_warning',		# warns of impending maintenance and asks for disconnect
+	'db_maintenance_disconnect',	# announces a forced disconnect and disconnects
+	'gm_table_mod'					# sent for any (registered) table modification, payload contains details
 ]
 
 #=====================================================================
@@ -37,9 +37,14 @@ class gmBackendListener(gmBorg.cBorg):
 		try:
 			self.already_inited
 			return
+
 		except AttributeError:
 			pass
 
+		self.debug = False
+		self.__notifications_received = 0
+		self.__messages_sent = 0
+
 		_log.info('starting backend notifications listener thread')
 
 		# the listener thread will regularly try to acquire
@@ -53,7 +58,7 @@ class gmBackendListener(gmBorg.cBorg):
 
 		self._conn = conn
 		self.backend_pid = self._conn.get_backend_pid()
-		_log.debug('connection has backend PID [%s]', self.backend_pid)
+		_log.debug('notification listener connection has backend PID [%s]', self.backend_pid)
 		self._conn.set_isolation_level(0)		# autocommit mode = psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT
 		self._cursor = self._conn.cursor()
 		try:
@@ -70,10 +75,14 @@ class gmBackendListener(gmBorg.cBorg):
 		self.__start_thread()
 
 		self.already_inited = True
+
 	#-------------------------------
 	# public API
 	#-------------------------------
 	def shutdown(self):
+		_log.debug('received %s notifications', self.__notifications_received)
+		_log.debug('sent %s messages', self.__messages_sent)
+
 		if self._listener_thread is None:
 			self.__shutdown_connection()
 			return
@@ -87,16 +96,16 @@ class gmBackendListener(gmBorg.cBorg):
 				if self._listener_thread.isAlive():
 					_log.error('listener thread still alive after join()')
 					_log.debug('active threads: %s' % threading.enumerate())
-			except:
+			except Exception:
 				pass
-		except:
-			print sys.exc_info()
+		except Exception:
+			print(sys.exc_info())
 
 		self._listener_thread = None
 
 		try:
 			self.__unregister_unspecific_notifications()
-		except:
+		except Exception:
 			_log.exception('unable to unregister unspecific notifications')
 
 		self.__shutdown_connection()
@@ -116,6 +125,7 @@ class gmBackendListener(gmBorg.cBorg):
 
 		# listen to unspecific notifications
 		self.__register_unspecific_notifications()
+
 	#-------------------------------
 	def __register_unspecific_notifications(self):
 		for sig in self.unspecific_notifications:
@@ -126,6 +136,7 @@ class gmBackendListener(gmBorg.cBorg):
 				self._cursor.execute(cmd)
 			finally:
 				self._conn_lock.release()
+
 	#-------------------------------
 	def __unregister_unspecific_notifications(self):
 		for sig in self.unspecific_notifications:
@@ -136,17 +147,18 @@ class gmBackendListener(gmBorg.cBorg):
 				self._cursor.execute(cmd)
 			finally:
 				self._conn_lock.release()
+
 	#-------------------------------
 	def __shutdown_connection(self):
 		_log.debug('shutting down connection with backend PID [%s]', self.backend_pid)
 		self._conn_lock.acquire(1)
 		try:
 			self._conn.rollback()
-			self._conn.close()
-		except:
-			pass				# connection can already be closed :-(
+		except Exception:
+			pass
 		finally:
 			self._conn_lock.release()
+
 	#-------------------------------
 	def __start_thread(self):
 		if self._conn is None:
@@ -154,11 +166,12 @@ class gmBackendListener(gmBorg.cBorg):
 
 		self._listener_thread = threading.Thread (
 			target = self._process_notifications,
-			name = self.__class__.__name__
+			name = self.__class__.__name__,
+			daemon = True
 		)
-		self._listener_thread.setDaemon(True)
 		_log.info('starting listener thread')
 		self._listener_thread.start()
+
 	#-------------------------------
 	# the actual thread code
 	#-------------------------------
@@ -167,7 +180,6 @@ class gmBackendListener(gmBorg.cBorg):
 		# loop until quitting
 		_have_quit_lock = None
 		while not _have_quit_lock:
-
 			# quitting ?
 			if self._quit_lock.acquire(0):
 				break
@@ -178,21 +190,18 @@ class gmBackendListener(gmBorg.cBorg):
 				ready_input_sockets = select.select([self._conn_fd], [], [], self._poll_interval)[0]
 			finally:
 				self._conn_lock.release()
-
 			# any input available ?
 			if len(ready_input_sockets) == 0:
 				# no, select.select() timed out
 				# give others a chance to grab the conn lock (eg listen/unlisten)
 				time.sleep(0.3)
 				continue
-
 			# data available, wait for it to fully arrive
 			self._conn_lock.acquire(1)
 			try:
 				self._conn.poll()
 			finally:
 				self._conn_lock.release()
-
 			# any notifications ?
 			while len(self._conn.notifies) > 0:
 				# if self._quit_lock can be acquired we may be in
@@ -207,26 +216,35 @@ class gmBackendListener(gmBorg.cBorg):
 					notification = self._conn.notifies.pop()
 				finally:
 					self._conn_lock.release()
+				self.__notifications_received += 1
+				if self.debug:
+					print(notification)
+				_log.debug('#%s: %s (first param is PID of sending backend)', self.__notifications_received, notification)
 				# decode payload
-				payload = notification.payload.split(u'::')
+				payload = notification.payload.split('::')
 				operation = None
 				table = None
-				pk_column = None
-				pk_row = None
+				pk_column_name = None
+				pk_of_row = None
 				pk_identity = None
 				for item in payload:
-					if item.startswith(u'operation='):
-						operation = item.split(u'=')[1]
-					if item.startswith(u'table='):
-						table = item.split(u'=')[1]
-					if item.startswith(u'PK name='):
-						pk_column = item.split(u'=')[1]
-					if item.startswith(u'row PK='):
-						pk_row = item.split(u'=')[1]
-					if item.startswith(u'person PK='):
-						pk_identity = item.split(u'=')[1]
+					if item.startswith('operation='):
+						operation = item.split('=')[1]
+					if item.startswith('table='):
+						table = item.split('=')[1]
+					if item.startswith('PK name='):
+						pk_column_name = item.split('=')[1]
+					if item.startswith('row PK='):
+						pk_of_row = int(item.split('=')[1])
+					if item.startswith('person PK='):
+						try:
+							pk_identity = int(item.split('=')[1])
+						except ValueError:
+							_log.exception('error in change notification trigger')
+							pk_identity = -1
 				# try sending intra-client signals:
 				# 1) generic signal
+				self.__messages_sent += 1
 				try:
 					results = gmDispatcher.send (
 						signal = notification.channel,
@@ -236,15 +254,19 @@ class gmBackendListener(gmBorg.cBorg):
 						pk_identity = pk_identity,
 						operation = operation,
 						table = table,
-						pk_column = pk_column,
-						pk_row = pk_row
+						pk_column_name = pk_column_name,
+						pk_of_row = pk_of_row,
+						message_index = self.__messages_sent,
+						notification_index = self.__notifications_received
 					)
-				except:
-					print "problem routing notification [%s] from backend [%s] to intra-client dispatcher" % (notification.channel, notification.pid)
-					print sys.exc_info()
+				except Exception:
+					print("problem routing notification [%s] from backend [%s] to intra-client dispatcher" % (notification.channel, notification.pid))
+					print(sys.exc_info())
 				# 2) dynamically emulated old style table specific signals
 				if table is not None:
-					signal = u'%s_mod_db' % table
+					self.__messages_sent += 1
+					signal = '%s_mod_db' % table
+					_log.debug('emulating old-style table specific signal [%s]', signal)
 					try:
 						results = gmDispatcher.send (
 							signal = signal,
@@ -254,12 +276,14 @@ class gmBackendListener(gmBorg.cBorg):
 							pk_identity = pk_identity,
 							operation = operation,
 							table = table,
-							pk_column = pk_column,
-							pk_row = pk_row
+							pk_column_name = pk_column_name,
+							pk_of_row = pk_of_row,
+							message_index = self.__messages_sent,
+							notification_index = self.__notifications_received
 						)
-					except:
-						print "problem routing notification [%s] from backend [%s] to intra-client dispatcher" % (signal, notification.pid)
-						print sys.exc_info()
+					except Exception:
+						print("problem routing notification [%s] from backend [%s] to intra-client dispatcher" % (signal, notification.pid))
+						print(sys.exc_info())
 
 				# there *may* be more pending notifications but
 				# we don't care when quitting
@@ -269,6 +293,7 @@ class gmBackendListener(gmBorg.cBorg):
 
 		# exit thread activity
 		return
+
 #=====================================================================
 # main
 #=====================================================================
@@ -299,16 +324,16 @@ if __name__ == "__main__":
 			global notifies
 			notifies += 1
 			sys.stdout.flush()
-			print "\nBackend says: patient data has been modified (%s. notification)" % notifies
+			print("\nBackend says: patient data has been modified (%s. notification)" % notifies)
 		#-------------------------------
 		try:
 			n = int(sys.argv[2])
-		except:
-			print "You can set the number of iterations\nwith the second command line argument"
+		except Exception:
+			print("You can set the number of iterations\nwith the second command line argument")
 			n = 100000
 
 		# try loop without backend listener
-		print "Looping", n, "times through dummy function"
+		print("Looping", n, "times through dummy function")
 		i = 0
 		t1 = time.time()
 		while i < n:
@@ -316,21 +341,21 @@ if __name__ == "__main__":
 			i += 1
 		t2 = time.time()
 		t_nothreads = t2-t1
-		print "Without backend thread, it took", t_nothreads, "seconds"
+		print("Without backend thread, it took", t_nothreads, "seconds")
 
 		listener = gmBackendListener(conn = gmPG2.get_raw_connection())
 
 		# now try with listener to measure impact
-		print "Now in a new shell connect psql to the"
-		print "database <gnumed_v9> on localhost, return"
-		print "here and hit <enter> to continue."
-		raw_input('hit <enter> when done starting psql')
-		print "You now have about 30 seconds to go"
-		print "to the psql shell and type"
-		print " notify patient_changed<enter>"
-		print "several times."
-		print "This should trigger our backend listening callback."
-		print "You can also try to stop the demo with Ctrl-C !"
+		print("Now in a new shell connect psql to the")
+		print("database <gnumed_v9> on localhost, return")
+		print("here and hit <enter> to continue.")
+		input('hit <enter> when done starting psql')
+		print("You now have about 30 seconds to go")
+		print("to the psql shell and type")
+		print(" notify patient_changed<enter>")
+		print("several times.")
+		print("This should trigger our backend listening callback.")
+		print("You can also try to stop the demo with Ctrl-C !")
 
 		listener.register_callback('patient_changed', OnPatientModified)
 
@@ -340,8 +365,8 @@ if __name__ == "__main__":
 				counter += 1
 				time.sleep(1)
 				sys.stdout.flush()
-				print '.',
-			print "Looping",n,"times through dummy function"
+				print('.')
+			print("Looping",n,"times through dummy function")
 			i = 0
 			t1 = time.time()
 			while i < n:
@@ -349,49 +374,49 @@ if __name__ == "__main__":
 				i += 1
 			t2 = time.time()
 			t_threaded = t2-t1
-			print "With backend thread, it took", t_threaded, "seconds"
-			print "Difference:", t_threaded-t_nothreads
+			print("With backend thread, it took", t_threaded, "seconds")
+			print("Difference:", t_threaded-t_nothreads)
 		except KeyboardInterrupt:
-			print "cancelled by user"
+			print("cancelled by user")
 
 		listener.shutdown()
 		listener.unregister_callback('patient_changed', OnPatientModified)
 	#-------------------------------
 	def run_monitor():
 
-		print "starting up backend notifications monitor"
+		print("starting up backend notifications monitor")
 
 		def monitoring_callback(*args, **kwargs):
 			try:
 				kwargs['originated_in_database']
-				print '==> got notification from database "%s":' % kwargs['signal']
+				print('==> got notification from database "%s":' % kwargs['signal'])
 			except KeyError:
-				print '==> received signal from client: "%s"' % kwargs['signal']
+				print('==> received signal from client: "%s"' % kwargs['signal'])
 			del kwargs['signal']
 			for key in kwargs.keys():
-				print '    [%s]: %s' % (key, kwargs[key])
+				print('    [%s]: %s' % (key, kwargs[key]))
 
 		gmDispatcher.connect(receiver = monitoring_callback)
 
 		listener = gmBackendListener(conn = gmPG2.get_raw_connection())
-		print "listening for the following notifications:"
-		print "1) unspecific:"
+		print("listening for the following notifications:")
+		print("1) unspecific:")
 		for sig in listener.unspecific_notifications:
-			print '   - %s' % sig
+			print('   - %s' % sig)
 
 		while True:
 			pat = gmPersonSearch.ask_for_patient()
 			if pat is None:
 				break
-			print "found patient", pat
+			print("found patient", pat)
 			gmPerson.set_active_patient(patient=pat)
-			print "now waiting for notifications, hit <ENTER> to select another patient"
-			raw_input()
+			print("now waiting for notifications, hit <ENTER> to select another patient")
+			input()
 
-		print "cleanup"
+		print("cleanup")
 		listener.shutdown()
 
-		print "shutting down backend notifications monitor"
+		print("shutting down backend notifications monitor")
 
 	#-------------------------------
 	if sys.argv[1] == 'monitor':
