@@ -1,6 +1,3 @@
-
-from __future__ import print_function
-
 __doc__ = """GNUmed database object business class.
 
 Overview
@@ -133,7 +130,6 @@ __license__ = "GPL v2 or later"
 
 
 import sys
-import types
 import inspect
 import logging
 import datetime
@@ -148,10 +144,12 @@ from Gnumed.pycommon.gmTools import tex_escape_string
 from Gnumed.pycommon.gmTools import xetex_escape_string
 from Gnumed.pycommon.gmTools import compare_dict_likes
 from Gnumed.pycommon.gmTools import format_dict_like
-from Gnumed.pycommon.gmTools import format_dict_likes_comparison
+from Gnumed.pycommon.gmTools import dicts2table
+from Gnumed.pycommon.gmTools import u_left_arrow
 
 
 _log = logging.getLogger('gm.db')
+
 #============================================================
 class cBusinessDBObject(object):
 	"""Represents business objects in the database.
@@ -170,8 +168,8 @@ class cBusinessDBObject(object):
 
 	<_cmd_fetch_payload>
 		- must return exactly one row
-		- where clause argument values are expected
-		  in self.pk_obj (taken from __init__(aPK_obj))
+		- WHERE clause argument values are expected in
+		  self.pk_obj (taken from __init__(aPK_obj))
 		- must return xmin of all rows that _cmds_store_payload
 		  will be updating, so views must support the xmin columns
 		  of their underlying tables
@@ -284,6 +282,7 @@ def create_xxx(xxx=None, xxx=None):
 
 	return cXxxXxx(aPK_obj = rows[0]['pk'])
 	#return cXxxXxx(row = {'data': r, 'idx': idx, 'pk_field': 'pk_XXX'})
+
 #------------------------------------------------------------
 def delete_xxx(pk_XXX=None):
 	args = {'pk': pk_XXX}
@@ -292,6 +291,21 @@ def delete_xxx(pk_XXX=None):
 	return True
 #------------------------------------------------------------
 
+#------------------------------------------------------------
+# widget code
+#------------------------------------------------------------
+def edit_xxx(parent=None, xxx=None, single_entry=False, presets=None):
+
+#------------------------------------------------------------
+def delete_xxx()
+
+#------------------------------------------------------------
+def manage_xxx()
+
+#------------------------------------------------------------
+# remember to add in clinical item generic workflows
+
+
 *********** end of template ***********
 
 	"""
@@ -301,7 +315,7 @@ def delete_xxx(pk_XXX=None):
 
 		Call from child classes:
 
-			super(cChildClass, self).__init__(aPK_obj = aPK_obj, row = row)
+			super(cChildClass, self).__init__(aPK_obj = aPK_obj, row = row, link_obj = link_obj)
 		"""
 		# initialize those "too early" because checking descendants might
 		# fail which will then call __str__ in stack trace logging if --debug
@@ -320,9 +334,10 @@ def delete_xxx(pk_XXX=None):
 		if aPK_obj is not None:
 			self.__init_from_pk(aPK_obj = aPK_obj, link_obj = link_obj)
 		else:
-			self._init_from_row_data(row=row)
+			self._init_from_row_data(row = row)
 
 		self._is_modified = False
+
 	#--------------------------------------------------------
 	def __init_from_pk(self, aPK_obj=None, link_obj=None):
 		"""Creates a new clinical item instance by its PK.
@@ -345,75 +360,75 @@ def delete_xxx(pk_XXX=None):
 			return True
 
 		if result is False:
-			raise gmExceptions.ConstructorError, "[%s:%s]: error loading instance" % (self.__class__.__name__, self.pk_obj)
+			raise gmExceptions.ConstructorError("[%s:%s]: error loading instance" % (self.__class__.__name__, self.pk_obj))
+
 	#--------------------------------------------------------
 	def _init_from_row_data(self, row=None):
 		"""Creates a new clinical item instance given its fields.
 
 		row must be a dict with the fields:
-			- pk_field: the name of the primary key field
 			- idx: a dict mapping field names to position
 			- data: the field values in a list (as returned by
 			  cursor.fetchone() in the DB-API)
+			- pk_field: the name of the primary key field
+			OR
+			- pk_obj: a dictionary suitable for passed to cursor.execute
+			  and holding the primary key values, used for composite PKs
 
-		row = {'data': rows[0], 'idx': idx, 'pk_field': 'pk_XXX (the PK column name)'}
-
+		row = {
+			'data': rows[0],
+			'idx': idx,
+			'pk_field': 'pk_XXX (the PK column name)',
+			'pk_obj': {'pk_col1': pk_col1_val, 'pk_col2': pk_col2_val}
+		}
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 		objects = [ cChildClass(row = {'data': r, 'idx': idx, 'pk_field': 'the PK column name'}) for r in rows ]
 		"""
-		try:
-			self._idx = row['idx']
-			self._payload = row['data']
-			self.pk_obj = self._payload[self._idx[row['pk_field']]]
-		except:
-			_log.exception('faulty <row> argument structure: %s' % row)
-			raise gmExceptions.ConstructorError, "[%s:??]: error loading instance from row data" % self.__class__.__name__
+		assert ('data' in row), "[%s:??]: 'data' missing from <row> argument: %s" % (self.__class__.__name__, row)
+		assert ('idx' in row), "[%s:??]: 'idx' missing from <row> argument: %s" % (self.__class__.__name__, row)
+		assert (len(row['idx']) == len(row['data'])), "[%s:??]: 'idx'<->'data' field count mismatch: %s" % (self.__class__.__name__, row)
+		faulty_pk = (('pk_field' not in row) and ('pk_obj' not in row))
+		assert not faulty_pk, "[%s:??]: either 'pk_field' or 'pk_obj' must exist in <row> argument: %s" % (self.__class__.__name__, row)
 
-		if len(self._idx.keys()) != len(self._payload):
-			_log.critical('field index vs. payload length mismatch: %s field names vs. %s fields' % (len(self._idx.keys()), len(self._payload)))
-			_log.critical('faulty <row> argument structure: %s' % row)
-			raise gmExceptions.ConstructorError, "[%s:??]: error loading instance from row data" % self.__class__.__name__
+		self._idx = row['idx']
+		self._payload = row['data']
+		if 'pk_field' in row:
+			self.pk_obj = row['data'][row['idx'][row['pk_field']]]
+		else:
+			self.pk_obj = row['pk_obj']
 
 		self.payload_most_recently_fetched = {}
 		for field in self._idx.keys():
 			self.payload_most_recently_fetched[field] = self._payload[self._idx[field]]
+
 	#--------------------------------------------------------
 	def __del__(self):
-		if u'_is_modified' in self.__dict__:
+		if '_is_modified' in self.__dict__:
 			if self._is_modified:
 				_log.critical('[%s:%s]: loosing payload changes' % (self.__class__.__name__, self.pk_obj))
 				_log.debug('most recently fetched: %s' % self.payload_most_recently_fetched)
 				_log.debug('modified: %s' % self._payload)
-	#--------------------------------------------------------
-	def __str__(self):
-		tmp = []
-		try:
-			for attr in self._idx.keys():
-				if self._payload[self._idx[attr]] is None:
-					tmp.append('%s: NULL' % attr)
-				else:
-					tmp.append('%s: >>%s<<' % (attr, self._payload[self._idx[attr]]))
-			return '[%s:%s]: %s' % (self.__class__.__name__, self.pk_obj, str(tmp))
-			#return '[%s:%s]:\n %s' % (self.__class__.__name__, self.pk_obj, '\n '.join(lines))
-		except:
-			return 'nascent [%s @ %s], cannot show payload and primary key' %(self.__class__.__name__, id(self))
 
 	#--------------------------------------------------------
-	def __unicode__(self):
+	def __str__(self):
 		lines = []
 		try:
 			for attr in self._idx.keys():
 				if self._payload[self._idx[attr]] is None:
-					lines.append(u'%s: NULL' % attr)
+					lines.append('%s: NULL' % attr)
 				else:
-					lines.append('%s: %s' % (attr, self._payload[self._idx[attr]]))
-			return '[%s:%s]:\n%s' % (self.__class__.__name__, self.pk_obj, u'\n'.join(lines))
-		except:
-			return 'nascent [%s @ %s], cannot show payload and primary key' %(self.__class__.__name__, id(self))
+					lines.append('%s: %s [%s]' % (
+						attr,
+						self._payload[self._idx[attr]],
+						type(self._payload[self._idx[attr]])
+					))
+			return '[%s:%s]:\n%s' % (self.__class__.__name__, self.pk_obj, '\n'.join(lines))
+		except Exception:
+			return 'likely nascent [%s @ %s], cannot show payload and primary key' %(self.__class__.__name__, id(self))
 
 	#--------------------------------------------------------
 	def __getitem__(self, attribute):
-		# use try: except: as it is faster and we want this as fast as possible
+		# use try: except KeyError: as it is faster and we want this as fast as possible
 
 		# 1) backend payload cache
 		try:
@@ -427,7 +442,7 @@ def delete_xxx(pk_XXX=None):
 			_log.warning('[%s]: no attribute [%s]' % (self.__class__.__name__, attribute))
 			_log.warning('[%s]: valid attributes: %s' % (self.__class__.__name__, str(self._idx.keys())))
 			_log.warning('[%s]: no getter method [get_%s]' % (self.__class__.__name__, attribute))
-			methods = filter(lambda x: x[0].startswith('get_'), inspect.getmembers(self, inspect.ismethod))
+			methods = [ m for m in inspect.getmembers(self, inspect.ismethod) if m[0].startswith('get_') ]
 			_log.warning('[%s]: valid getter methods: %s' % (self.__class__.__name__, str(methods)))
 			raise KeyError('[%s]: cannot read from key [%s]' % (self.__class__.__name__, attribute))
 
@@ -457,7 +472,7 @@ def delete_xxx(pk_XXX=None):
 				del self._ext_cache[attribute]
 			except KeyError:
 				pass
-			if type(value) is types.TupleType:
+			if type(value) == tuple:
 				if setter(*value):
 					self._is_modified = True
 					return
@@ -469,14 +484,16 @@ def delete_xxx(pk_XXX=None):
 		# 3) don't know what to do with <attribute>
 		_log.error('[%s]: cannot find attribute <%s> or setter method [set_%s]' % (self.__class__.__name__, attribute, attribute))
 		_log.warning('[%s]: settable attributes: %s' % (self.__class__.__name__, str(self.__class__._updatable_fields)))
-		methods = filter(lambda x: x[0].startswith('set_'), inspect.getmembers(self, inspect.ismethod))
+		methods = [ m for m in inspect.getmembers(self, inspect.ismethod) if m[0].startswith('set_') ]
 		_log.warning('[%s]: valid setter methods: %s' % (self.__class__.__name__, str(methods)))
 		raise AttributeError('[%s]: cannot set [%s]' % (self.__class__.__name__, attribute))
+
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
 	def same_payload(self, another_object=None):
 		raise NotImplementedError('comparison between [%s] and [%s] not implemented' % (self, another_object))
+
 	#--------------------------------------------------------
 	def is_modified(self):
 		return self._is_modified
@@ -493,9 +510,9 @@ def delete_xxx(pk_XXX=None):
 		return self.__class__._updatable_fields
 
 	#--------------------------------------------------------
-	def fields_as_dict(self, date_format='%Y %b %d  %H:%M', none_string=u'', escape_style=None, bool_strings=None):
+	def fields_as_dict(self, date_format='%Y %b %d  %H:%M', none_string='', escape_style=None, bool_strings=None):
 		if bool_strings is None:
-			bools = {True: u'True', False: u'False'}
+			bools = {True: 'True', False: 'False'}
 		else:
 			bools = {True: bool_strings[0], False: bool_strings[1]}
 		data = {}
@@ -516,79 +533,129 @@ def delete_xxx(pk_XXX=None):
 				if date_format is None:
 					data[field] = val
 					continue
-				data[field] = pydt_strftime(val, format = date_format, encoding = 'utf8')
-				if escape_style in [u'latex', u'tex']:
+				data[field] = pydt_strftime(val, format = date_format)
+				if escape_style in ['latex', 'tex']:
 					data[field] = tex_escape_string(data[field])
-				elif escape_style in [u'xetex', u'xelatex']:
+				elif escape_style in ['xetex', 'xelatex']:
 					data[field] = xetex_escape_string(data[field])
 				continue
 
 			try:
-				data[field] = unicode(val, encoding = 'utf8', errors = 'replace')
+				data[field] = str(val, encoding = 'utf8', errors = 'replace')
 			except TypeError:
 				try:
-					data[field] = unicode(val)
+					data[field] = str(val)
 				except (UnicodeDecodeError, TypeError):
 					val = '%s' % str(val)
 					data[field] = val.decode('utf8', 'replace')
-			if escape_style in [u'latex', u'tex']:
+			if escape_style in ['latex', 'tex']:
 				data[field] = tex_escape_string(data[field])
-			elif escape_style in [u'xetex', u'xelatex']:
+			elif escape_style in ['xetex', 'xelatex']:
 				data[field] = xetex_escape_string(data[field])
 
 		return data
+
 	#--------------------------------------------------------
 	def get_patient(self):
 		_log.error('[%s:%s]: forgot to override get_patient()' % (self.__class__.__name__, self.pk_obj))
 		return None
 
+	#--------------------------------------------------------
+	def _get_patient_pk(self):
+		try:
+			return self._payload[self._idx['pk_patient']]
+		except KeyError:
+			pass
+		try:
+			return self._payload[self._idx['pk_identity']]
+		except KeyError:
+			return None
+
+	patient_pk = property(_get_patient_pk)
+
+	#--------------------------------------------------------
+	def _get_staff_id(self):
+		try:
+			return self._payload[self._idx['pk_staff']]
+		except KeyError:
+			_log.debug('[%s]: .pk_staff should be added to the view', self.__class__.__name__)
+		try:
+			return self._payload[self._idx['pk_provider']]
+		except KeyError:
+			pass
+		mod_by = None
+		try:
+			mod_by = self._payload[self._idx['modified_by_raw']]
+		except KeyError:
+			_log.debug('[%s]: .modified_by_raw should be added to the view', self.__class__.__name__)
+		if mod_by is not None:
+			# find by DB account
+			args = {'db_u': mod_by}
+			cmd = "SELECT pk FROM dem.staff WHERE db_user = %(db_u)s"
+			rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+			if len(rows) > 0:
+				# logically, they are all the same provider, because they share the DB account
+				return rows[0][0]
+
+		mod_by = self._payload[self._idx['modified_by']]
+		# is .modified_by a "<DB-account>" ?
+		if mod_by.startswith('<') and mod_by.endswith('>'):
+			# find by DB account
+			args = {'db_u': mod_by.lstrip('<').rstrip('>')}
+			cmd = "SELECT pk FROM dem.staff WHERE db_user = %(db_u)s"
+			rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+			if len(rows) > 0:
+				# logically, they are all the same provider, because they share the DB account
+				return rows[0][0]
+
+		# .modified_by is probably dem.staff.short_alias
+		args = {'alias': mod_by}
+		cmd = "SELECT pk FROM dem.staff WHERE short_alias = %(alias)s"
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+		if len(rows) > 0:
+			# logically, they are all the same provider, because they share the DB account
+			return rows[0][0]
+
+		_log.error('[%s]: cannot retrieve staff ID for [%s]', self.__class__.__name__, mod_by)
+		return None
+
+	staff_id = property(_get_staff_id)
+
 	#--------------------------------------------------------
 	def format(self, *args, **kwargs):
 		return format_dict_like (
-			self.fields_as_dict(none_string = u'<?>'),
+			self.fields_as_dict(none_string = '<?>'),
 			tabular = True,
 			value_delimiters = None
-		).split(u'\n')
+		).split('\n')
 
 	#--------------------------------------------------------
 	def _get_revision_history(self, query, args, title):
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': query, 'args': args}], get_col_idx = True)
-		lines = []
-		lines.append(u'%s (%s versions)' % (title, rows[0]['row_version'] + 1))
-		if len(rows) == 1:
-			lines.append(u'')
-			lines.extend(format_dict_like (
-					rows[0],
-					left_margin = 1,
-					tabular = True,
-					value_delimiters = None,
-					eol = None
-			))
-			return lines
 
-		for row_idx in range(len(rows)-1):
-			lines.append(u'')
-			row_older = rows[row_idx + 1]
-			row_newer = rows[row_idx]
-			lines.extend(format_dict_likes_comparison (
-				row_older,
-				row_newer,
-				title_left = _('Revision #%s') % row_older['row_version'],
-				title_right = _('Revision #%s') % row_newer['row_version'],
-				left_margin = 0,
-				key_delim = u' | ',
-				data_delim = u' | ',
-				missing_string = u'',
-				ignore_diff_in_keys = ['audit__action_applied', 'audit__action_when', 'audit__action_by', 'pk_audit', 'row_version', 'modified_when', 'modified_by']
+		lines = []
+		if rows == 0:
+			lines.append('%s (no versions)' % title)
+		else:
+			lines.append('%s (%s versions)' % (title, rows[0]['row_version'] + 1))
+			column_labels = [ 'rev %s (%s)' % (r['row_version'], pydt_strftime(r['audit__action_when'], format = '%Y %b %d %H:%M', none_str = 'live row')) for r in rows ]
+			lines.extend (dicts2table (
+				rows,
+				left_margin = 1,
+				eol = None,
+				keys2ignore = ['audit__action_when', 'row_version', 'pk_audit'],
+				show_only_changes = True,
+				column_labels = column_labels,
+				date_format = '%Y %b %d %H:%M',
+				equality_value = u_left_arrow
 			))
 		return lines
 
 	#--------------------------------------------------------
 	def refetch_payload(self, ignore_changes=False, link_obj=None):
-		"""Fetch field values from backend.
-		"""
+		"""Fetch field values from backend."""
 		if self._is_modified:
-			compare_dict_likes(self.original_payload, self.fields_as_dict(date_format = None, none_string = None), u'original payload', u'modified payload')
+			compare_dict_likes(self.original_payload, self.fields_as_dict(date_format = None, none_string = None), 'original payload', 'modified payload')
 			if ignore_changes:
 				_log.critical('[%s:%s]: loosing payload changes' % (self.__class__.__name__, self.pk_obj))
 				#_log.debug('most recently fetched: %s' % self.payload_most_recently_fetched)
@@ -597,26 +664,31 @@ def delete_xxx(pk_XXX=None):
 				_log.critical('[%s:%s]: cannot reload, payload changed' % (self.__class__.__name__, self.pk_obj))
 				return False
 
-		if type(self.pk_obj) == types.DictType:
-			arg = self.pk_obj
+		if isinstance(self.pk_obj, dict):
+			args = self.pk_obj
 		else:
-			arg = [self.pk_obj]
+			args = [self.pk_obj]
 		rows, self._idx = gmPG2.run_ro_queries (
 			link_obj = link_obj,
-			queries = [{'cmd': self.__class__._cmd_fetch_payload, 'args': arg}],
+			queries = [{'cmd': self.__class__._cmd_fetch_payload, 'args': args}],
 			get_col_idx = True
 		)
 		if len(rows) == 0:
 			_log.error('[%s:%s]: no such instance' % (self.__class__.__name__, self.pk_obj))
 			return False
+		if len(rows) > 1:
+			raise AssertionError('[%s:%s]: %s instances !' % (self.__class__.__name__, self.pk_obj, len(rows)))
 		self._payload = rows[0]
 		return True
+
 	#--------------------------------------------------------
 	def __noop(self):
 		pass
+
 	#--------------------------------------------------------
 	def save(self, conn=None):
 		return self.save_payload(conn = conn)
+
 	#--------------------------------------------------------
 	def save_payload(self, conn=None):
 		"""Store updated values (if any) in database.
@@ -651,13 +723,14 @@ def delete_xxx(pk_XXX=None):
 			get_col_idx = True
 		)
 
-		# this can happen if:
-		# - someone else updated the row so XMIN does not match anymore
-		# - the PK went away (rows were deleted from under us)
-		# - another WHERE condition of the UPDATE did not produce any rows to update
-		# - savepoints are used since subtransactions may relevantly change the xmin/xmax ...
+		# success ?
 		if len(rows) == 0:
-			return (False, (u'cannot update row', _('[%s:%s]: row not updated (nothing returned), row in use ?') % (self.__class__.__name__, self.pk_obj)))
+			# nothing updated - this can happen if:
+			# - someone else updated the row so XMIN does not match anymore
+			# - the PK went away (rows were deleted from under us)
+			# - another WHERE condition of the UPDATE did not produce any rows to update
+			# - savepoints are used since subtransactions may relevantly change the xmin/xmax ...
+			return (False, ('cannot update row', _('[%s:%s]: row not updated (nothing returned), row in use ?') % (self.__class__.__name__, self.pk_obj)))
 
 		# update cached values from should-be-first-and-only
 		# result row of last query,
@@ -671,9 +744,9 @@ def delete_xxx(pk_XXX=None):
 			except KeyError:
 				conn.rollback()
 				close_conn()
-				_log.error('[%s:%s]: cannot update instance, XMIN refetch key mismatch on [%s]' % (self.__class__.__name__, self.pk_obj, key))
+				_log.error('[%s:%s]: cannot update instance, XMIN-refetch key mismatch on [%s]' % (self.__class__.__name__, self.pk_obj, key))
 				_log.error('payload keys: %s' % str(self._idx))
-				_log.error('XMIN refetch keys: %s' % str(idx))
+				_log.error('XMIN-refetch keys: %s' % str(idx))
 				_log.error(args)
 				raise
 
@@ -692,81 +765,13 @@ def delete_xxx(pk_XXX=None):
 
 		return (True, None)
 
-#============================================================
-def jsonclasshintify(obj):
-	# this should eventually be somewhere else
-	""" turn the data into a list of dicts, adding "class hints".
-		all objects get turned into dictionaries which the other end
-		will interpret as "object", via the __jsonclass__ hint,
-		as specified by the JSONRPC protocol standard.
-	"""
-	if isinstance(obj, list):
-		return map(jsonclasshintify, obj)
-	elif isinstance(obj, gmPG2.dbapi.tz.FixedOffsetTimezone):
-		# this will get decoded as "from jsonobjproxy import {clsname}"
-		# at the remote (client) end.
-		res = {'__jsonclass__': ["jsonobjproxy.FixedOffsetTimezone"]}
-		res['name'] = obj._name
-		res['offset'] = jsonclasshintify(obj._offset)
-		return res
-	elif isinstance(obj, datetime.timedelta):
-		# this will get decoded as "from jsonobjproxy import {clsname}"
-		# at the remote (client) end.
-		res = {'__jsonclass__': ["jsonobjproxy.TimeDelta"]}
-		res['days'] = obj.days
-		res['seconds'] = obj.seconds
-		res['microseconds'] = obj.microseconds
-		return res
-	elif isinstance(obj, datetime.time):
-		# this will get decoded as "from jsonobjproxy import {clsname}"
-		# at the remote (client) end.
-		res = {'__jsonclass__': ["jsonobjproxy.Time"]}
-		res['hour'] = obj.hour
-		res['minute'] = obj.minute
-		res['second'] = obj.second
-		res['microsecond'] = obj.microsecond
-		res['tzinfo'] = jsonclasshintify(obj.tzinfo)
-		return res
-	elif isinstance(obj, datetime.datetime):
-		# this will get decoded as "from jsonobjproxy import {clsname}"
-		# at the remote (client) end.
-		res = {'__jsonclass__': ["jsonobjproxy.DateTime"]}
-		res['year'] = obj.year
-		res['month'] = obj.month
-		res['day'] = obj.day
-		res['hour'] = obj.hour
-		res['minute'] = obj.minute
-		res['second'] = obj.second
-		res['microsecond'] = obj.microsecond
-		res['tzinfo'] = jsonclasshintify(obj.tzinfo)
-		return res
-	elif isinstance(obj, cBusinessDBObject):
-		# this will get decoded as "from jsonobjproxy import {clsname}"
-		# at the remote (client) end.
-		res = {'__jsonclass__': ["jsonobjproxy.%s" % obj.__class__.__name__]}
-		for k in obj.get_fields():
-			t = jsonclasshintify(obj[k])
-			res[k] = t
-		print("props", res, dir(obj))
-		for attribute in dir(obj):
-			if not attribute.startswith("get_"):
-				continue
-			k = attribute[4:]
-			if k in res:
-				continue
-			getter = getattr(obj, attribute, None)
-			if callable(getter):
-				res[k] = jsonclasshintify(getter())
-		return res
-	return obj
-
 #============================================================
 if __name__ == '__main__':
 
 	if len(sys.argv) < 2:
 		sys.exit()
 
-	if sys.argv[1] != u'test':
+	if sys.argv[1] != 'test':
 		sys.exit()
 
 	#--------------------------------------------------------
@@ -789,12 +794,12 @@ if __name__ == '__main__':
 		'pk_field': 'bogus_pk',
 		'idx': {'bogus_pk': 0, 'bogus_field': 1, 'bogus_date': 2},
 		'data': [-1, 'bogus_data', datetime.datetime.now()]
+		#'data': {'bogus_pk': -1, 'bogus_field': 'bogus_data', 'bogus_date': datetime.datetime.now()}
 	}
 	obj = cTestObj(row=data)
-	#print(obj['wrong_field'])
-	#print(jsonclasshintify(obj))
-	#obj['wrong_field'] = 1
-	#print(obj.fields_as_dict())
 	print(obj.format())
+	#print(obj['wrong_field'])
+	#obj['wrong_field'] = 1
+	print(obj.fields_as_dict())
 
 #============================================================
