@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-from __future__ import print_function
+
 
 __doc__ = """GNUmed internetworking tools."""
 
@@ -12,12 +12,8 @@ __license__ = "GPL v2 or later (details at http://www.gnu.org)"
 import sys
 import os.path
 import logging
-import urllib2 as wget
-import urllib
-import MimeWriter
-import mimetypes
-import mimetools
-import StringIO
+import urllib.request
+import urllib.error
 import zipfile
 import webbrowser
 import io
@@ -30,6 +26,7 @@ from Gnumed.pycommon import gmLog2
 from Gnumed.pycommon import gmTools
 from Gnumed.pycommon import gmShellAPI
 from Gnumed.pycommon import gmCfg2
+from Gnumed.pycommon import gmMimeLib
 
 
 _log = logging.getLogger('gm.net')
@@ -54,13 +51,13 @@ def download_file(url, filename=None, suffix=None):
 	_log.debug('downloading [%s] into [%s]', url, filename)
 
 	try:
-		dl_name, headers = urllib.urlretrieve(url, filename)
+		dl_name, headers = urllib.request.urlretrieve(url, filename)
 	except (ValueError, OSError, IOError):
 		_log.exception('cannot download from [%s]', url)
 		gmLog2.log_stack_trace()
 		return None
 
-	_log.debug(u'%s' % headers)
+	_log.debug('%s' % headers)
 	return dl_name
 #===========================================================================
 # data pack handling
@@ -73,7 +70,7 @@ def download_data_pack(pack_url, filename=None, md5_url=None):
 	_log.debug('downloading data pack from: %s', pack_url)
 	dp_fname = download_file(pack_url, filename = filename, suffix = 'zip')
 	_log.debug('downloading MD5 from: %s', md5_url)
-	md5_fname = download_file(md5_url, filename = dp_fname + u'.md5')
+	md5_fname = download_file(md5_url, filename = dp_fname + '.md5')
 
 	md5_file = io.open(md5_fname, mode = 'rt', encoding = 'utf8')
 	md5_expected = md5_file.readline().strip('\n')
@@ -103,6 +100,7 @@ def unzip_data_pack(filename=None):
 	data_pack.extractall(unzip_dir)
 
 	return unzip_dir
+
 #---------------------------------------------------------------------------
 def install_data_pack(data_pack=None, conn=None):
 	from Gnumed.pycommon import gmPsql
@@ -110,7 +108,7 @@ def install_data_pack(data_pack=None, conn=None):
 	sql_script = os.path.join(data_pack['unzip_dir'], 'install-data-pack.sql')
 	if psql.run(sql_script) == 0:
 		curs = conn.cursor()
-		curs.execute(u'select gm.log_script_insertion(%(name)s, %(ver)s)', {'name': data_pack['pack_url'], 'ver': u'current'})
+		curs.execute('select gm.log_script_insertion(%(name)s, %(ver)s)', {'name': data_pack['pack_url'], 'ver': 'current'})
 		curs.close()
 		conn.commit()
 		return True
@@ -128,13 +126,13 @@ def download_data_pack_old(url, target_dir=None):
 
 	gmTools.mkdir(directory = target_dir)
 
-	# FIXME: rewrite to use urllib.urlretrieve() and 
+	# FIXME: rewrite to use urllib.request.urlretrieve() and 
 
 	paths = gmTools.gmPaths()
 	local_script = os.path.join(paths.local_base_dir, '..', 'external-tools', 'gm-download_data')
 
-	candidates = [u'gm-download_data', u'gm-download_data.bat', local_script, u'gm-download_data.bat']
-	args = u' %s %s' % (url, target_dir)
+	candidates = ['gm-download_data', 'gm-download_data.bat', local_script, 'gm-download_data.bat']
+	args = ' %s %s' % (url, target_dir)
 
 	success = gmShellAPI.run_first_available_in_shell (
 		binaries = candidates,
@@ -164,8 +162,8 @@ def compare_versions(left_version, right_version):
 	left_parts = left_version.split('.')
 	right_parts = right_version.split('.')
 
-	tmp, left_major = gmTools.input2decimal(u'%s.%s' % (left_parts[0], left_parts[1]))
-	tmp, right_major = gmTools.input2decimal(u'%s.%s' % (right_parts[0], right_parts[1]))
+	tmp, left_major = gmTools.input2decimal('%s.%s' % (left_parts[0], left_parts[1]))
+	tmp, right_major = gmTools.input2decimal('%s.%s' % (right_parts[0], right_parts[1]))
 
 	if left_major < right_major:
 		_log.debug('left version [%s] < right version [%s]: major part', left_version, right_version)
@@ -196,13 +194,13 @@ def check_for_update(url=None, current_branch=None, current_version=None, consid
 	False: up to date
 	None: don't know
 	"""
-	if current_version == u'GIT HEAD':
+	if current_version == 'GIT HEAD':
 		_log.debug('GIT HEAD always up to date')
 		return (False, None)
 
 	try:
-		remote_file = wget.urlopen(url)
-	except (wget.URLError, ValueError, OSError, IOError):
+		remote_file = urllib.request.urlopen(url)
+	except (urllib.error.URLError, ValueError, OSError, IOError):
 		# IOError: socket.error
 		_log.exception("cannot retrieve version file from [%s]", url)
 		return (None, _('Cannot retrieve version information from:\n\n%s') % url)
@@ -211,7 +209,8 @@ def check_for_update(url=None, current_branch=None, current_version=None, consid
 
 	cfg = gmCfg2.gmCfgData()
 	try:
-		cfg.add_stream_source(source = 'gm-versions', stream = remote_file)
+		#remote_file.read().decode(resource.headers.get_content_charset())
+		cfg.add_stream_source(source = 'gm-versions', stream = remote_file, encoding = u'utf8')
 	except (UnicodeDecodeError):
 		remote_file.close()
 		_log.exception("cannot read version file from [%s]", url)
@@ -290,54 +289,128 @@ def check_for_update(url=None, current_branch=None, current_version=None, consid
 	msg += _(' Your current version: "%s"\n') % current_version
 	if consider_latest_branch:
 		if new_release_on_current_branch_available:
-			msg += u'\n'
+			msg += '\n'
 			msg += _(' New version: "%s"') % latest_release_on_current_branch
-			msg += u'\n'
+			msg += '\n'
 			msg += _(' - bug fixes only\n')
 			msg += _(' - database fixups may be needed\n')
 		if new_release_on_latest_branch_available:
 			if current_branch != latest_branch:
-				msg += u'\n'
+				msg += '\n'
 				msg += _(' New version: "%s"') % latest_release_on_latest_branch
-				msg += u'\n'
+				msg += '\n'
 				msg += _(' - bug fixes and new features\n')
 				msg += _(' - database upgrade required\n')
 	else:
-		msg += u'\n'
+		msg += '\n'
 		msg += _(' New version: "%s"') % latest_release_on_current_branch
-		msg += u'\n'
+		msg += '\n'
 		msg += _(' - bug fixes only\n')
 		msg += _(' - database fixups may be needed\n')
 
-	msg += u'\n\n'
+	msg += '\n\n'
 	msg += _(
 		'Note, however, that this version may not yet\n'
 		'be available *pre-packaged* for your system.'
 	)
 
-	msg += u'\n\n'
+	msg += '\n\n'
 	msg += _('Details are found on <http://wiki.gnumed.de>.\n')
-	msg += u'\n'
+	msg += '\n'
 	msg += _('Version information loaded from:\n\n %s') % url
 
 	return (True, msg)
+
 #===========================================================================
 # mail handling
 #---------------------------------------------------------------------------
-default_mail_sender = u'gnumed@gmx.net'
-default_mail_receiver = u'gnumed-devel@gnu.org'
-default_mail_server = u'mail.gmx.net'
+default_mail_sender = 'gnumed@gmx.net'
+default_mail_receiver = 'gnumed-devel@gnu.org'
+default_mail_server = 'mail.gmx.net'
 
-def send_mail(sender=None, receiver=None, message=None, server=None, auth=None, debug=False, subject=None, encoding='quoted-printable', attachments=None):
-	# FIXME: How to generate and send mails: a step by step tutorial
-	# FIXME: http://groups.google.com/group/comp.lang.python/browse_thread/thread/e0793c1007361398/
-	# FIXME: google for aspineux blog
+
+#---------------------------------------------------------------------------
+from email.mime.text import MIMEText
+from email.mime.multipart import MIMEMultipart
+from email.mime.image import MIMEImage
+from email.mime.audio import MIMEAudio
+from email.mime.application import MIMEApplication
+
+def compose_email(sender=None, receiver=None, message=None, subject=None, files2attach=None):
+	# filenames unicode
+	# file content binary or utf8
+	# files2attach = [(filename, mimetype-or-None), ...]
 
 	if message is None:
-		return False
+		raise ValueError('<message> is None, cannot compose email')
 
 	message = message.lstrip().lstrip('\r\n').lstrip()
 
+	if sender is None:
+		sender = default_mail_sender
+
+	if receiver is None:
+		receiver = [default_mail_receiver]
+
+	if subject is None:
+		subject = 'compose_email() test'
+
+	if files2attach is None:
+		email = MIMEText(message, 'plain', 'utf8')
+	else:
+		email = MIMEMultipart()
+		email.attach(MIMEText(message, 'plain', 'utf8'))
+
+	email['From'] = sender
+	email['To'] = ', '.join(receiver)
+	email['Subject'] = subject
+
+	if files2attach is None:
+		return email
+
+	for file2attach in files2attach:
+		filename = file2attach[0]
+		try:
+			mimetype = file2attach[1]
+		except IndexError:
+			mimetype = gmMimeLib.guess_mimetype(filename = filename)
+		# text/*
+		if mimetype.startswith('text/'):
+			txt = io.open(filename, mode = 'rt', encoding = 'utf8')
+			attachment = MIMEText(txt.read(), 'plain', 'utf8')
+			txt.close()
+		# image/*
+		elif mimetype.startswith('image/'):
+			img = io.open(filename, mode = 'rb')
+			attachment = MIMEImage(img.read())
+			img.close()
+		# audio/*
+		elif mimetype.startswith('audio/'):
+			song = io.open(filename, mode = 'rb')
+			attachment = MIMEAudio(song.read())
+			song.close()
+		# catch-all application/*
+		else:
+			_log.debug('attaching [%s] with type [%s]', filename, mimetype)
+			mime_subtype = mimetype.split('/', 1)[1]
+			data = io.open(filename, mode = 'rb')
+			attachment = MIMEApplication(data.read(), mime_subtype)
+			data.close()
+
+		try:
+			attachment.replace_header('Content-Disposition', 'attachment; filename="%s"' % gmTools.fname_from_path(filename))
+		except KeyError:
+			attachment.add_header('Content-Disposition', 'attachment; filename="%s"' % gmTools.fname_from_path(filename))
+		email.attach(attachment)
+
+	return email
+
+#---------------------------------------------------------------------------
+def send_email(sender=None, receiver=None, email=None, server=None, auth=None, debug=False):
+
+	if email is None:
+		raise ValueError('<email> is None, cannot send')
+
 	if sender is None:
 		sender = default_mail_sender
 
@@ -347,66 +420,26 @@ def send_mail(sender=None, receiver=None, message=None, server=None, auth=None,
 	if server is None:
 		server = default_mail_server
 
-	if subject is None:
-		subject = u'gmTools.py: send_mail() test'
-
-	msg = StringIO.StringIO()
-	writer = MimeWriter.MimeWriter(msg)
-	writer.addheader('To', u', '.join(receiver))
-	writer.addheader('From', sender)
-	writer.addheader('Subject', subject[:50].replace('\r', '/').replace('\n', '/'))
-	writer.addheader('MIME-Version', '1.0')
-
-	writer.startmultipartbody('mixed')
-
-	# start with a text/plain part
-	part = writer.nextpart()
-	body = part.startbody('text/plain')
-	part.flushheaders()
-	body.write(message.encode(encoding))
-
-	# now add the attachments
-	if attachments is not None:
-		for attmt in attachments:
-			filename = os.path.basename(attmt[0])
-			try:
-				mtype = attmt[1]
-				encoding = attmt[2]
-			except IndexError:
-				mtype, encoding = mimetypes.guess_type(attmt[0])
-				if mtype is None:
-					mtype = 'application/octet-stream'
-					encoding = 'base64'
-				elif mtype == 'text/plain':
-					encoding = 'quoted-printable'
-				else:
-					encoding = 'base64'
-
-			part = writer.nextpart()
-			part.addheader('Content-Transfer-Encoding', encoding)
-			body = part.startbody("%s; name=%s" % (mtype, filename))
-			mimetools.encode(open(attmt[0], 'rb'), body, encoding)
-
-	writer.lastpart()
-
 	import smtplib
 	failed = False
 	refused = []
 	try:
 		session = smtplib.SMTP(server)
 		session.set_debuglevel(debug)
-		session.starttls()
+		try:
+			session.starttls()
+		except smtplib.SMTPException:
+			_log.error('cannot enable TLS on [%s]', server)
 		session.ehlo()
 		if auth is not None:
 			session.login(auth['user'], auth['password'])
-		refused = session.sendmail(sender, receiver, msg.getvalue())
+		refused = session.sendmail(sender, receiver, email.as_string())
 		session.quit()
 	except smtplib.SMTPException:
 		failed = True
-		_log.exception('cannot send mail')
+		_log.exception('cannot send email')
 		gmLog2.log_stack_trace()
 
-	msg.close()
 	if len(refused) > 0:
 		_log.error("refused recipients: %s" % refused)
 
@@ -414,6 +447,25 @@ def send_mail(sender=None, receiver=None, message=None, server=None, auth=None,
 		return False
 
 	return True
+
+#---------------------------------------------------------------------------
+def compose_and_send_email(sender=None, receiver=None, message=None, server=None, auth=None, debug=False, subject=None, attachments=None):
+	email = compose_email (
+		sender = sender,
+		receiver = receiver,
+		message = message,
+		subject = subject,
+		files2attach = attachments
+	)
+	return send_email (
+		sender = sender,
+		receiver = receiver,
+		email = email,
+		server = server,
+		auth = auth,
+		debug = debug
+	)
+
 #===========================================================================
 # main
 #---------------------------------------------------------------------------
@@ -426,22 +478,28 @@ if __name__ == '__main__':
 		sys.exit()
 
 	#-----------------------------------------------------------------------
-	def test_send_mail():
-		msg = u"""
-To: %s
-From: %s
-Subject: gmTools test suite mail
+	def test_compose_email():
+		email = compose_email (
+			message = 'compose_email() test: üü ßß',
+			files2attach = [[sys.argv[2]]]
+		)
+		print(email.as_string())
+		return email
 
-This is a test mail from the gmTools.py module.
-""" % (default_mail_receiver, default_mail_sender)
-		print("mail sending succeeded:", send_mail (
-#			sender = 'abc@xyz.123',
-			receiver = [default_mail_receiver, u'karsten.hilbert@gmx.net'],
-			message = msg,
-			auth = {'user': default_mail_sender, 'password': u'gnumed-at-gmx-net'}, # u'gm/bugs/gmx'
-			debug = True,
-			attachments = [[sys.argv[2]]]
+	#-----------------------------------------------------------------------
+	def test_send_email():
+		email = compose_email (
+			message = 'compose_email() test: üü ßß',
+			files2attach = [[sys.argv[2]]]
+		)
+		print(send_email (
+#			receiver = u'ncq@localhost',
+			email = email,
+#			server = 'localhost',
+			auth = {'user': default_mail_sender, 'password': 'gnumed-at-gmx-net'},
+			debug = True
 		))
+
 	#-----------------------------------------------------------------------
 	def test_check_for_update():
 
@@ -468,14 +526,17 @@ This is a test mail from the gmTools.py module.
 		print(url, "->", dl_name)
 		unzip_dir = unzip_data_pack(dl_name)
 		print("unzipped into", unzip_dir)
+
 	#-----------------------------------------------------------------------
 	def test_browser():
 		success = open_url_in_browser(sys.argv[2])
 		print(success)
 		open_url_in_browser(sys.argv[2], abc=222)
+
 	#-----------------------------------------------------------------------
 	#test_check_for_update()
-	test_send_mail()
+	#test_compose_email()
+	test_send_email()
 	#test_dl_data_pack()
 	#test_browser()
 
