@@ -12,7 +12,6 @@ import logging
 import subprocess
 import shlex
 
-
 _log = logging.getLogger('gm.shell')
 
 #===========================================================================
@@ -21,17 +20,18 @@ def is_cmd_in_path(cmd=None):
 	_log.debug('cmd: [%s]', cmd)
 	dirname = os.path.dirname(cmd)
 	_log.debug('dir: [%s]', dirname)
-	if dirname != u'':
+	if dirname != '':
 		_log.info('command with full or relative path, not searching in PATH for binary')
 		return (None, None)
 
+	#env_paths = str(os.environ['PATH'], encoding = sys.getfilesystemencoding(), errors = 'replace')
 	env_paths = os.environ['PATH']
 	_log.debug('${PATH}: %s', env_paths)
 	for path in env_paths.split(os.pathsep):
-		candidate = os.path.join(path, cmd).encode(sys.getfilesystemencoding())
+		candidate = os.path.join(path, cmd)
 		if os.access(candidate, os.X_OK):
 			_log.debug('found [%s]', candidate)
-			return (True, candidate.decode(sys.getfilesystemencoding()))
+			return (True, candidate)
 		else:
 			_log.debug('not found: %s', candidate)
 
@@ -91,6 +91,7 @@ def is_executable_by_wine(cmd=None):
 
 	_log.warning('Windows path [%s] not verifiable under UNIX: %s', exe_path, full_path)
 	return (False, None)
+
 #===========================================================================
 def detect_external_binary(binary=None):
 	"""<binary> is the name of the executable with or without .exe/.bat"""
@@ -136,6 +137,7 @@ def detect_external_binary(binary=None):
 		_log.debug('not running under Windows, not testing .exe/.bat')
 
 	return (False, None)
+
 #===========================================================================
 def find_first_binary(binaries=None):
 	found = False
@@ -150,6 +152,7 @@ def find_first_binary(binaries=None):
 			break
 
 	return (found, binary)
+
 #===========================================================================
 def run_command_in_shell(command=None, blocking=False, acceptable_return_codes=None):
 	"""Runs a command in a subshell via standard-C system().
@@ -200,10 +203,9 @@ def run_command_in_shell(command=None, blocking=False, acceptable_return_codes=N
 		# - all this, of course, only works in shells which support
 		#   detaching jobs with " &" (so, most POSIX shells)
 		if blocking is True:
-			if command[-2:] == ' &':
-				command = command[:-2]
+			command = command.rstrip(' &')
 		elif blocking is False:
-			if command[-2:] != ' &':
+			if not command.strip().endswith('&'):
 				command += ' &'
 
 	_log.info('running shell command >>>%s<<<', command)
@@ -239,6 +241,7 @@ def run_command_in_shell(command=None, blocking=False, acceptable_return_codes=N
 			_log.debug('platform does not support os.WTERMSIG()')
 
 	return exited_normally
+
 #===========================================================================
 def run_first_available_in_shell(binaries=None, args=None, blocking=False, run_last_one_anyway=False, acceptable_return_codes=None):
 
@@ -253,6 +256,58 @@ def run_first_available_in_shell(binaries=None, args=None, blocking=False, run_l
 			return False
 
 	return run_command_in_shell(command = '%s %s' % (binary, args), blocking = blocking, acceptable_return_codes = acceptable_return_codes)
+
+#===========================================================================
+def _log_output(level, stdout=None, stderr=None):
+	lines2log = ['process output:']
+	if stdout is not None:
+		lines2log.extend([ ' STDOUT: %s' % line for line in stdout.split('\n') ])
+	if stderr is not None:
+		lines2log.extend([ ' STDERR: %s' % line for line in stderr.split('\n') ])
+	_log.log(level, '\n'.join(lines2log))
+
+#===========================================================================
+def run_process(cmd_line=None, timeout=None, encoding='utf8', input_data=None, acceptable_return_codes=None, verbose=False):
+	assert (cmd_line is not None), '<cmd_line> must not be None'
+
+	if acceptable_return_codes is None:
+		acceptable_return_codes = [0]
+	_log.info('running: %s' % cmd_line)
+	try:
+		if input_data is None:
+			proc_result = subprocess.run (
+				args = cmd_line,
+				stdin = subprocess.PIPE,
+				stdout = subprocess.PIPE,
+				stderr = subprocess.PIPE,
+				timeout = timeout,
+				encoding = encoding,
+				errors = 'replace'
+			)
+		else:
+			proc_result = subprocess.run (
+				args = cmd_line,
+				input = input_data,
+				stdout = subprocess.PIPE,
+				stderr = subprocess.PIPE,
+				timeout = timeout,
+				encoding = encoding,
+				errors = 'replace'
+			)
+	except (subprocess.TimeoutExpired, FileNotFoundError):
+		_log.exception('there was a problem running external process')
+		return False, -1, ''
+	_log.info('exit code [%s]', proc_result.returncode)
+	if verbose:
+		_log_output(logging.DEBUG, stdout = proc_result.stdout, stderr = proc_result.stderr)
+	if proc_result.returncode not in acceptable_return_codes:
+		_log.error('there was a problem executing the external process')
+		_log.debug('expected one of: %s', acceptable_return_codes)
+		if not verbose:
+			_log_output(logging.ERROR, stdout = proc_result.stdout, stderr = proc_result.stderr)
+		return False, proc_result.returncode, ''
+	return True, proc_result.returncode, proc_result.stdout
+
 #===========================================================================
 # main
 #---------------------------------------------------------------------------
@@ -261,7 +316,7 @@ if __name__ == '__main__':
 	if len(sys.argv) < 2:
 		sys.exit()
 
-	if sys.argv[1] != u'test':
+	if sys.argv[1] != 'test':
 		sys.exit()
 
 	logging.basicConfig(level = logging.DEBUG)
@@ -269,29 +324,29 @@ if __name__ == '__main__':
 	def test_detect_external_binary():
 		found, path = detect_external_binary(binary = sys.argv[2])
 		if found:
-			print "found as:", path
+			print("found as:", path)
 		else:
-			print sys.argv[2], "not found"
+			print(sys.argv[2], "not found")
 	#---------------------------------------------------------
 	def test_run_command_in_shell():
-		print "-------------------------------------"
-		print "running:", sys.argv[2]
+		print("-------------------------------------")
+		print("running:", sys.argv[2])
 		if run_command_in_shell(command=sys.argv[2], blocking=False):
-			print "-------------------------------------"
-			print "success"
+			print("-------------------------------------")
+			print("success")
 		else:
-			print "-------------------------------------"
-			print "failure, consult log"
+			print("-------------------------------------")
+			print("failure, consult log")
 	#---------------------------------------------------------
 	def test_is_cmd_in_path():
-		print is_cmd_in_path(cmd = sys.argv[2])
+		print(is_cmd_in_path(cmd = sys.argv[2]))
 	#---------------------------------------------------------
 	def test_is_executable_by_wine():
-		print is_executable_by_wine(cmd = sys.argv[2])
+		print(is_executable_by_wine(cmd = sys.argv[2]))
 	#---------------------------------------------------------
-	test_run_command_in_shell()
+	#test_run_command_in_shell()
 	#test_detect_external_binary()
-	#test_is_cmd_in_path()
+	test_is_cmd_in_path()
 	#test_is_executable_by_wine()
 
 #===========================================================================
