@@ -14,7 +14,6 @@ import re, string, sys, time, datetime as pyDT, logging
 
 
 # 3rd party
-import mx.DateTime as mxDT
 import wx
 try:
 	import wx.calendar as wxcal
@@ -117,7 +116,7 @@ class cIntervalPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	def _get_data_tooltip(self):
 		intv = self.GetData()
 		if intv is None:
-			return u''
+			return ''
 		return gmDateTime.format_interval (
 			interval = intv,
 			accuracy_wanted = self.display_accuracy
@@ -132,14 +131,14 @@ class cIntervalPhraseWheel(gmPhraseWheel.cPhraseWheel):
 			return
 
 		if value is None:
-			value = u''
+			value = ''
 
 		super(cIntervalPhraseWheel, self).SetValue(value)
 	#--------------------------------------------------------
-	def SetText(self, value=u'', data=None, suppress_smarts=False):
+	def SetText(self, value='', data=None, suppress_smarts=False):
 
 		if data is not None:
-			if value.strip() == u'':
+			if value.strip() == '':
 				value = gmDateTime.format_interval (
 					interval = data,
 					accuracy_wanted = self.display_accuracy
@@ -149,7 +148,7 @@ class cIntervalPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	#--------------------------------------------------------
 	def SetData(self, data=None):
 		if data is None:
-			super(cIntervalPhraseWheel, self).SetText(u'', None)
+			super(cIntervalPhraseWheel, self).SetText('', None)
 			return
 
 		value = gmDateTime.format_interval (
@@ -201,6 +200,7 @@ class cCalendarDatePickerDlg(wx.Dialog):
 		cal.Bind(wx.EVT_KEY_DOWN, self.__on_key_down)
 		cal.SetFocus()
 		self.cal = cal
+
 	#-----------------------------------------------------------
 	def __on_key_down(self, evt):
 		code = evt.KeyCode
@@ -306,7 +306,7 @@ class cDateMatchProvider(gmMatchProvider.cMatchProvider):
 #					)
 #					lbl = gmDateTime.pydt_strftime(date, format = '%Y-%m-%d', accuracy = gmDateTime.acc_days)
 #					matches = (True, [{'data': date, 'label': lbl}])
-#		dlg.Destroy()
+#		dlg.DestroyLater()
 #
 #		return matches
 
@@ -321,6 +321,9 @@ class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 		self.phrase_separators = None
 
 		self.static_tooltip_extra = _('<ALT-C/K>: pick from (c/k)alendar')
+
+		self.__weekday_keys = [wx.WXK_F1, wx.WXK_F2, wx.WXK_F3, wx.WXK_F4, wx.WXK_F5, wx.WXK_F6, wx.WXK_F7]
+
 	#--------------------------------------------------------
 	# internal helpers
 	#--------------------------------------------------------
@@ -339,7 +342,7 @@ class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 		dlg.CentreOnScreen()
 		decision = dlg.ShowModal()
 		date = dlg.cal.Date
-		dlg.Destroy()
+		dlg.DestroyLater()
 
 		if decision != wx.ID_OK:
 			return
@@ -359,6 +362,13 @@ class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 		val = gmDateTime.pydt_strftime(date, format = '%Y-%m-%d', accuracy = gmDateTime.acc_days)
 		self.SetText(value = val, data = date, suppress_smarts = True)
 
+	#--------------------------------------------------------
+	def __pick_from_weekday(self, weekday):
+		self.is_valid_timestamp(empty_is_valid = True)
+		target_date = gmDateTime.get_date_of_weekday_in_week_of_date(weekday, base_dt = self.date)
+		val = gmDateTime.pydt_strftime(target_date, format = '%Y-%m-%d', accuracy = gmDateTime.acc_days)
+		self.SetText(value = val, data = target_date, suppress_smarts = True)
+
 	#--------------------------------------------------------
 	# phrasewheel internal API
 	#--------------------------------------------------------
@@ -383,30 +393,43 @@ class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	#--------------------------------------------------------
 	def _on_key_down(self, event):
 
+		if event.GetUnicodeKey() == wx.WXK_NONE:
+			key = event.GetKeyCode()
+			if key in self.__weekday_keys:
+				self.__pick_from_weekday(self.__weekday_keys.index(key))
+				return
+
 		# <ALT-C> / <ALT-K> -> calendar
 		if event.AltDown() is True:
-			char = unichr(event.GetUnicodeKey())
-			if char in u'ckCK':
+			char = chr(event.GetUnicodeKey())
+			if char in 'ckCK':
 				self.__pick_from_calendar()
 				return
 
-		super(cDateInputPhraseWheel, self)._on_key_down(event)
+		super()._on_key_down(event)
 
 	#--------------------------------------------------------
 	def _get_data_tooltip(self):
 		if len(self._data) == 0:
-			return u''
+			return ''
 
 		date = self.GetData()
 		# if match provider only provided completions
 		# but not a full date with it
 		if date is None:
-			return u''
+			return ''
 
-		return gmDateTime.pydt_strftime (
-			date,
-			format = '%A, %d. %B %Y (%x)',
-			accuracy = gmDateTime.acc_days
+		now = gmDateTime.pydt_now_here()
+		if date > now:
+			intv = date - now
+			template = _('%s\n (a %s in %s)')
+		else:
+			intv = now - date
+			template = _('%s\n (a %s %s ago)')
+		return template % (
+			gmDateTime.pydt_strftime(date, format = '%B %d %Y -- %c', accuracy = gmDateTime.acc_days),
+			gmDateTime.pydt_strftime(date, format = '%A', accuracy = gmDateTime.acc_days),
+			gmDateTime.format_interval(interval = intv, accuracy_wanted = gmDateTime.acc_days, verbose = True)
 		)
 
 	#--------------------------------------------------------
@@ -425,12 +448,12 @@ class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 			return
 
 		if value is None:
-			value = u''
+			value = ''
 
-		super(self.__class__, self).SetValue(value)
+		super().SetValue(value)
 
 	#--------------------------------------------------------
-	def SetText(self, value=u'', data=None, suppress_smarts=False):
+	def SetText(self, value='', data=None, suppress_smarts=False):
 
 		if data is not None:
 			if isinstance(data, gmDateTime.cFuzzyTimestamp):
@@ -440,15 +463,15 @@ class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 					second = 11,
 					microsecond = 111111
 				)
-			if value.strip() == u'':
+			if value.strip() == '':
 				value = gmDateTime.pydt_strftime(data, format = '%Y-%m-%d', accuracy = gmDateTime.acc_days)
 
-		super(self.__class__, self).SetText(value = value, data = data, suppress_smarts = suppress_smarts)
+		super().SetText(value = value, data = data, suppress_smarts = suppress_smarts)
 
 	#--------------------------------------------------------
 	def SetData(self, data=None):
 		if data is None:
-			gmPhraseWheel.cPhraseWheel.SetText(self, u'', None)
+			gmPhraseWheel.cPhraseWheel.SetText(self, '', None)
 			return
 		self.SetText(data = data)
 
@@ -460,13 +483,13 @@ class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 		return super(self.__class__, self).GetData()
 
 	#--------------------------------------------------------
-	def is_valid_timestamp(self, allow_empty=True):
+	def is_valid_timestamp(self, empty_is_valid=True):
 		if len(self._data) > 0:
 			self.display_as_valid(True)
 			return True
 
-		if self.GetValue().strip() == u'':
-			if allow_empty:
+		if self.GetValue().strip() == '':
+			if empty_is_valid:
 				self.display_as_valid(True)
 				return True
 			else:
@@ -474,7 +497,7 @@ class cDateInputPhraseWheel(gmPhraseWheel.cPhraseWheel):
 				return False
 
 		# skip showing calendar on '*' from here
-		if self.GetValue().strip() == u'*':
+		if self.GetValue().strip() == '*':
 			self.display_as_valid(False)
 			return False
 
@@ -572,6 +595,9 @@ class cFuzzyTimestampInput(gmPhraseWheel.cPhraseWheel):
 		self.selection_only = True
 		self.selection_only_error_msg = _('Cannot interpret input as timestamp.')
 		self.display_accuracy = None
+
+		self.__weekday_keys = [wx.WXK_F1, wx.WXK_F2, wx.WXK_F3, wx.WXK_F4, wx.WXK_F5, wx.WXK_F6, wx.WXK_F7]
+
 	#--------------------------------------------------------
 	# internal helpers
 	#--------------------------------------------------------
@@ -579,12 +605,22 @@ class cFuzzyTimestampInput(gmPhraseWheel.cPhraseWheel):
 		if val is None:
 			val = self.GetValue()
 		val = val.strip()
-		if val == u'':
+		if val == '':
 			return None
 		success, matches = self.matcher.getMatchesByPhrase(val)
 		if len(matches) == 1:
 			return matches[0]['data']
 		return None
+
+	#--------------------------------------------------------
+	def __pick_from_weekday(self, weekday):
+		self.is_valid_timestamp(empty_is_valid = True)
+		dt = self.GetData()
+		if dt is not None:
+			dt = dt.timestamp
+		target_date = gmDateTime.get_date_of_weekday_in_week_of_date(weekday, base_dt = dt)
+		self.SetText(data = target_date, suppress_smarts = True)
+
 	#--------------------------------------------------------
 	# phrasewheel internal API
 	#--------------------------------------------------------
@@ -599,39 +635,93 @@ class cFuzzyTimestampInput(gmPhraseWheel.cPhraseWheel):
 
 		# let the base class do its thing
 		gmPhraseWheel.cPhraseWheel._on_lose_focus(self, event)
+
 	#--------------------------------------------------------
 	def _picklist_item2display_string(self, item=None):
 		data = item['data']
 		if data is not None:
 			return data.format_accurately(accuracy = self.display_accuracy)
 		return item['field_label']
+
+	#--------------------------------------------------------
+	def _on_key_down(self, event):
+
+		if event.GetUnicodeKey() == wx.WXK_NONE:
+			key = event.GetKeyCode()
+			if key in self.__weekday_keys:
+				self.__pick_from_weekday(self.__weekday_keys.index(key))
+				return
+
+#		# <ALT-C> / <ALT-K> -> calendar
+#		if event.AltDown() is True:
+#			char = chr(event.GetUnicodeKey())
+#			if char in 'ckCK':
+#				self.__pick_from_calendar()
+#				return
+
+		super()._on_key_down(event)
+
+	#--------------------------------------------------------
+	def _get_data_tooltip(self):
+		if len(self._data) == 0:
+			return ''
+
+		date = self.GetData()
+		# if match provider only provided completions
+		# but not a full date with it
+		if date is None:
+			return ''
+		ts = date.timestamp
+		now = gmDateTime.pydt_now_here()
+		if ts > now:
+			intv = ts - now
+			template = _('%s\n %s\n in %s')
+		else:
+			intv = now - ts
+			template = _('%s\n%s\n%s ago')
+		txt = template % (
+			date.format_accurately(self.display_accuracy),
+			gmDateTime.pydt_strftime (
+				ts,
+				format = '%A, %B-%d %Y (%c)',
+			),
+			gmDateTime.format_interval (
+				interval = intv,
+				accuracy_wanted = gmDateTime.acc_days,
+				verbose = True
+			)
+		)
+		return txt
+
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
-	def SetText(self, value=u'', data=None, suppress_smarts=False):
+	def SetText(self, value='', data=None, suppress_smarts=False):
 
 		if data is not None:
 			if isinstance(data, pyDT.datetime):
-				data = gmDateTime.cFuzzyTimestamp(timestamp=data)
-			if value.strip() == u'':
+				data = gmDateTime.cFuzzyTimestamp(timestamp = data)
+			if value.strip() == '':
 				value = data.format_accurately(accuracy = self.display_accuracy)
 
-		gmPhraseWheel.cPhraseWheel.SetText(self, value = value, data = data, suppress_smarts = suppress_smarts)
+		super().SetText(value = value, data = data, suppress_smarts = suppress_smarts)
+
 	#--------------------------------------------------------
 	def SetData(self, data=None):
 		if data is None:
-			gmPhraseWheel.cPhraseWheel.SetText(self, u'', None)
+			gmPhraseWheel.cPhraseWheel.SetText(self, '', None)
 		else:
 			if isinstance(data, pyDT.datetime):
 				data = gmDateTime.cFuzzyTimestamp(timestamp=data)
 			gmPhraseWheel.cPhraseWheel.SetText(self, value = data.format_accurately(accuracy = self.display_accuracy), data = data)
+
 	#--------------------------------------------------------
 	def is_valid_timestamp(self, empty_is_valid=True):
 		if self.GetData() is not None:
 			return True
 
 		# skip empty value
-		if self.GetValue().strip() == u'':
+		if self.GetValue().strip() == '':
 			if empty_is_valid:
 				return True
 			return False
@@ -647,6 +737,7 @@ class cFuzzyTimestampInput(gmPhraseWheel.cPhraseWheel):
 		)
 
 		return True
+
 #==================================================
 # main
 #--------------------------------------------------
@@ -669,14 +760,14 @@ if __name__ == '__main__':
 		mp.setThresholds(aWord = 998, aSubstring = 999)
 		val = None
 		while val != 'exit':
-			print "************************************"
-			val = raw_input('Enter date fragment ("exit" to quit): ')
+			print("************************************")
+			val = input('Enter date fragment ("exit" to quit): ')
 			found, matches = mp.getMatches(aFragment=val)
 			for match in matches:
 				#print match
-				print match['label']
-				print match['data']
-				print "---------------"
+				print(match['label'])
+				print(match['data'])
+				print("---------------")
 	#--------------------------------------------------------
 	def test_fuzzy_picker():
 		app = wx.PyWidgetTester(size = (300, 40))
