@@ -50,12 +50,6 @@ from Gnumed.wxpython import gmGuiHelpers
 # constant defs
 _log = logging.getLogger('gm.ui')
 
-
-try:
-	_('dummy-no-need-to-translate-but-make-epydoc-happy')
-except NameError:
-	_ = lambda x:x
-
 #============================================================
 # image tags related widgets
 #------------------------------------------------------------
@@ -74,16 +68,18 @@ def edit_tag_image(parent=None, tag_image=None, single_entry=False):
 			)
 			return False
 
-	ea = cTagImageEAPnl(parent = parent, id = -1)
+	ea = cTagImageEAPnl(parent, -1)
 	ea.data = tag_image
 	ea.mode = gmTools.coalesce(tag_image, 'new', 'edit')
-	dlg = gmEditArea.cGenericEditAreaDlg2(parent = parent, id = -1, edit_area = ea, single_entry = single_entry)
+	dlg = gmEditArea.cGenericEditAreaDlg2(parent, -1, edit_area = ea, single_entry = single_entry)
 	dlg.SetTitle(gmTools.coalesce(tag_image, _('Adding new tag'), _('Editing tag')))
 	if dlg.ShowModal() == wx.ID_OK:
-		dlg.Destroy()
+		dlg.DestroyLater()
 		return True
-	dlg.Destroy()
+
+	dlg.DestroyLater()
 	return False
+
 #------------------------------------------------------------
 def manage_tag_images(parent=None):
 
@@ -92,10 +88,10 @@ def manage_tag_images(parent=None):
 
 	#------------------------------------------------------------
 	def go_to_openclipart_org(tag_image):
-		gmNetworkTools.open_url_in_browser(url = u'http://www.openclipart.org')
-		gmNetworkTools.open_url_in_browser(url = u'http://commons.wikimedia.org/wiki/Category:Symbols_of_disabilities')
-		gmNetworkTools.open_url_in_browser(url = u'http://www.duckduckgo.com')
-		gmNetworkTools.open_url_in_browser(url = u'http://images.google.com')
+		gmNetworkTools.open_url_in_browser(url = 'http://www.openclipart.org')
+		gmNetworkTools.open_url_in_browser(url = 'http://commons.wikimedia.org/wiki/Category:Symbols_of_disabilities')
+		gmNetworkTools.open_url_in_browser(url = 'http://www.duckduckgo.com')
+		gmNetworkTools.open_url_in_browser(url = 'http://images.google.com')
 		return True
 
 	#------------------------------------------------------------
@@ -112,11 +108,11 @@ def manage_tag_images(parent=None):
 
 	#------------------------------------------------------------
 	def refresh(lctrl):
-		tags = gmDemographicRecord.get_tag_images(order_by = u'l10n_description')
+		tags = gmDemographicRecord.get_tag_images(order_by = 'l10n_description')
 		items = [ [
 			t['l10n_description'],
-			gmTools.bool2subst(t['is_in_use'], u'X', u''),
-			u'%s' % t['size'],
+			gmTools.bool2subst(t['is_in_use'], 'X', ''),
+			'%s' % t['size'],
 			t['pk_tag_image']
 		] for t in tags ]
 		lctrl.set_string_items(items)
@@ -130,7 +126,7 @@ def manage_tag_images(parent=None):
 		parent = parent,
 		msg = msg,
 		caption = _('Showing tags with images.'),
-		columns = [_('Tag name'), _('In use'), _('Image size'), u'#'],
+		columns = [_('Tag name'), _('In use'), _('Image size'), '#'],
 		single_selection = True,
 		new_callback = edit,
 		edit_callback = edit,
@@ -140,6 +136,7 @@ def manage_tag_images(parent=None):
 	)
 
 	return tag
+
 #------------------------------------------------------------
 from Gnumed.wxGladeWidgets import wxgTagImageEAPnl
 
@@ -162,6 +159,7 @@ class cTagImageEAPnl(wxgTagImageEAPnl.wxgTagImageEAPnl, gmEditArea.cGenericEditA
 			self.mode = 'edit'
 
 		self.__selected_image_file = None
+
 	#----------------------------------------------------------------
 	# generic Edit Area mixin API
 	#----------------------------------------------------------------
@@ -169,10 +167,10 @@ class cTagImageEAPnl(wxgTagImageEAPnl.wxgTagImageEAPnl, gmEditArea.cGenericEditA
 
 		valid = True
 
-		if self.mode == u'new':
+		if self.mode == 'new':
 			if self.__selected_image_file is None:
 				valid = False
-				gmDispatcher.send(signal = 'statustext', msg = _('Must pick an image file for a new tag.'), beep = True)
+				self.StatusText = _('Must pick an image file for a new tag.')
 				self._BTN_pick_image.SetFocus()
 
 		if self.__selected_image_file is not None:
@@ -181,10 +179,10 @@ class cTagImageEAPnl(wxgTagImageEAPnl.wxgTagImageEAPnl, gmEditArea.cGenericEditA
 			except Exception:
 				valid = False
 				self.__selected_image_file = None
-				gmDispatcher.send(signal = 'statustext', msg = _('Cannot open the image file [%s].') % self.__selected_image_file, beep = True)
+				self.StatusText = _('Cannot open the image file [%s].')
 				self._BTN_pick_image.SetFocus()
 
-		if self._TCTRL_description.GetValue().strip() == u'':
+		if self._TCTRL_description.GetValue().strip() == '':
 			valid = False
 			self.display_tctrl_as_valid(self._TCTRL_description, False)
 			self._TCTRL_description.SetFocus()
@@ -233,9 +231,9 @@ class cTagImageEAPnl(wxgTagImageEAPnl.wxgTagImageEAPnl, gmEditArea.cGenericEditA
 		return True
 	#----------------------------------------------------------------
 	def _refresh_as_new(self):
-		self._TCTRL_description.SetValue(u'')
-		self._TCTRL_filename.SetValue(u'')
-		self._BMP_image.SetBitmap(bitmap = wx.EmptyBitmapRGBA(100, 100, red=0, green=0, blue=0, alpha=0))
+		self._TCTRL_description.SetValue('')
+		self._TCTRL_filename.SetValue('')
+		self._BMP_image.SetBitmap(bitmap = wx.Bitmap(wx.Image(100, 100, clear = True)))
 
 		self.__selected_image_file = None
 
@@ -246,10 +244,10 @@ class cTagImageEAPnl(wxgTagImageEAPnl.wxgTagImageEAPnl, gmEditArea.cGenericEditA
 	#----------------------------------------------------------------
 	def _refresh_from_existing(self):
 		self._TCTRL_description.SetValue(self.data['l10n_description'])
-		self._TCTRL_filename.SetValue(gmTools.coalesce(self.data['filename'], u''))
+		self._TCTRL_filename.SetValue(gmTools.coalesce(self.data['filename'], ''))
 		fname = self.data.export_image2file()
 		if fname is None:
-			self._BMP_image.SetBitmap(bitmap = wx.EmptyBitmapRGBA(100, 100, red=0, green=0, blue=0, alpha=0))
+			self._BMP_image.SetBitmap(bitmap = wx.Bitmap(wx.Image(100, 100, clear = True)))
 		else:
 			self._BMP_image.SetBitmap(bitmap = gmGuiHelpers.file2scaled_image(filename = fname, height = 100))
 
@@ -281,7 +279,7 @@ def select_patient_tags(parent=None, patient=None):
 		tags = patient.tags
 		items = [ [
 			t['l10n_description'],
-			gmTools.coalesce(t['comment'], u'')
+			gmTools.coalesce(t['comment'], '')
 		] for t in tags ]
 		lctrl.set_string_items(items)
 		#lctrl.set_column_widths(widths = [wx.LIST_AUTOSIZE, wx.LIST_AUTOSIZE])
@@ -337,16 +335,16 @@ class cImageTagPresenterPnl(wxgVisualSoapPresenterPnl.wxgVisualSoapPresenterPnl)
 
 		self.clear()
 
-		for tag in patient.get_tags(order_by = u'l10n_description'):
+		for tag in patient.get_tags(order_by = 'l10n_description'):
 			fname = tag.export_image2file()
 			if fname is None:
 				_log.warning('cannot export image data of tag [%s]', tag['l10n_description'])
 				continue
 			img = gmGuiHelpers.file2scaled_image(filename = fname, height = 20)
 			bmp = wx_genstatbmp.GenStaticBitmap(self, -1, img, style = wx.NO_BORDER)
-			bmp.SetToolTipString(u'%s%s' % (
+			bmp.SetToolTip('%s%s' % (
 				tag['l10n_description'],
-				gmTools.coalesce(tag['comment'], u'', u'\n\n%s')
+				gmTools.coalesce(tag['comment'], '', '\n\n%s')
 			))
 			bmp.tag = tag
 			bmp.Bind(wx.EVT_RIGHT_UP, self._on_bitmap_rightclicked)
@@ -355,6 +353,7 @@ class cImageTagPresenterPnl(wxgVisualSoapPresenterPnl.wxgVisualSoapPresenterPnl)
 			self.__bitmaps.append(bmp)
 
 		self.GetParent().Layout()
+
 	#--------------------------------------------------------
 	def clear(self):
 		while len(self._SZR_bitmaps.GetChildren()) > 0:
@@ -362,7 +361,7 @@ class cImageTagPresenterPnl(wxgVisualSoapPresenterPnl.wxgVisualSoapPresenterPnl)
 #		for child_idx in range(len(self._SZR_bitmaps.GetChildren())):
 #			self._SZR_bitmaps.Detach(child_idx)
 		for bmp in self.__bitmaps:
-			bmp.Destroy()
+			bmp.DestroyLater()
 		self.__bitmaps = []
 	#--------------------------------------------------------
 	# internal helpers
@@ -383,17 +382,17 @@ class cImageTagPresenterPnl(wxgVisualSoapPresenterPnl.wxgVisualSoapPresenterPnl)
 		comment = wx.GetTextFromUser (
 			message = msg,
 			caption = _('Editing tag comment'),
-			default_value = gmTools.coalesce(self.__current_tag['comment'], u''),
+			default_value = gmTools.coalesce(self.__current_tag['comment'], ''),
 			parent = self
 		)
 
-		if comment == u'':
+		if comment == '':
 			return
 
 		if comment.strip() == self.__current_tag['comment']:
 			return
 
-		if comment == u' ':
+		if comment == ' ':
 			self.__current_tag['comment'] = None
 		else:
 			self.__current_tag['comment'] = comment.strip()
@@ -416,10 +415,10 @@ class cKOrganizerSchedulePnl(gmDataMiningWidgets.cPatientListingPnl):
 		kwargs['message'] = _("Today's KOrganizer appointments ...")
 		kwargs['button_defs'] = [
 			{'label': _('Reload'), 'tooltip': _('Reload appointments from KOrganizer')},
-			{'label': u''},
-			{'label': u''},
-			{'label': u''},
-			{'label': u'KOrganizer', 'tooltip': _('Launch KOrganizer')}
+			{'label': ''},
+			{'label': ''},
+			{'label': ''},
+			{'label': 'KOrganizer', 'tooltip': _('Launch KOrganizer')}
 		]
 		gmDataMiningWidgets.cPatientListingPnl.__init__(self, *args, **kwargs)
 
@@ -448,7 +447,7 @@ class cKOrganizerSchedulePnl(gmDataMiningWidgets.cPatientListingPnl):
 		try:
 			csv_file = io.open(self.fname , mode = 'rt', encoding = 'utf8', errors = 'replace')
 		except IOError:
-			gmDispatcher.send(signal = u'statustext', msg = _('Cannot access KOrganizer transfer file [%s]') % self.fname, beep = True)
+			gmDispatcher.send(signal = 'statustext', msg = _('Cannot access KOrganizer transfer file [%s]') % self.fname, beep = True)
 			return
 
 		csv_lines = gmTools.unicode_csv_reader (
@@ -459,8 +458,8 @@ class cKOrganizerSchedulePnl(gmDataMiningWidgets.cPatientListingPnl):
 		self._LCTRL_items.set_columns ([
 			_('Place'),
 			_('Start'),
-			u'',
-			u'',
+			'',
+			'',
 			_('Patient'),
 			_('Comment')
 		])
@@ -491,8 +490,8 @@ def edit_occupation():
 		old_job = curr_jobs[0]['l10n_occupation']
 		update = curr_jobs[0]['modified_when'].strftime('%m/%Y')
 	else:
-		old_job = u''
-		update = u''
+		old_job = ''
+		update = ''
 
 	msg = _(
 		'Please enter the primary occupation of the patient.\n'
@@ -508,7 +507,7 @@ def edit_occupation():
 		default_value = old_job,
 		parent = None
 	)
-	if new_job.strip() == u'':
+	if new_job.strip() == '':
 		return
 
 	for job in curr_jobs:
@@ -522,7 +521,7 @@ def edit_occupation():
 class cOccupationPhraseWheel(gmPhraseWheel.cPhraseWheel):
 
 	def __init__(self, *args, **kwargs):
-		query = u"SELECT distinct name, _(name) from dem.occupation where _(name) %(fragment_condition)s"
+		query = "SELECT distinct name, _(name) from dem.occupation where _(name) %(fragment_condition)s"
 		mp = gmMatchProvider.cMatchProvider_SQL2(queries=query)
 		mp.setThresholds(1, 3, 5)
 		gmPhraseWheel.cPhraseWheel.__init__ (
@@ -530,7 +529,7 @@ class cOccupationPhraseWheel(gmPhraseWheel.cPhraseWheel):
 			*args,
 			**kwargs
 		)
-		self.SetToolTipString(_("Type or select an occupation."))
+		self.SetToolTip(_("Type or select an occupation."))
 		self.capitalisation_mode = gmTools.CAPS_FIRST
 		self.matcher = mp
 
@@ -608,7 +607,7 @@ def disable_identity(identity=None):
 class cLastnamePhraseWheel(gmPhraseWheel.cPhraseWheel):
 
 	def __init__(self, *args, **kwargs):
-		query = u"SELECT distinct lastnames, lastnames from dem.names where lastnames %(fragment_condition)s order by lastnames limit 25"
+		query = "SELECT distinct lastnames, lastnames from dem.names where lastnames %(fragment_condition)s order by lastnames limit 25"
 		mp = gmMatchProvider.cMatchProvider_SQL2(queries=query)
 		mp.setThresholds(3, 5, 9)
 		gmPhraseWheel.cPhraseWheel.__init__ (
@@ -616,14 +615,14 @@ class cLastnamePhraseWheel(gmPhraseWheel.cPhraseWheel):
 			*args,
 			**kwargs
 		)
-		self.SetToolTipString(_("Type or select a last name (family name/surname)."))
+		self.SetToolTip(_("Type or select a last name (family name/surname)."))
 		self.capitalisation_mode = gmTools.CAPS_NAMES
 		self.matcher = mp
 #------------------------------------------------------------
 class cFirstnamePhraseWheel(gmPhraseWheel.cPhraseWheel):
 
 	def __init__(self, *args, **kwargs):
-		query = u"""
+		query = """
 			(SELECT distinct firstnames, firstnames from dem.names where firstnames %(fragment_condition)s order by firstnames limit 20)
 				union
 			(SELECT distinct name, name from dem.name_gender_map where name %(fragment_condition)s order by name limit 20)"""
@@ -634,14 +633,14 @@ class cFirstnamePhraseWheel(gmPhraseWheel.cPhraseWheel):
 			*args,
 			**kwargs
 		)
-		self.SetToolTipString(_("Type or select a first name (forename/Christian name/given name)."))
+		self.SetToolTip(_("Type or select a first name (forename/Christian name/given name)."))
 		self.capitalisation_mode = gmTools.CAPS_NAMES
 		self.matcher = mp
 #------------------------------------------------------------
 class cNicknamePhraseWheel(gmPhraseWheel.cPhraseWheel):
 
 	def __init__(self, *args, **kwargs):
-		query = u"""
+		query = """
 			(SELECT distinct preferred, preferred from dem.names where preferred %(fragment_condition)s order by preferred limit 20)
 				union
 			(SELECT distinct firstnames, firstnames from dem.names where firstnames %(fragment_condition)s order by firstnames limit 20)
@@ -654,7 +653,7 @@ class cNicknamePhraseWheel(gmPhraseWheel.cPhraseWheel):
 			*args,
 			**kwargs
 		)
-		self.SetToolTipString(_("Type or select an alias (nick name, preferred name, call name, warrior name, artist name)."))
+		self.SetToolTip(_("Type or select an alias (nick name, preferred name, call name, warrior name, artist name)."))
 		# nicknames CAN start with lower case !
 		#self.capitalisation_mode = gmTools.CAPS_NAMES
 		self.matcher = mp
@@ -662,7 +661,7 @@ class cNicknamePhraseWheel(gmPhraseWheel.cPhraseWheel):
 class cTitlePhraseWheel(gmPhraseWheel.cPhraseWheel):
 
 	def __init__(self, *args, **kwargs):
-		query = u"SELECT distinct title, title from dem.identity where title %(fragment_condition)s"
+		query = "SELECT distinct title, title from dem.identity where title %(fragment_condition)s"
 		mp = gmMatchProvider.cMatchProvider_SQL2(queries=query)
 		mp.setThresholds(1, 3, 9)
 		gmPhraseWheel.cPhraseWheel.__init__ (
@@ -670,7 +669,7 @@ class cTitlePhraseWheel(gmPhraseWheel.cPhraseWheel):
 			*args,
 			**kwargs
 		)
-		self.SetToolTipString(_("Type or select a title. Note that the title applies to the person, not to a particular name !"))
+		self.SetToolTip(_("Type or select a title. Note that the title applies to the person, not to a particular name !"))
 		self.matcher = mp
 #------------------------------------------------------------
 class cGenderSelectionPhraseWheel(gmPhraseWheel.cPhraseWheel):
@@ -681,7 +680,7 @@ class cGenderSelectionPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	def __init__(self, *args, **kwargs):
 
 		if cGenderSelectionPhraseWheel._gender_map is None:
-			cmd = u"""
+			cmd = """
 				SELECT tag, l10n_label, sort_weight
 				from dem.v_gender_labels
 				order by sort_weight desc"""
@@ -695,7 +694,7 @@ class cGenderSelectionPhraseWheel(gmPhraseWheel.cPhraseWheel):
 					'weight': gender[idx['sort_weight']]
 				}
 
-		mp = gmMatchProvider.cMatchProvider_FixedList(aSeq = cGenderSelectionPhraseWheel._gender_map.values())
+		mp = gmMatchProvider.cMatchProvider_FixedList(aSeq = list(cGenderSelectionPhraseWheel._gender_map.values()))
 		mp.setThresholds(1, 1, 3)
 
 		gmPhraseWheel.cPhraseWheel.__init__(self, *args, **kwargs)
@@ -706,7 +705,7 @@ class cGenderSelectionPhraseWheel(gmPhraseWheel.cPhraseWheel):
 class cExternalIDTypePhraseWheel(gmPhraseWheel.cPhraseWheel):
 
 	def __init__(self, *args, **kwargs):
-		query = u"""
+		query = """
 			SELECT DISTINCT ON (list_label)
 				pk AS data,
 				name AS field_label,
@@ -719,18 +718,18 @@ class cExternalIDTypePhraseWheel(gmPhraseWheel.cPhraseWheel):
 		mp = gmMatchProvider.cMatchProvider_SQL2(queries=query)
 		mp.setThresholds(1, 3, 5)
 		gmPhraseWheel.cPhraseWheel.__init__(self, *args, **kwargs)
-		self.SetToolTipString(_("Enter or select a type for the external ID."))
+		self.SetToolTip(_("Enter or select a type for the external ID."))
 		self.matcher = mp
 	#--------------------------------------------------------
 	def _get_data_tooltip(self):
 		if self.GetData() is None:
 			return None
-		return self._data.values()[0]['list_label']
+		return list(self._data.values())[0]['list_label']
 #------------------------------------------------------------
 class cExternalIDIssuerPhraseWheel(gmPhraseWheel.cPhraseWheel):
 
 	def __init__(self, *args, **kwargs):
-		query = u"""
+		query = """
 SELECT distinct issuer, issuer
 from dem.enum_ext_id_types
 where issuer %(fragment_condition)s
@@ -738,7 +737,7 @@ order by issuer limit 25"""
 		mp = gmMatchProvider.cMatchProvider_SQL2(queries=query)
 		mp.setThresholds(1, 3, 5)
 		gmPhraseWheel.cPhraseWheel.__init__(self, *args, **kwargs)
-		self.SetToolTipString(_("Type or select an ID issuer."))
+		self.SetToolTip(_("Type or select an ID issuer."))
 		self.capitalisation_mode = gmTools.CAPS_FIRST
 		self.matcher = mp
 #------------------------------------------------------------
@@ -756,7 +755,7 @@ class cExternalIDEditAreaPnl(wxgExternalIDEditAreaPnl.wxgExternalIDEditAreaPnl,
 		try:
 			data = kwargs['external_id']
 			del kwargs['external_id']
-		except:
+		except Exception:
 			data = None
 
 		wxgExternalIDEditAreaPnl.wxgExternalIDEditAreaPnl.__init__(self, *args, **kwargs)
@@ -782,14 +781,14 @@ class cExternalIDEditAreaPnl(wxgExternalIDEditAreaPnl.wxgExternalIDEditAreaPnl,
 		# do not test .GetData() because adding external
 		# IDs will create types as necessary
 		#if self._PRW_type.GetData() is None:
-		if self._PRW_type.GetValue().strip() == u'':
+		if self._PRW_type.GetValue().strip() == '':
 			validity = False
 			self._PRW_type.display_as_valid(False)
 			self._PRW_type.SetFocus()
 		else:
 			self._PRW_type.display_as_valid(True)
 
-		if self._TCTRL_value.GetValue().strip() == u'':
+		if self._TCTRL_value.GetValue().strip() == '':
 			validity = False
 			self.display_tctrl_as_valid(tctrl = self._TCTRL_value, valid = False)
 		else:
@@ -802,8 +801,8 @@ class cExternalIDEditAreaPnl(wxgExternalIDEditAreaPnl.wxgExternalIDEditAreaPnl,
 		data['pk_type'] = None
 		data['name'] = self._PRW_type.GetValue().strip()
 		data['value'] = self._TCTRL_value.GetValue().strip()
-		data['issuer'] = gmTools.none_if(self._PRW_issuer.GetValue().strip(), u'')
-		data['comment'] = gmTools.none_if(self._TCTRL_comment.GetValue().strip(), u'')
+		data['issuer'] = gmTools.none_if(self._PRW_issuer.GetValue().strip(), '')
+		data['comment'] = gmTools.none_if(self._TCTRL_comment.GetValue().strip(), '')
 
 		self.id_holder.add_external_id (
 			type_name = data['name'],
@@ -818,8 +817,8 @@ class cExternalIDEditAreaPnl(wxgExternalIDEditAreaPnl.wxgExternalIDEditAreaPnl,
 	def _save_as_update(self):
 		self.data['name'] = self._PRW_type.GetValue().strip()
 		self.data['value'] = self._TCTRL_value.GetValue().strip()
-		self.data['issuer'] = gmTools.none_if(self._PRW_issuer.GetValue().strip(), u'')
-		self.data['comment'] = gmTools.none_if(self._TCTRL_comment.GetValue().strip(), u'')
+		self.data['issuer'] = gmTools.none_if(self._PRW_issuer.GetValue().strip(), '')
+		self.data['comment'] = gmTools.none_if(self._TCTRL_comment.GetValue().strip(), '')
 
 		self.id_holder.update_external_id (
 			pk_id = self.data['pk_id'],
@@ -832,10 +831,10 @@ class cExternalIDEditAreaPnl(wxgExternalIDEditAreaPnl.wxgExternalIDEditAreaPnl,
 		return True
 	#----------------------------------------------------------------
 	def _refresh_as_new(self):
-		self._PRW_type.SetText(value = u'', data = None)
-		self._TCTRL_value.SetValue(u'')
-		self._PRW_issuer.SetText(value = u'', data = None)
-		self._TCTRL_comment.SetValue(u'')
+		self._PRW_type.SetText(value = '', data = None)
+		self._TCTRL_value.SetValue('')
+		self._PRW_issuer.SetText(value = '', data = None)
+		self._TCTRL_comment.SetValue('')
 	#----------------------------------------------------------------
 	def _refresh_as_new_from_existing(self):
 		self._refresh_as_new()
@@ -845,7 +844,7 @@ class cExternalIDEditAreaPnl(wxgExternalIDEditAreaPnl.wxgExternalIDEditAreaPnl,
 		self._PRW_type.SetText(value = self.data['name'], data = self.data['pk_type'])
 		self._TCTRL_value.SetValue(self.data['value'])
 		self._PRW_issuer.SetText(self.data['issuer'])
-		self._TCTRL_comment.SetValue(gmTools.coalesce(self.data['comment'], u''))
+		self._TCTRL_comment.SetValue(gmTools.coalesce(self.data['comment'], ''))
 	#----------------------------------------------------------------
 	# internal helpers
 	#----------------------------------------------------------------
@@ -858,7 +857,7 @@ class cExternalIDEditAreaPnl(wxgExternalIDEditAreaPnl.wxgExternalIDEditAreaPnl,
 		if pk_curr_type is None:
 			return True
 		rows, idx = gmPG2.run_ro_queries(queries = [{
-			'cmd': u"SELECT issuer FROM dem.enum_ext_id_types WHERE pk = %s",
+			'cmd': "SELECT issuer FROM dem.enum_ext_id_types WHERE pk = %s",
 			'args': [pk_curr_type]
 		}])
 		if len(rows) == 0:
@@ -886,7 +885,7 @@ def _empty_dob_allowed():
 def _validate_dob_field(dob_prw):
 
 	# valid timestamp ?
-	if dob_prw.is_valid_timestamp(allow_empty = False):			# properly colors the field
+	if dob_prw.is_valid_timestamp(empty_is_valid = False):			# properly colors the field
 		dob = dob_prw.date
 		# but year also usable ?
 		if (dob.year > 1899) and (dob < gmDateTime.pydt_now_here()):
@@ -919,9 +918,9 @@ def _validate_dob_field(dob_prw):
 		return False
 
 	# invalid timestamp but not empty
-	if dob_prw.GetValue().strip() != u'':
+	if dob_prw.GetValue().strip() != '':
 		dob_prw.display_as_valid(False)
-		gmDispatcher.send(signal = u'statustext', msg = _('Invalid date of birth.'))
+		gmDispatcher.send(signal = 'statustext', msg = _('Invalid date of birth.'))
 		dob_prw.SetFocus()
 		return False
 
@@ -934,7 +933,7 @@ def _validate_tob_field(ctrl):
 
 	val = ctrl.GetValue().strip()
 
-	if val == u'':
+	if val == '':
 		return True
 
 	converted, hours = gmTools.input2int(val[:2], 0, 23)
@@ -995,8 +994,8 @@ class cIdentityEAPnl(wxgIdentityEAPnl.wxgIdentityEAPnl, gmEditArea.cGenericEditA
 			has_error = True
 			self.display_ctrl_as_valid(ctrl = self._TCTRL_tob, valid = False)
 
-		if not self._PRW_dod.is_valid_timestamp(allow_empty = True):
-			gmDispatcher.send(signal = u'statustext', msg = _('Invalid date of death.'))
+		if not self._PRW_dod.is_valid_timestamp(empty_is_valid = True):
+			self.StatusText = _('Invalid date of death.')
 			self._PRW_dod.SetFocus()
 			has_error = True
 
@@ -1008,7 +1007,7 @@ class cIdentityEAPnl(wxgIdentityEAPnl.wxgIdentityEAPnl, gmEditArea.cGenericEditA
 	#----------------------------------------------------------------
 	def _save_as_update(self):
 
-		if self._PRW_dob.GetValue().strip() == u'':
+		if self._PRW_dob.GetValue().strip() == '':
 			if not _empty_dob_allowed():
 				return False
 			self.data['dob'] = None
@@ -1016,14 +1015,14 @@ class cIdentityEAPnl(wxgIdentityEAPnl.wxgIdentityEAPnl, gmEditArea.cGenericEditA
 			self.data['dob'] = self._PRW_dob.GetData()
 		self.data['dob_is_estimated'] = self._CHBOX_estimated_dob.GetValue()
 		val = self._TCTRL_tob.GetValue().strip()
-		if val == u'':
+		if val == '':
 			self.data['tob'] = None
 		else:
 			self.data['tob'] = pydt.time(int(val[:2]), int(val[3:5]))
 		self.data['gender'] = self._PRW_gender.GetData()
-		self.data['title'] = gmTools.none_if(self._PRW_title.GetValue().strip(), u'')
+		self.data['title'] = gmTools.none_if(self._PRW_title.GetValue().strip(), '')
 		self.data['deceased'] = self._PRW_dod.GetData()
-		self.data['comment'] = gmTools.none_if(self._TCTRL_comment.GetValue().strip(), u'')
+		self.data['comment'] = gmTools.none_if(self._TCTRL_comment.GetValue().strip(), '')
 
 		self.data.save()
 		return True
@@ -1033,12 +1032,12 @@ class cIdentityEAPnl(wxgIdentityEAPnl.wxgIdentityEAPnl, gmEditArea.cGenericEditA
 	#----------------------------------------------------------------
 	def _refresh_from_existing(self):
 
-		self._LBL_info.SetLabel(u'ID: #%s' % (
+		self._LBL_info.SetLabel('ID: #%s' % (
 			self.data.ID
 			# FIXME: add 'deleted' status
 		))
 		if self.data['dob'] is None:
-			val = u''
+			val = ''
 		else:
 			val = gmDateTime.pydt_strftime (
 				self.data['dob'],
@@ -1048,11 +1047,11 @@ class cIdentityEAPnl(wxgIdentityEAPnl.wxgIdentityEAPnl, gmEditArea.cGenericEditA
 		self._PRW_dob.SetText(value = val, data = self.data['dob'])
 		self._CHBOX_estimated_dob.SetValue(self.data['dob_is_estimated'])
 		if self.data['tob'] is None:
-			self._TCTRL_tob.SetValue(u'')
+			self._TCTRL_tob.SetValue('')
 		else:
 			self._TCTRL_tob.SetValue(self.data['tob'].strftime('%H:%M'))
 		if self.data['deceased'] is None:
-			val = u''
+			val = ''
 		else:
 			val = gmDateTime.pydt_strftime (
 				self.data['deceased'],
@@ -1062,8 +1061,8 @@ class cIdentityEAPnl(wxgIdentityEAPnl.wxgIdentityEAPnl, gmEditArea.cGenericEditA
 		self._PRW_dod.SetText(value = val, data = self.data['deceased'])
 		self._PRW_gender.SetData(self.data['gender'])
 		#self._PRW_ethnicity.SetValue()
-		self._PRW_title.SetText(gmTools.coalesce(self.data['title'], u''))
-		self._TCTRL_comment.SetValue(gmTools.coalesce(self.data['comment'], u''))
+		self._PRW_title.SetText(gmTools.coalesce(self.data['title'], ''))
+		self._TCTRL_comment.SetValue(gmTools.coalesce(self.data['comment'], ''))
 	#----------------------------------------------------------------
 	def _refresh_as_new_from_existing(self):
 		pass
@@ -1106,14 +1105,14 @@ class cPersonNameEAPnl(wxgPersonNameEAPnl.wxgPersonNameEAPnl, gmEditArea.cGeneri
 	def _valid_for_save(self):
 		validity = True
 
-		if self._PRW_lastname.GetValue().strip() == u'':
+		if self._PRW_lastname.GetValue().strip() == '':
 			validity = False
 			self._PRW_lastname.display_as_valid(False)
 			self._PRW_lastname.SetFocus()
 		else:
 			self._PRW_lastname.display_as_valid(True)
 
-		if self._PRW_firstname.GetValue().strip() == u'':
+		if self._PRW_firstname.GetValue().strip() == '':
 			validity = False
 			self._PRW_firstname.display_as_valid(False)
 			self._PRW_firstname.SetFocus()
@@ -1132,24 +1131,23 @@ class cPersonNameEAPnl(wxgPersonNameEAPnl.wxgPersonNameEAPnl, gmEditArea.cGeneri
 			data = self.__identity.add_name(first, last, active)
 		except gmPG2.dbapi.IntegrityError as exc:
 			_log.exception('cannot save new name')
-			exc = gmPG2.make_pg_exception_fields_unicode(exc)
 			gmGuiHelpers.gm_show_error (
 				aTitle = _('Adding name'),
 				aMessage = _(
 					'Cannot add this name to the patient !\n'
 					'\n'
 					' %s'
-				) % exc.u_pgerror
+				) % exc.pgerror
 			)
 			return False
 
 		old_nick = self.__identity['active_name']['preferred']
-		new_nick = gmTools.none_if(self._PRW_nick.GetValue().strip(), u'')
+		new_nick = gmTools.none_if(self._PRW_nick.GetValue().strip(), '')
 		if active:
 			data['preferred'] = gmTools.coalesce(new_nick, old_nick)
 		else:
 			data['preferred'] = new_nick
-		data['comment'] = gmTools.none_if(self._TCTRL_comment.GetValue().strip(), u'')
+		data['comment'] = gmTools.none_if(self._TCTRL_comment.GetValue().strip(), '')
 		data.save()
 
 		self.data = data
@@ -1170,8 +1168,8 @@ class cPersonNameEAPnl(wxgPersonNameEAPnl.wxgPersonNameEAPnl, gmEditArea.cGeneri
 		# editable fields only ?
 		if new_name == current_name:
 			self.data['active_name'] = self._CHBOX_active.GetValue()
-			self.data['preferred'] = gmTools.none_if(self._PRW_nick.GetValue().strip(), u'')
-			self.data['comment'] = gmTools.none_if(self._TCTRL_comment.GetValue().strip(), u'')
+			self.data['preferred'] = gmTools.none_if(self._PRW_nick.GetValue().strip(), '')
+			self.data['comment'] = gmTools.none_if(self._TCTRL_comment.GetValue().strip(), '')
 			self.data.save()
 		# else clone name and update that
 		else:
@@ -1179,45 +1177,43 @@ class cPersonNameEAPnl(wxgPersonNameEAPnl.wxgPersonNameEAPnl, gmEditArea.cGeneri
 				name = self.__identity.add_name(first, last, active)
 			except gmPG2.dbapi.IntegrityError as exc:
 				_log.exception('cannot clone name when editing existing name')
-				exc = gmPG2.make_pg_exception_fields_unicode(exc)
 				gmGuiHelpers.gm_show_error (
 					aTitle = _('Editing name'),
 					aMessage = _(
 						'Cannot clone a copy of this name !\n'
 						'\n'
 						' %s'
-					) % exc.u_pgerror
-#					) % str(exc)
+					) % exc.pgerror
 				)
 				return False
-			name['preferred'] = gmTools.none_if(self._PRW_nick.GetValue().strip(), u'')
-			name['comment'] = gmTools.none_if(self._TCTRL_comment.GetValue().strip(), u'')
+			name['preferred'] = gmTools.none_if(self._PRW_nick.GetValue().strip(), '')
+			name['comment'] = gmTools.none_if(self._TCTRL_comment.GetValue().strip(), '')
 			name.save()
 			self.data = name
 
 		return True
 	#----------------------------------------------------------------
 	def _refresh_as_new(self):
-		self._PRW_firstname.SetText(value = u'', data = None)
-		self._PRW_lastname.SetText(value = u'', data = None)
-		self._PRW_nick.SetText(value = u'', data = None)
-		self._TCTRL_comment.SetValue(u'')
+		self._PRW_firstname.SetText(value = '', data = None)
+		self._PRW_lastname.SetText(value = '', data = None)
+		self._PRW_nick.SetText(value = '', data = None)
+		self._TCTRL_comment.SetValue('')
 		self._CHBOX_active.SetValue(False)
 
 		self._PRW_firstname.SetFocus()
 	#----------------------------------------------------------------
 	def _refresh_as_new_from_existing(self):
 		self._refresh_as_new()
-		self._PRW_firstname.SetText(value = u'', data = None)
-		self._PRW_nick.SetText(gmTools.coalesce(self.data['preferred'], u''))
+		self._PRW_firstname.SetText(value = '', data = None)
+		self._PRW_nick.SetText(gmTools.coalesce(self.data['preferred'], ''))
 
 		self._PRW_lastname.SetFocus()
 	#----------------------------------------------------------------
 	def _refresh_from_existing(self):
 		self._PRW_firstname.SetText(self.data['firstnames'])
 		self._PRW_lastname.SetText(self.data['lastnames'])
-		self._PRW_nick.SetText(gmTools.coalesce(self.data['preferred'], u''))
-		self._TCTRL_comment.SetValue(gmTools.coalesce(self.data['comment'], u''))
+		self._PRW_nick.SetText(gmTools.coalesce(self.data['preferred'], ''))
+		self._TCTRL_comment.SetValue(gmTools.coalesce(self.data['comment'], ''))
 		self._CHBOX_active.SetValue(self.data['active_name'])
 
 		self._TCTRL_comment.SetFocus()
@@ -1260,8 +1256,8 @@ class cPersonNamesManagerPnl(gmListWidgets.cGenericListManagerPnl):
 					gmTools.bool2str(n['active_name'], 'X', ''),
 					n['lastnames'],
 					n['firstnames'],
-					gmTools.coalesce(n['preferred'], u''),
-					gmTools.coalesce(n['comment'], u'')
+					gmTools.coalesce(n['preferred'], ''),
+					gmTools.coalesce(n['comment'], '')
 				] for n in names ]
 		)
 		self._LCTRL_items.set_column_widths()
@@ -1284,9 +1280,9 @@ class cPersonNamesManagerPnl(gmListWidgets.cGenericListManagerPnl):
 		dlg = gmEditArea.cGenericEditAreaDlg2(self, -1, edit_area = ea, single_entry = True)
 		dlg.SetTitle(_('Adding new name'))
 		if dlg.ShowModal() == wx.ID_OK:
-			dlg.Destroy()
+			dlg.DestroyLater()
 			return True
-		dlg.Destroy()
+		dlg.DestroyLater()
 		return False
 	#--------------------------------------------------------
 	def _edit_name(self, name):
@@ -1294,19 +1290,19 @@ class cPersonNamesManagerPnl(gmListWidgets.cGenericListManagerPnl):
 		dlg = gmEditArea.cGenericEditAreaDlg2(self, -1, edit_area = ea, single_entry = True)
 		dlg.SetTitle(_('Editing name'))
 		if dlg.ShowModal() == wx.ID_OK:
-			dlg.Destroy()
+			dlg.DestroyLater()
 			return True
-		dlg.Destroy()
+		dlg.DestroyLater()
 		return False
 	#--------------------------------------------------------
 	def _del_name(self, name):
 
 		if len(self.__identity.get_names()) == 1:
-			gmDispatcher.send(signal = u'statustext', msg = _('Cannot delete the only name of a person.'), beep = True)
+			gmDispatcher.send(signal = 'statustext', msg = _('Cannot delete the only name of a person.'), beep = True)
 			return False
 
 		if name['active_name']:
-			gmDispatcher.send(signal = u'statustext', msg = _('Cannot delete the active name of a person.'), beep = True)
+			gmDispatcher.send(signal = 'statustext', msg = _('Cannot delete the active name of a person.'), beep = True)
 			return False
 
 		go_ahead = gmGuiHelpers.gm_show_question (
@@ -1374,8 +1370,8 @@ class cPersonIDsManagerPnl(gmListWidgets.cGenericListManagerPnl):
 			items = [ [
 					i['name'],
 					i['value'],
-					gmTools.coalesce(i['issuer'], u''),
-					gmTools.coalesce(i['comment'], u'')
+					gmTools.coalesce(i['issuer'], ''),
+					gmTools.coalesce(i['comment'], '')
 				] for i in ids
 			]
 		)
@@ -1398,9 +1394,9 @@ class cPersonIDsManagerPnl(gmListWidgets.cGenericListManagerPnl):
 		dlg = gmEditArea.cGenericEditAreaDlg2(self, -1, edit_area = ea)
 		dlg.SetTitle(_('Adding new external ID'))
 		if dlg.ShowModal() == wx.ID_OK:
-			dlg.Destroy()
+			dlg.DestroyLater()
 			return True
-		dlg.Destroy()
+		dlg.DestroyLater()
 		return False
 	#--------------------------------------------------------
 	def _edit_id(self, ext_id):
@@ -1409,9 +1405,9 @@ class cPersonIDsManagerPnl(gmListWidgets.cGenericListManagerPnl):
 		dlg = gmEditArea.cGenericEditAreaDlg2(self, -1, edit_area = ea, single_entry = True)
 		dlg.SetTitle(_('Editing external ID'))
 		if dlg.ShowModal() == wx.ID_OK:
-			dlg.Destroy()
+			dlg.DestroyLater()
 			return True
-		dlg.Destroy()
+		dlg.DestroyLater()
 		return False
 	#--------------------------------------------------------
 	def _del_id(self, ext_id):
@@ -1509,25 +1505,25 @@ class cPersonSocialNetworkManagerPnl(wxgPersonSocialNetworkManagerPnl.wxgPersonS
 		tt = _('Link another person in this database as the emergency contact:\n\nEnter person name part or identifier and hit <enter>.')
 
 		if self.__identity is None:
-			self._TCTRL_er_contact.SetValue(u'')
+			self._TCTRL_er_contact.SetValue('')
 			self._TCTRL_person.person = None
-			self._TCTRL_person.SetToolTipString(tt)
+			self._TCTRL_person.SetToolTip(tt)
 
-			self._PRW_provider.SetText(value = u'', data = None)
+			self._PRW_provider.SetText(value = '', data = None)
 			return
 
-		self._TCTRL_er_contact.SetValue(gmTools.coalesce(self.__identity['emergency_contact'], u''))
+		self._TCTRL_er_contact.SetValue(gmTools.coalesce(self.__identity['emergency_contact'], ''))
 		if self.__identity['pk_emergency_contact'] is not None:
 			ident = gmPerson.cPerson(aPK_obj = self.__identity['pk_emergency_contact'])
 			self._TCTRL_person.person = ident
-			tt = u'%s\n\n%s\n\n%s' % (
+			tt = '%s\n\n%s\n\n%s' % (
 				tt,
 				ident['description_gender'],
-				u'\n'.join([
-					u'%s: %s%s' % (
+				'\n'.join([
+					'%s: %s%s' % (
 						c['l10n_comm_type'],
 						c['url'],
-						gmTools.bool2subst(c['is_confidential'], _(' (confidential !)'), u'', u'')
+						gmTools.bool2subst(c['is_confidential'], _(' (confidential !)'), '', '')
 					)
 					for c in ident.get_comm_channels()
 				])
@@ -1535,10 +1531,10 @@ class cPersonSocialNetworkManagerPnl(wxgPersonSocialNetworkManagerPnl.wxgPersonS
 		else:
 			self._TCTRL_person.person = None
 
-		self._TCTRL_person.SetToolTipString(tt)
+		self._TCTRL_person.SetToolTip(tt)
 
 		if self.__identity['pk_primary_provider'] is None:
-			self._PRW_provider.SetText(value = u'', data = None)
+			self._PRW_provider.SetText(value = '', data = None)
 		else:
 			self._PRW_provider.SetData(data = self.__identity['pk_primary_provider'])
 
@@ -1562,7 +1558,7 @@ class cPersonSocialNetworkManagerPnl(wxgPersonSocialNetworkManagerPnl.wxgPersonS
 			self.__identity['emergency_contact'] = self._TCTRL_er_contact.GetValue().strip()
 			if self._TCTRL_person.person is not None:
 				self.__identity['pk_emergency_contact'] = self._TCTRL_person.person.ID
-			if self._PRW_provider.GetValue().strip == u'':
+			if self._PRW_provider.GetValue().strip == '':
 				self.__identity['pk_primary_provider'] = None
 			else:
 				self.__identity['pk_primary_provider'] = self._PRW_provider.GetData()
@@ -1699,8 +1695,8 @@ class cPatOccupationsPanel(wx.Panel):
 		PNL_form = wx.Panel(self, -1)
 		# occupation
 		STT_occupation = wx.StaticText(PNL_form, -1, _('Occupation'))
-		self.PRW_occupation = cOccupationPhraseWheel(parent = PNL_form,	id = -1)
-		self.PRW_occupation.SetToolTipString(_("primary occupation of the patient"))
+		self.PRW_occupation = cOccupationPhraseWheel(PNL_form, -1)
+		self.PRW_occupation.SetToolTip(_("primary occupation of the patient"))
 		# known since
 		STT_occupation_updated = wx.StaticText(PNL_form, -1, _('Last updated'))
 		self.TTC_occupation_updated = wx.TextCtrl(PNL_form, -1, style = wx.TE_READONLY)
@@ -1773,8 +1769,8 @@ class cNotebookedPatEditionPanel(wx.Panel, gmRegetMixin.cRegetOnPaintMixin):
 	# event handling
 	#--------------------------------------------------------
 	def __register_interests(self):
-		gmDispatcher.connect(signal = u'pre_patient_unselection', receiver = self._on_pre_patient_unselection)
-		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
+		gmDispatcher.connect(signal = 'pre_patient_unselection', receiver = self._on_pre_patient_unselection)
+		gmDispatcher.connect(signal = 'post_patient_selection', receiver = self._on_post_patient_selection)
 	#--------------------------------------------------------
 	def _on_pre_patient_unselection(self):
 		self.__patient_notebook.identity = None
@@ -1842,7 +1838,7 @@ if __name__ == "__main__":
 	def activate_patient():
 		patient = gmPersonSearch.ask_for_patient()
 		if patient is None:
-			print "No patient. Exiting gracefully..."
+			print("No patient. Exiting gracefully...")
 			sys.exit(0)
 		from Gnumed.wxpython.gmPatSearchWidgets import set_active_patient
 		set_active_patient(patient = patient)
