@@ -16,6 +16,7 @@ import wx
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
 from Gnumed.pycommon import gmDispatcher
+from Gnumed.pycommon.gmTools import decorate_window_title
 
 
 _log = logging.getLogger('gm.ui')
@@ -74,7 +75,7 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		if self._TCTRL_xxx.GetValue().strip() == u'':
 			validity = False
 			self.display_tctrl_as_valid(tctrl = self._TCTRL_xxx, valid = False)
-			self.status_message = _('No entry in field xxx.')
+			self.StatusText = _('No entry in field xxx.')
 			self._TCTRL_xxx.SetFocus()
 		else:
 			self.display_tctrl_as_valid(tctrl = self._TCTRL_xxx, valid = True)
@@ -82,7 +83,7 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		if self._PRW_xxx.GetData() is None:
 			validity = False
 			self._PRW_xxx.display_as_valid(False)
-			self.status_message = _('No entry in field xxx.')
+			self.StatusText = _('No entry in field xxx.')
 			self._PRW_xxx.SetFocus()
 		else:
 			self._PRW_xxx.display_as_valid(True)
@@ -162,7 +163,7 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		self.__data = None
 		self.successful_save_msg = None
 		self.__tctrl_validity_colors = {
-			True: wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW),
+			True: wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOW),
 			False: 'pink'
 		}
 		self._refresh_as_new()
@@ -201,10 +202,10 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 	data = property(_get_data, _set_data)
 
 	#----------------------------------------------------------------
-	def show_msg(self, msg):
-		gmDispatcher.send(signal = 'statustext', msg = msg)
+	def _set_status_text(self, msg, beep=False):
+		gmDispatcher.send(signal = 'statustext_ea', msg = msg, beep = beep)
 
-	status_message = property(lambda x:x, show_msg)
+	StatusText = property(lambda x:x, _set_status_text)
 
 	#----------------------------------------------------------------
 	# generic edit area dialog API
@@ -224,7 +225,7 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 			return False
 
 		# remove messages about previous invalid save attempts
-		gmDispatcher.send(signal = 'statustext', msg = u'')
+		gmDispatcher.send(signal = 'statustext_ea', msg = '')
 
 		if self.__mode in ['new', 'new_from_existing']:
 			if self._save_as_new():
@@ -298,6 +299,12 @@ class cGenericEditAreaDlg2(wxgGenericEditAreaDlg2.wxgGenericEditAreaDlg2):
 		except KeyError:
 			single_entry = False
 
+		try:
+			title = kwargs['title']
+		except KeyError:
+			title = self.__class__.__name__
+		kwargs['title'] = decorate_window_title(title)
+
 		wxgGenericEditAreaDlg2.wxgGenericEditAreaDlg2.__init__(self, *args, **kwargs)
 
 		self.left_extra_button = None
@@ -314,8 +321,8 @@ class cGenericEditAreaDlg2(wxgGenericEditAreaDlg2.wxgGenericEditAreaDlg2):
 		dummy_ea_pnl = self._PNL_ea
 		ea_pnl_szr = dummy_ea_pnl.GetContainingSizer()
 		ea_pnl_parent = dummy_ea_pnl.GetParent()
-		ea_pnl_szr.Remove(dummy_ea_pnl)
-		dummy_ea_pnl.Destroy()
+		#ea_pnl_szr.Remove(dummy_ea_pnl)						# not available in wxp4 anymore, BUT
+		dummy_ea_pnl.DestroyLater()									# in wxp4 .DestroyLater() auto-Remove()s :-)
 		del dummy_ea_pnl
 		new_ea_min_size = new_ea.GetMinSize()
 		new_ea.Reparent(ea_pnl_parent)
@@ -333,26 +340,28 @@ class cGenericEditAreaDlg2(wxgGenericEditAreaDlg2.wxgGenericEditAreaDlg2):
 
 		# attach listener
 		self._TCTRL_status.SetValue('')
-		gmDispatcher.connect(signal = u'statustext', receiver = self._on_set_statustext)
+		gmDispatcher.connect(signal = 'statustext_ea', receiver = self._on_set_statustext)
 
 		# redraw layout
-		#self.Layout()
 		main_szr = self.GetSizer()
 		main_szr.Fit(self)
 		self.Layout()
 		#self.Refresh()				# apparently not needed (27.3.2011)
 
 		self._PNL_ea.refresh()
+
 	#--------------------------------------------------------
-	def _on_set_statustext(self, msg=None, loglevel=None, beep=True):
+	def _on_set_statustext(self, msg=None, loglevel=None, beep=False):
 		if msg is None:
-			self._TCTRL_status.SetValue('')
-			return
-		if msg.strip() == u'':
-			self._TCTRL_status.SetValue('')
-			return
-		self._TCTRL_status.SetValue(msg)
-		return
+			msg = ''
+		try:
+			self._TCTRL_status.SetValue(msg.strip())
+			if beep:
+				wx.Bell()
+		except RuntimeError:
+			# likely we are being deleted but a statustext arrived late via a signal
+			pass
+
 	#--------------------------------------------------------
 	def _adjust_clear_revert_buttons(self):
 		if self._PNL_ea.data is None:
@@ -365,25 +374,29 @@ class cGenericEditAreaDlg2(wxgGenericEditAreaDlg2.wxgGenericEditAreaDlg2):
 			self._BTN_clear.Hide()
 			self._BTN_revert.Enable(True)
 			self._BTN_revert.Show()
+
 	#--------------------------------------------------------
 	def _on_save_button_pressed(self, evt):
 		if self._PNL_ea.save():
-			gmDispatcher.disconnect(signal = u'statustext', receiver = self._on_set_statustext)
+			gmDispatcher.disconnect(signal = 'statustext_ea', receiver = self._on_set_statustext)
 			if self.IsModal():
 				self.EndModal(wx.ID_OK)
 			else:
 				self.Close()
+
 	#--------------------------------------------------------
 	def _on_revert_button_pressed(self, evt):
 		self._PNL_ea.refresh()
+
 	#--------------------------------------------------------
 	def _on_clear_button_pressed(self, evt):
 		self._PNL_ea.refresh()
+
 	#--------------------------------------------------------
 	def _on_forward_button_pressed(self, evt):
 		if self._PNL_ea.save():
 			if self._PNL_ea.successful_save_msg is not None:
-				gmDispatcher.send(signal = 'statustext', msg = self._PNL_ea.successful_save_msg)
+				gmDispatcher.send(signal = 'statustext_ea', msg = self._PNL_ea.successful_save_msg)
 			self._PNL_ea.mode = 'new_from_existing'
 
 			self._adjust_clear_revert_buttons()
@@ -414,6 +427,11 @@ class cGenericEditAreaDlg2(wxgGenericEditAreaDlg2.wxgGenericEditAreaDlg2):
 			self.EndModal(wx.ID_OK)
 		else:
 			self.Close()
+
+	#------------------------------------------------------------
+	def SetTitle(self, title):
+		super().SetTitle(decorate_window_title(title))
+
 	#------------------------------------------------------------
 	# properties
 	#------------------------------------------------------------
@@ -429,13 +447,24 @@ class cGenericEditAreaDlg2(wxgGenericEditAreaDlg2.wxgGenericEditAreaDlg2):
 			raise ValueError('<left extra button> callback is not a callable: %s' % callback)
 		self.__left_extra_button_callback = callback
 		self._BTN_extra_left.SetLabel(label)
-		self._BTN_extra_left.SetToolTipString(tooltip)
+		self._BTN_extra_left.SetToolTip(tooltip)
 		self._BTN_extra_left.Enable(True)
 		self._BTN_extra_left.Show()
 
 	left_extra_button = property(lambda x:x, _set_left_extra_button)
 
-
+#====================================================================
+#====================================================================
+#====================================================================
+#====================================================================
+#====================================================================
+#====================================================================
+#====================================================================
+#====================================================================
+#====================================================================
+#====================================================================
+#====================================================================
+#====================================================================
 #====================================================================
 #====================================================================
 #====================================================================
@@ -501,11 +530,11 @@ class cEditAreaPopup(wx.Dialog):
 		self.__editarea.Reparent(self)
 
 		self.__btn_SAVE = wx.Button(self, self.__wxID_BTN_SAVE, _("Save"))
-		self.__btn_SAVE.SetToolTipString(_('save entry into medical record'))
+		self.__btn_SAVE.SetToolTip(_('save entry into medical record'))
 		self.__btn_RESET = wx.Button(self, self.__wxID_BTN_RESET, _("Reset"))
-		self.__btn_RESET.SetToolTipString(_('reset entry'))
+		self.__btn_RESET.SetToolTip(_('reset entry'))
 		self.__btn_CANCEL = wx.Button(self, wx.ID_CANCEL, _("Cancel"))
-		self.__btn_CANCEL.SetToolTipString(_('discard entry and cancel'))
+		self.__btn_CANCEL.SetToolTip(_('discard entry and cancel'))
 
 		szr_buttons = wx.BoxSizer(wx.HORIZONTAL)
 		szr_buttons.Add(self.__btn_SAVE, 1, wx.EXPAND | wx.ALL, 1)
@@ -633,9 +662,9 @@ class cEditArea2(wx.Panel):
 		return 1
 	#--------------------------------------------------------
 	def __deregister_events(self):
-		gmDispatcher.disconnect(signal = u'pre_patient_unselection', receiver = self._on_pre_patient_unselection)
-		gmDispatcher.disconnect(signal = u'post_patient_selection', receiver = self.on_post_patient_selection)
-		gmDispatcher.disconnect(signal = u'application_closing', receiver = self._on_application_closing)
+		gmDispatcher.disconnect(signal = 'pre_patient_unselection', receiver = self._on_pre_patient_unselection)
+		gmDispatcher.disconnect(signal = 'post_patient_selection', receiver = self.on_post_patient_selection)
+		gmDispatcher.disconnect(signal = 'application_closing', receiver = self._on_application_closing)
 	#--------------------------------------------------------
 	# handlers
 	#--------------------------------------------------------
@@ -645,7 +674,7 @@ class cEditArea2(wx.Panel):
 	#--------------------------------------------------------
 	def _on_OK_btn_pressed(self, event):
 		"""Only active if _make_standard_buttons was called in child class."""
-		# FIXME: this try: except: block seems to large
+		# FIXME: this try: except Exception: block seems to large
 		try:
 			event.Skip()
 			if self.data is None:
@@ -658,7 +687,7 @@ class cEditArea2(wx.Panel):
 			# nasty evil popup dialogue box
 			# but for invalid input we want to interrupt user
 			gmGuiHelpers.gm_show_error (err, _("Invalid Input"))
-		except:
+		except Exception:
 			_log.exception( "save data problem in [%s]" % self.__class__.__name__)
 	#--------------------------------------------------------
 	def _on_clear_btn_pressed(self, event):
@@ -788,9 +817,9 @@ class cEditArea2(wx.Panel):
 	def _make_standard_buttons(self, parent):
 		"""Generates OK/CLEAR buttons for edit area."""
 		self.btn_OK = wx.Button(parent, self.__wxID_BTN_OK, _("OK"))
-		self.btn_OK.SetToolTipString(_('save entry into medical record'))
+		self.btn_OK.SetToolTip(_('save entry into medical record'))
 		self.btn_Clear = wx.Button(parent, self.__wxID_BTN_CLEAR, _("Clear"))
-		self.btn_Clear.SetToolTipString(_('initialize input fields for new entry'))
+		self.btn_Clear.SetToolTip(_('initialize input fields for new entry'))
 
 		szr_buttons = wx.BoxSizer(wx.HORIZONTAL)
 		szr_buttons.Add(self.btn_OK, 1, wx.EXPAND | wx.ALL, 1)
@@ -814,7 +843,7 @@ class cEditAreaField(wx.TextCtrl):
 class cEditArea(wx.Panel):
 	def __init__(self, parent, id, pos, size, style):
 
-		print "class [%s] is deprecated, use cEditArea2 instead" % self.__class__.__name__
+		print("class [%s] is deprecated, use cEditArea2 instead" % self.__class__.__name__)
 
 		# init main background panel
 		wx.Panel.__init__(self, parent, id, pos=pos, size=size, style=wx.NO_BORDER | wx.TAB_TRAVERSAL)
@@ -1005,9 +1034,9 @@ class cEditArea(wx.Panel):
 	def _make_standard_buttons(self, parent):
 		"""Generates OK/CLEAR buttons for edit area."""
 		self.btn_OK = wx.Button(parent, ID_BTN_OK, _("OK"))
-		self.btn_OK.SetToolTipString(_('save entry into medical record'))
+		self.btn_OK.SetToolTip(_('save entry into medical record'))
 		self.btn_Clear = wx.Button(parent, ID_BTN_CLEAR, _("Clear"))
-		self.btn_Clear.SetToolTipString(_('initialize input fields for new entry'))
+		self.btn_Clear.SetToolTip(_('initialize input fields for new entry'))
 
 		szr_buttons = wx.BoxSizer(wx.HORIZONTAL)
 		szr_buttons.Add(self.btn_OK, 1, wx.EXPAND | wx.ALL, 1)
@@ -1034,16 +1063,16 @@ class cEditArea(wx.Panel):
 		wx.EVT_SIZE (self.fields_pnl, self._on_resize_fields)
 
 		# client internal signals
-		gmDispatcher.connect(signal = u'pre_patient_unselection', receiver = self._on_pre_patient_unselection)
-		gmDispatcher.connect(signal = u'application_closing', receiver = self._on_application_closing)
-		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self.on_post_patient_selection)
+		gmDispatcher.connect(signal = 'pre_patient_unselection', receiver = self._on_pre_patient_unselection)
+		gmDispatcher.connect(signal = 'application_closing', receiver = self._on_application_closing)
+		gmDispatcher.connect(signal = 'post_patient_selection', receiver = self.on_post_patient_selection)
 
 		return 1
 	#--------------------------------------------------------
 	# handlers
 	#--------------------------------------------------------
 	def _on_OK_btn_pressed(self, event):
-		# FIXME: this try: except: block seems to large
+		# FIXME: this try: except Exception: block seems to large
 		try:
 			event.Skip()
 			if self.data is None:
@@ -1056,7 +1085,7 @@ class cEditArea(wx.Panel):
 			# nasty evil popup dialogue box
 			# but for invalid input we want to interrupt user
 			gmGuiHelpers.gm_show_error (err, _("Invalid Input"))
-		except:
+		except Exception:
 			_log.exception( "save data  problem in [%s]" % self.__class__.__name__)
 	#--------------------------------------------------------
 	def _on_clear_btn_pressed(self, event):
@@ -1098,12 +1127,12 @@ class cEditArea(wx.Panel):
 class gmEditArea(cEditArea):
 	def __init__(self, parent, id, aType = None):
 
-		print "class [%s] is deprecated, use cEditArea2 instead" % self.__class__.__name__
+		print("class [%s] is deprecated, use cEditArea2 instead" % self.__class__.__name__)
 
 		# sanity checks
 		if aType not in _known_edit_area_types:
 			_log.error('unknown edit area type: [%s]' % aType)
-			raise gmExceptions.ConstructorError, 'unknown edit area type: [%s]' % aType
+			raise gmExceptions.ConstructorError('unknown edit area type: [%s]' % aType)
 		self._type = aType
 
 		# init main background panel
@@ -1264,7 +1293,7 @@ class gmEditArea(cEditArea):
 	def _makeExtraColumns(self , parent, lines, weightMap = {} ):
 		"""this is a utlity method to add extra columns"""
 		#add an extra column if the class has attribute "extraColumns"
-		if u"extraColumns" in self.__class__.__dict__:
+		if "extraColumns" in self.__class__.__dict__:
 			for x in self.__class__.extraColumns:
 				lines = self._addColumn(parent, lines, x, weightMap)
 		return lines
@@ -1315,13 +1344,13 @@ class gmEditArea(cEditArea):
 				continue
 			try:
 				field.SetValue( str(v) )
-			except:
+			except Exception:
 				try:
 					if type(v) == type(''):
 						v = 0
 
 					field.SetValue( v)
-				except:
+				except Exception:
 					pass
 		self.setDataId(id)
 		#self.monitoring_dirty = 1
@@ -1346,7 +1375,7 @@ class gmEditArea(cEditArea):
 		for f in fields:
 			try:
 				values[f] = self.input_fields[f].GetValue()
-			except:
+			except Exception:
 				pass
 		return values
 #====================================================================
@@ -1491,13 +1520,13 @@ class gmPastHistoryEditArea(gmEditArea):
 		try :
 			year = self._getBirthYear() + int(self.fld_age_noted.GetValue().strip() )
 			self.fld_date_noted.SetValue( str (year) )
-		except:
+		except Exception:
 			pass
 
 	def _getBirthYear(self):
 		try:
 			birthyear = int(str(self._patient['dob']).split('-')[0]) 
-		except:
+		except Exception:
 #			birthyear = time.localtime()[0]
 			birthyear = 1
 
@@ -1508,7 +1537,7 @@ class gmPastHistoryEditArea(gmEditArea):
 		try:
 			age = int(self.fld_date_noted.GetValue().strip() ) - self._getBirthYear() 
 			self.fld_age_noted.SetValue( str (age) )
-		except:
+		except Exception:
 			pass
 
 	__init_values = {
@@ -1532,7 +1561,7 @@ class gmPastHistoryEditArea(gmEditArea):
 		try:
 			#return	time.localtime()[0] - self._patient.getBirthYear()
 			return 1
-		except:
+		except Exception:
 			return 0
 
 	def _get_init_values(self):
@@ -1690,12 +1719,7 @@ class gmReferralEditArea(gmEditArea):
 #====================================================================
 class gmPrescriptionEditArea(gmEditArea):
 	def __init__(self, parent, id):
-		try:
-			gmEditArea.__init__(self, parent, id, aType = 'prescription')
-		except gmExceptions.ConstructorError:
-			_log.exceptions('cannot instantiate prescription edit area')
-			raise
-
+		gmEditArea.__init__(self, parent, id, aType = 'prescription')
 
 	#----------------------------------------------------------------
 	def _make_edit_lines(self, parent):
@@ -1709,7 +1733,7 @@ class gmPrescriptionEditArea(gmEditArea):
 		self.txt_directions= cEditAreaField(parent)
 		self.txt_for = cEditAreaField(parent)
 		self.txt_progress = cEditAreaField(parent)
-		
+
 		lines.append(self.txt_problem)
 		lines.append(self.txt_class)
 		lines.append(self.txt_generic)
@@ -1781,7 +1805,7 @@ class gmPnlEditAreaPrompts(wx.Panel):
 		self.SetAutoLayout(True)
 #====================================================================
 #Class central to gnumed data input
-#allows data entry of multiple different types.e.g scripts,
+#allows data entry of multiple different types, e.g scripts,
 #referrals, measurements, recalls etc
 #@TODO : just about everything
 #section = calling section eg allergies, script
