@@ -3,7 +3,13 @@
 __author__  = "K. Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL v2 or later (details at http://www.gnu.org)"
 
-import logging, exceptions, traceback, re as regex, sys, os, shutil, datetime as pyDT
+import logging
+import traceback
+import sys
+import os
+import shutil
+import datetime as pyDT
+import re as regex
 
 
 import wx
@@ -58,13 +64,14 @@ def set_is_public_database(value):
 #-------------------------------------------------------------------------
 def __ignore_dead_objects_from_async(t, v, tb):
 
-	if t != wx._core.PyDeadObjectError:
+	if t != RuntimeError:
 		return False
 
 	wx.EndBusyCursor()
 
 	# try to ignore those, they come about from doing
 	# async work in wx as Robin tells us
+	_log.error('RuntimeError = dead object: %s', v)
 	_log.warning('continuing and hoping for the best')
 	return True
 
@@ -75,7 +82,7 @@ def __handle_exceptions_on_shutdown(t, v, tb):
 		return False
 
 	# dead object error ?
-	if t == wx._core.PyDeadObjectError:
+	if t == RuntimeError:
 		return True
 
 	gmLog2.log_stack_trace('exception on shutdown', t, v, tb)
@@ -84,13 +91,13 @@ def __handle_exceptions_on_shutdown(t, v, tb):
 #-------------------------------------------------------------------------
 def __handle_import_error(t, v, tb):
 
-	if t == exceptions.OSError:
+	if t == OSError:
 		if not hasattr(t, 'winerror'):
 			return False
 		if getattr(t, 'winerror') != 126:
 			return False
 	else:
-		if t != exceptions.ImportError:
+		if t != ImportError:
 			return False
 
 	wx.EndBusyCursor()
@@ -118,7 +125,7 @@ def __handle_ctrl_c(t, v, tb):
 	if t != KeyboardInterrupt:
 		return False
 
-	print "<Ctrl-C>: Shutting down ..."
+	print("<Ctrl-C>: Shutting down ...")
 	top_win = wx.GetApp().GetTopWindow()
 	wx.CallAfter(top_win.Close)
 	return True
@@ -132,7 +139,7 @@ def __handle_access_violation(t, v, tb):
 	_log.error('access permissions violation detected')
 	wx.EndBusyCursor()
 	gmLog2.flush()
-	txt = u' ' + v.errmsg
+	txt = ' ' + v.errmsg
 	if v.source is not None:
 		txt += _('\n Source: %s') % v.source
 	if v.code is not None:
@@ -156,44 +163,10 @@ def __handle_access_violation(t, v, tb):
 #-------------------------------------------------------------------------
 def __handle_lost_db_connection(t, v, tb):
 
-	if t not in [gmPG2.dbapi.OperationalError, gmPG2.dbapi.InterfaceError]:
-		return False
-
-	try:
-		msg = gmPG2.extract_msg_from_pg_exception(exc = v)
-	except:
-		msg = u'cannot extract message from PostgreSQL exception'
-		print msg
-		print v
-		return False
-
-	conn_lost = False
-
-	if t == gmPG2.dbapi.OperationalError:
-		conn_lost = (
-			('erver' in msg)
-				and
-			(
-				('term' in msg)
-					or
-				('abnorm' in msg)
-					or
-				('end' in msg)
-					or
-				('oute' in msg)
-			)
-		)
-
-	if t == gmPG2.dbapi.InterfaceError:
-		conn_lost = (
-			('onnect' in msg)
-				and
-			(('close' in msg) or ('end' in msg))
-		)
-
-	if not conn_lost:
+	if not gmPG2.exception_is_connection_loss(v):
 		return False
 
+	gmPG2.log_pg_exception_details(v)
 	gmLog2.log_stack_trace('lost connection', t, v, tb)
 	wx.EndBusyCursor()
 	gmLog2.flush()
@@ -213,7 +186,7 @@ def __handle_lost_db_connection(t, v, tb):
 
 #-------------------------------------------------------------------------
 def __handle_wxgtk_assertion(t, v, tb):
-	if t != wx.PyAssertionError:
+	if t != wx.wxAssertionError:
 		return False
 	_log.exception('a wxGTK assertion failed:')
 	_log.warning('continuing and hoping for the best')
@@ -270,12 +243,12 @@ def handle_uncaught_exception_wx(t, v, tb):
 		'%s_%s%s' % (name, pyDT.datetime.now().strftime('%Y-%m-%d_%H-%M-%S'), ext)
 	))
 
-	dlg = cUnhandledExceptionDlg(parent = None, id = -1, exception = (t, v, tb), logfile = new_name)
+	dlg = cUnhandledExceptionDlg(None, -1, exception = (t, v, tb), logfile = new_name)
 	dlg.ShowModal()
 	comment = dlg._TCTRL_comment.GetValue()
-	dlg.Destroy()
-	if (comment is not None) and (comment.strip() != u''):
-		_log.error(u'user comment: %s', comment.strip())
+	dlg.DestroyLater()
+	if (comment is not None) and (comment.strip() != ''):
+		_log.error('user comment: %s', comment.strip())
 
 	_log.warning('syncing log file for backup to [%s]', new_name)
 	gmLog2.flush()
@@ -299,6 +272,10 @@ def install_wx_exception_handler():
 
 	gmDispatcher.connect(signal = 'application_closing', receiver = _on_application_closing)
 
+	global APP_PID
+	APP_PID = os.getpid()
+	_log.debug('registered PID [%s] for aborting if necessary', APP_PID)
+
 	global _prev_excepthook
 	_prev_excepthook = sys.excepthook
 	sys.excepthook = handle_uncaught_exception_wx
@@ -323,105 +300,105 @@ def _on_application_closing():
 # ========================================================================
 def mail_log(parent=None, comment=None, helpdesk=None, sender=None):
 
-		if (comment is None) or (comment.strip() == u''):
-			comment = wx.GetTextFromUser (
-				message = _(
-					'Please enter a short note on what you\n'
-					'were about to do in GNUmed:'
-				),
-				caption = _('Sending bug report'),
-				parent = parent
-			)
-			if comment.strip() == u'':
-				comment = u'<user did not comment on bug report>'
-
-		receivers = []
-		if helpdesk is not None:
-			receivers = regex.findall (
-				'[\S]+@[\S]+',
-				helpdesk.strip(),
-				flags = regex.UNICODE
-			)
-		if len(receivers) == 0:
-			if _is_public_database:
-				receivers = [u'gnumed-bugs@gnu.org']
-
-		receiver_string = wx.GetTextFromUser (
+	if (comment is None) or (comment.strip() == ''):
+		comment = wx.GetTextFromUser (
 			message = _(
-				'Edit the list of email addresses to send the\n'
-				'bug report to (separate addresses by spaces).\n'
-				'\n'
-				'Note that <gnumed-bugs@gnu.org> refers to\n'
-				'the public (!) GNUmed bugs mailing list.'
+				'Please enter a short note on what you\n'
+				'were about to do in GNUmed:'
 			),
 			caption = _('Sending bug report'),
-			default_value = ','.join(receivers),
 			parent = parent
 		)
-		if receiver_string.strip() == u'':
-			return
+		if comment.strip() == '':
+			comment = '<user did not comment on bug report>'
 
+	receivers = []
+	if helpdesk is not None:
 		receivers = regex.findall (
 			'[\S]+@[\S]+',
-			receiver_string,
+			helpdesk.strip(),
 			flags = regex.UNICODE
 		)
+	if len(receivers) == 0:
+		if _is_public_database:
+			receivers = ['gnumed-bugs@gnu.org']
 
-		dlg = gmGuiHelpers.c2ButtonQuestionDlg (
-			parent,
-			-1,
-			caption = _('Sending bug report'),
-			question = _(
-				'Your bug report will be sent to:\n'
-				'\n'
-				'%s\n'
-				'\n'
-				'Make sure you have reviewed the log file for potentially\n'
-				'sensitive information before sending out the bug report.\n'
-				'\n'
-				'Note that emailing the report may take a while depending\n'
-				'on the speed of your internet connection.\n'
-			) % u'\n'.join(receivers),
-			button_defs = [
-				{'label': _('Send report'), 'tooltip': _('Yes, send the bug report.')},
-				{'label': _('Cancel'), 'tooltip': _('No, do not send the bug report.')}
-			],
-			show_checkbox = True,
-			checkbox_msg = _('include log file in bug report')
-		)
-		dlg._CHBOX_dont_ask_again.SetValue(_is_public_database)
-		go_ahead = dlg.ShowModal()
-		if go_ahead == wx.ID_NO:
-			dlg.Destroy()
-			return
+	receiver_string = wx.GetTextFromUser (
+		message = _(
+			'Edit the list of email addresses to send the\n'
+			'bug report to (separate addresses by spaces).\n'
+			'\n'
+			'Note that <gnumed-bugs@gnu.org> refers to\n'
+			'the public (!) GNUmed bugs mailing list.'
+		),
+		caption = _('Sending bug report'),
+		default_value = ','.join(receivers),
+		parent = parent
+	)
+	if receiver_string.strip() == '':
+		return
 
-		include_log = dlg._CHBOX_dont_ask_again.GetValue()
-		if not _is_public_database:
-			if include_log:
-				result = gmGuiHelpers.gm_show_question (
-					_(
-						'The database you are connected to is marked as\n'
-						'"in-production with controlled access".\n'
-						'\n'
-						'You indicated that you want to include the log\n'
-						'file in your bug report. While this is often\n'
-						'useful for debugging the log file might contain\n'
-						'bits of patient data which must not be sent out\n'
-						'without de-identification.\n'
-						'\n'
-						'Please confirm that you want to include the log !'
-					),
-					_('Sending bug report')
-				)
-				include_log = (result is True)
+	receivers = regex.findall (
+		'[\S]+@[\S]+',
+		receiver_string,
+		flags = regex.UNICODE
+	)
 
-		if sender is None:
+	dlg = gmGuiHelpers.c2ButtonQuestionDlg (
+		parent,
+		-1,
+		caption = _('Sending bug report'),
+		question = _(
+			'Your bug report will be sent to:\n'
+			'\n'
+			'%s\n'
+			'\n'
+			'Make sure you have reviewed the log file for potentially\n'
+			'sensitive information before sending out the bug report.\n'
+			'\n'
+			'Note that emailing the report may take a while depending\n'
+			'on the speed of your internet connection.\n'
+		) % '\n'.join(receivers),
+		button_defs = [
+			{'label': _('Send report'), 'tooltip': _('Yes, send the bug report.')},
+			{'label': _('Cancel'), 'tooltip': _('No, do not send the bug report.')}
+		],
+		show_checkbox = True,
+		checkbox_msg = _('include log file in bug report')
+	)
+	dlg._CHBOX_dont_ask_again.SetValue(_is_public_database)
+	go_ahead = dlg.ShowModal()
+	if go_ahead == wx.ID_NO:
+		dlg.DestroyLater()
+		return
+
+	include_log = dlg._CHBOX_dont_ask_again.GetValue()
+	if not _is_public_database:
+		if include_log:
+			result = gmGuiHelpers.gm_show_question (
+				_(
+					'The database you are connected to is marked as\n'
+					'"in-production with controlled access".\n'
+					'\n'
+					'You indicated that you want to include the log\n'
+					'file in your bug report. While this is often\n'
+					'useful for debugging the log file might contain\n'
+					'bits of patient data which must not be sent out\n'
+					'without de-identification.\n'
+					'\n'
+					'Please confirm that you want to include the log !'
+				),
+				_('Sending bug report')
+			)
+			include_log = (result is True)
+
+	if sender is None:
+		sender = _('<not supplied>')
+	else:
+		if sender.strip() == '':
 			sender = _('<not supplied>')
-		else:
-			if sender.strip() == u'':
-				sender = _('<not supplied>')
 
-		msg = u"""\
+	msg = """\
 Report sent via GNUmed's handler for unexpected exceptions.
 
 user comment  : %s
@@ -438,33 +415,34 @@ sender email  : %s
  importance medium
 
 """ % (comment, _client_version, _local_account, _staff_name, sender)
-		if include_log:
-			_log.error(comment)
-			_log.warning('syncing log file for emailing')
-			gmLog2.flush()
-			attachments = [ [_logfile_name, 'text/plain', 'quoted-printable'] ]
-		else:
-			attachments = None
+	if include_log:
+		_log.error(comment)
+		_log.warning('syncing log file for emailing')
+		gmLog2.flush()
+		attachments = [ [_logfile_name, 'text/plain', 'quoted-printable'] ]
+	else:
+		attachments = None
 
-		dlg.Destroy()
+	dlg.DestroyLater()
 
-		wx.BeginBusyCursor()
-		_cfg = gmCfg2.gmCfgData()
-		try:
-			gmNetworkTools.compose_and_send_email (
-				sender = '%s <%s>' % (_staff_name, gmNetworkTools.default_mail_sender),
-				receiver = receivers,
-				subject = u'<bug>: %s' % comment,
-				message = msg,
-				server = gmNetworkTools.default_mail_server,
-				auth = {'user': gmNetworkTools.default_mail_sender, 'password': u'gnumed-at-gmx-net'},
-				debug = _cfg.get(option = 'debug'),
-				attachments = attachments
-			)
-			gmDispatcher.send(signal='statustext', msg = _('Bug report has been emailed.'))
-		except:
-			_log.exception('cannot send bug report')
-			gmDispatcher.send(signal='statustext', msg = _('Bug report COULD NOT be emailed.'))
+	wx.BeginBusyCursor()
+	_cfg = gmCfg2.gmCfgData()
+	try:
+		gmNetworkTools.compose_and_send_email (
+			sender = '%s <%s>' % (_staff_name, gmNetworkTools.default_mail_sender),
+			receiver = receivers,
+			subject = '<bug>: %s' % comment,
+			message = msg,
+			server = gmNetworkTools.default_mail_server,
+			auth = {'user': gmNetworkTools.default_mail_sender, 'password': 'gnumed-at-gmx-net'},
+			debug = _cfg.get(option = 'debug'),
+			attachments = attachments
+		)
+		gmDispatcher.send(signal='statustext', msg = _('Bug report has been emailed.'))
+	except Exception:
+		_log.exception('cannot send bug report')
+		gmDispatcher.send(signal='statustext', msg = _('Bug report COULD NOT be emailed.'))
+	finally:
 		wx.EndBusyCursor()
 
 # ========================================================================
@@ -481,21 +459,26 @@ class cUnhandledExceptionDlg(wxgUnhandledExceptionDlg.wxgUnhandledExceptionDlg):
 
 		wxgUnhandledExceptionDlg.wxgUnhandledExceptionDlg.__init__(self, *args, **kwargs)
 
+		self.Title = '%s [PID %s]' % (self.Title, APP_PID)
+
 		if _sender_email is not None:
 			self._TCTRL_sender.SetValue(_sender_email)
 		self._TCTRL_helpdesk.SetValue(_helpdesk)
 		self._TCTRL_logfile.SetValue(self.logfile)
 		t, v, tb = exception
-		self._TCTRL_exc_type.SetValue(str(t))
-		self._TCTRL_exc_value.SetValue(str(v))
-		self._TCTRL_traceback.SetValue(''.join(traceback.format_tb(tb)))
-
+		self._TCTRL_traceback.SetValue ('%s: %s\n%s: %s\n%s' % (
+			'type', t,
+			'value', v,
+			''.join(traceback.format_tb(tb))
+		))
 		self.Fit()
+
 	#------------------------------------------
 	def _on_close_gnumed_button_pressed(self, evt):
+		evt.Skip()
 		comment = self._TCTRL_comment.GetValue()
-		if (comment is not None) and (comment.strip() != u''):
-			_log.error(u'user comment: %s', comment.strip())
+		if (comment is not None) and (comment.strip() != ''):
+			_log.error('user comment: %s', comment.strip())
 		_log.warning('syncing log file for backup to [%s]', self.logfile)
 		gmLog2.flush()
 		try:
@@ -504,10 +487,22 @@ class cUnhandledExceptionDlg(wxgUnhandledExceptionDlg.wxgUnhandledExceptionDlg):
 			_log.error('cannot backup log file')
 		top_win = wx.GetApp().GetTopWindow()
 		wx.CallAfter(top_win.Close)
-		evt.Skip()
+
+	#------------------------------------------
+	def _on_abort_gnumed_button_pressed(self, evt):
+		print('running os._exit(-999)')
+		os._exit(-999)
+		print('running os.kill(9) on current process')
+		os.kill(APP_PID, 9)
+		print('running sys.exit()')
+		sys.exit(-999)
+		# ideas:
+		# - start timer thread with abort code from [close] button
+		# - start a detached shell script at the OS level with "sleep 20 ; kill -15 PID ; kill -9 PID"
+
 	#------------------------------------------
 	def _on_mail_button_pressed(self, evt):
-
+		evt.Skip()
 		mail_log (
 			parent = self,
 			comment = self._TCTRL_comment.GetValue().strip(),
@@ -515,11 +510,11 @@ class cUnhandledExceptionDlg(wxgUnhandledExceptionDlg.wxgUnhandledExceptionDlg):
 			sender = self._TCTRL_sender.GetValue().strip()
 		)
 
-		evt.Skip()
 	#------------------------------------------
 	def _on_view_log_button_pressed(self, evt):
+		evt.Skip()
 		from Gnumed.pycommon import gmMimeLib
 		gmLog2.flush()
 		gmMimeLib.call_viewer_on_file(_logfile_name, block = False)
-		evt.Skip()
+
 # ========================================================================
