@@ -66,7 +66,7 @@ def configure_visual_progress_note_editor():
 			)
 			return False, value
 
-		if value.strip() == u'':
+		if value.strip() == '':
 			gmDispatcher.send (
 				signal = 'statustext',
 				msg = _('You need to actually set an editor.'),
@@ -95,7 +95,7 @@ def configure_visual_progress_note_editor():
 			'will be replaced by the file name of the\n'
 			'note template.'
 		),
-		option = u'external.tools.visual_soap_editor_cmd',
+		option = 'external.tools.visual_soap_editor_cmd',
 		bias = 'user',
 		default_value = None,
 		validator = is_valid
@@ -114,17 +114,17 @@ def select_file_as_visual_progress_note_template(parent=None):
 		defaultDir = os.path.expanduser('~'),
 		defaultFile = '',
 		#wildcard = "%s (*)|*|%s (*.*)|*.*" % (_('all files'), _('all files (Win)')),
-		style = wx.OPEN | wx.FILE_MUST_EXIST
+		style = wx.FD_OPEN | wx.FD_FILE_MUST_EXIST
 	)
 	result = dlg.ShowModal()
 
 	if result == wx.ID_CANCEL:
-		dlg.Destroy()
+		dlg.DestroyLater()
 		return None
 
 	full_filename = dlg.GetPath()
 	dlg.Hide()
-	dlg.Destroy()
+	dlg.DestroyLater()
 	return full_filename
 
 #------------------------------------------------------------
@@ -145,7 +145,7 @@ def select_visual_progress_note_template(parent=None):
 		]
 	)
 	result = dlg.ShowModal()
-	dlg.Destroy()
+	dlg.DestroyLater()
 
 	# 1) select from template
 	if result == wx.ID_YES:
@@ -160,7 +160,7 @@ def select_visual_progress_note_template(parent=None):
 			return (None, None)
 		filename = template.save_to_file()
 		if filename is None:
-			gmDispatcher.send(signal = u'statustext', msg = _('Cannot export visual progress note template for [%s].') % template['name_long'])
+			gmDispatcher.send(signal = 'statustext', msg = _('Cannot export visual progress note template for [%s].') % template['name_long'])
 			return (None, None)
 		return (filename, True)
 
@@ -197,19 +197,19 @@ def edit_visual_progress_note(filename=None, episode=None, discard_unmodified=Fa
 	if doc_part is not None:
 		filename = doc_part.save_to_file()
 		if filename is None:
-			gmDispatcher.send(signal = u'statustext', msg = _('Cannot export visual progress note to file.'))
+			gmDispatcher.send(signal = 'statustext', msg = _('Cannot export visual progress note to file.'))
 			return None
 
 	dbcfg = gmCfg.cCfgSQL()
 	editor = dbcfg.get2 (
-		option = u'external.tools.visual_soap_editor_cmd',
+		option = 'external.tools.visual_soap_editor_cmd',
 		workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 		bias = 'user'
 	)
 
 	if editor is None:
-		_log.error(u'no editor for visual progress notes configured, trying mimetype editor')
-		gmDispatcher.send(signal = u'statustext', msg = _('Editor for visual progress note not configured.'), beep = False)
+		_log.error('no editor for visual progress notes configured, trying mimetype editor')
+		gmDispatcher.send(signal = 'statustext', msg = _('Editor for visual progress note not configured.'), beep = False)
 		mimetype = gmMimeLib.guess_mimetype(filename = filename)
 		editor = gmMimeLib.get_editor_cmd(mimetype = mimetype, filename = filename)
 		if editor is None:
@@ -218,27 +218,27 @@ def edit_visual_progress_note(filename=None, episode=None, discard_unmodified=Fa
 			if not success:
 				_log.debug('problem running mimetype <%s> viewer', mimetype)
 				gmGuiHelpers.gm_show_error (
-					_(	u'There is no editor for visual progress notes defined.\n'
-						u'Also, there is no editor command defined for the file type\n'
-						u'\n'
-						u' [%s].\n'
-						u'\n'
-						u'Therefor GNUmed attempted to at least *show* this\n'
-						u'visual progress note. That failed as well, however:\n'
-						u'\n'
-						u'%s'
+					_(	'There is no editor for visual progress notes defined.\n'
+						'Also, there is no editor command defined for the file type\n'
+						'\n'
+						' [%s].\n'
+						'\n'
+						'Therefor GNUmed attempted to at least *show* this\n'
+						'visual progress note. That failed as well, however:\n'
+						'\n'
+						'%s'
 					) % (mimetype, msg),
 					_('Editing visual progress note')
 				)
 				editor = configure_visual_progress_note_editor()
 				if editor is None:
-					gmDispatcher.send(signal = u'statustext', msg = _('Editor for visual progress note not configured.'), beep = True)
+					gmDispatcher.send(signal = 'statustext', msg = _('Editor for visual progress note not configured.'), beep = True)
 					return None
 
-	if u'%(img)s' in editor:
-		editor = editor % {u'img': filename}
+	if '%(img)s' in editor:
+		editor = editor % {'img': filename}
 	else:
-		editor = u'%s %s' % (editor, filename)
+		editor = '%s %s' % (editor, filename)
 
 	if discard_unmodified:
 		original_stat = os.stat(filename)
@@ -250,22 +250,22 @@ def edit_visual_progress_note(filename=None, episode=None, discard_unmodified=Fa
 		if not success:
 			_log.debug('problem running mimetype <%s> viewer', mimetype)
 			gmGuiHelpers.gm_show_error (
-				_(	u'There was a problem running the editor\n'
-					u'\n'
-					u' [%s] (%s)\n'
-					u'\n'
-					u'on the visual progress note.\n'
-					u'\n'
-					u'Therefor GNUmed attempted to at least *show* it.\n'
-					u'That failed as well, however:\n'
-					u'\n'
-					u'%s'
+				_(	'There was a problem running the editor\n'
+					'\n'
+					' [%s] (%s)\n'
+					'\n'
+					'on the visual progress note.\n'
+					'\n'
+					'Therefor GNUmed attempted to at least *show* it.\n'
+					'That failed as well, however:\n'
+					'\n'
+					'%s'
 				) % (editor, mimetype, msg),
 				_('Editing visual progress note')
 			)
 			editor = configure_visual_progress_note_editor()
 			if editor is None:
-				gmDispatcher.send(signal = u'statustext', msg = _('Editor for visual progress note not configured.'), beep = True)
+				gmDispatcher.send(signal = 'statustext', msg = _('Editor for visual progress note not configured.'), beep = True)
 		return None
 
 	try:
@@ -293,14 +293,14 @@ def edit_visual_progress_note(filename=None, episode=None, discard_unmodified=Fa
 				_log.debug('visual progress note (template) not modified')
 				# ask user to decide
 				msg = _(
-					u'You either created a visual progress note from a template\n'
-					u'in the database (rather than from a file on disk) or you\n'
-					u'edited an existing visual progress note.\n'
-					u'\n'
-					u'The template/original was not modified at all, however.\n'
-					u'\n'
-					u'Do you still want to save the unmodified image as a\n'
-					u'visual progress note into the EMR of the patient ?\n'
+					'You either created a visual progress note from a template\n'
+					'in the database (rather than from a file on disk) or you\n'
+					'edited an existing visual progress note.\n'
+					'\n'
+					'The template/original was not modified at all, however.\n'
+					'\n'
+					'Do you still want to save the unmodified image as a\n'
+					'visual progress note into the EMR of the patient ?\n'
 				)
 				save_unmodified = gmGuiHelpers.gm_show_question (
 					msg,
@@ -342,7 +342,7 @@ class cVisualSoapTemplatePhraseWheel(gmPhraseWheel.cPhraseWheel):
 
 		gmPhraseWheel.cPhraseWheel.__init__ (self, *args, **kwargs)
 
-		query = u"""
+		query = """
 SELECT
 	pk AS data,
 	name_short AS list_label,
@@ -385,7 +385,10 @@ class cVisualSoapPresenterPnl(wxgVisualSoapPresenterPnl.wxgVisualSoapPresenterPn
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
-	def refresh(self, document_folder=None, episodes=None, encounter=None, async=False):
+	def refresh(self, document_folder=None, episodes=None, encounter=None, do_async=False):
+		if not self:
+			# our C/C++ part may be dead already, due to async behaviour
+			return
 
 		if document_folder is None:
 			self.clear()
@@ -398,12 +401,12 @@ class cVisualSoapPresenterPnl(wxgVisualSoapPresenterPnl.wxgVisualSoapPresenterPn
 			self.GetParent().Layout()
 			return
 
-		if not async:
+		if not do_async:
 			cookie, parts_list = self._worker__export_doc_parts(docs = soap_docs)
 			self.__show_exported_parts(parts_list = parts_list)
 			return
 
-		self.__worker_cookie = u'%sCookie-%s' % (self.__class__.__name__, random.random())
+		self.__worker_cookie = '%sCookie-%s' % (self.__class__.__name__, random.random())
 		_log.debug('starting worker thread, cookie: %s', self.__worker_cookie)
 		gmWorkerThread.execute_in_worker_thread (
 			payload_function = self._worker__export_doc_parts,
@@ -414,11 +417,12 @@ class cVisualSoapPresenterPnl(wxgVisualSoapPresenterPnl.wxgVisualSoapPresenterPn
 
 	#--------------------------------------------------------
 	def clear(self):
-		while len(self._SZR_soap.GetChildren()) > 0:
-			self._SZR_soap.Detach(0)
+		if self._SZR_soap:
+			while len(self._SZR_soap.GetChildren()) > 0:
+				self._SZR_soap.Detach(0)
 		for bmp in self.__bitmaps:
 			bmp.Unbind(wx.EVT_LEFT_UP)
-			bmp.Destroy()
+			bmp.DestroyLater()
 		self.__bitmaps = []
 
 	#--------------------------------------------------------
@@ -437,21 +441,21 @@ class cVisualSoapPresenterPnl(wxgVisualSoapPresenterPnl.wxgVisualSoapPresenterPn
 	def _worker__export_doc_parts(self, docs=None, cookie=None):
 		# this is used as the worker thread payload
 		_log.debug('cookie [%s]', cookie)
-		conn = gmPG2.get_raw_connection(readonly = True, connection_name = threading.current_thread().name)
+		conn = gmPG2.get_connection(readonly = True, connection_name = threading.current_thread().name, pooled = False)
 		parts_list = []
 		for soap_doc in docs:
 			parts = soap_doc.parts
 			if len(parts) == 0:
 				continue
-			parts_counter = u''
+			parts_counter = ''
 			if len(parts) > 1:
-				parts_counter = _(u' [part 1 of %s]') % len(parts)
+				parts_counter = _(' [part 1 of %s]') % len(parts)
 			part = parts[0]
 			fname = part.save_to_file(conn = conn)
 			if fname is None:
 				continue
-			tt_header = _(u'Created: %s%s') % (gmDateTime.pydt_strftime(part['date_generated'], '%Y %b %d'), parts_counter)
-			tt_footer = gmTools.coalesce(part['doc_comment'], u'').strip()
+			tt_header = _('Created: %s%s') % (gmDateTime.pydt_strftime(part['date_generated'], '%Y %b %d'), parts_counter)
+			tt_footer = gmTools.coalesce(part['doc_comment'], '').strip()
 			parts_list.append([fname, part, tt_header, tt_footer])
 		conn.close()
 		_log.debug('worker finished')
@@ -459,7 +463,6 @@ class cVisualSoapPresenterPnl(wxgVisualSoapPresenterPnl.wxgVisualSoapPresenterPn
 
 	#--------------------------------------------------------
 	def _forwarder__show_exported_doc_parts(self, worker_result):
-		gmPG2.discard_pooled_connection()
 		# this is the worker thread completion callback
 		cookie, parts_list = worker_result
 		# worker still the one we are interested in ?
@@ -488,7 +491,7 @@ class cVisualSoapPresenterPnl(wxgVisualSoapPresenterPnl.wxgVisualSoapPresenterPn
 				height = 150
 			)
 			tip = agw_stt.SuperToolTip (
-				u'',
+				'',
 				bodyImage = img,
 				header = tt_header,
 				footer = tt_footer
