@@ -51,6 +51,6 @@ class gmCurrentSubstancesPlugin(gmPlugin.cNotebookPlugin):
 # main
 #----------------------------------------------------------------------
 if __name__ == "__main__":
-	print "no test code"
+	print("no test code")
 
 #======================================================================
